/**
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 **/
package org.objectweb.orchestra.demo.telecom;

import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;


/**
 * TelecomSessionBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: TelecomSessionBean.java,v 1.1 2006/10/04 15:33:29 lejeuneg Exp $
 *
 * @ejb:bean name="TelecomSessionBean"
 *           display-name="TelecomSessionBean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="org/objectweb/orchestra/demo/telecom/TelecomSessionHome"
 *           view-type="remote"
 *
 * @ejb.home remote-class="org.objectweb.orchestra.demo.telecom.TelecomSessionHome"
 * @ejb.interface remote-class="org.objectweb.orchestra.demo.telecom.TelecomSession"
 *
 * @ejb:transaction type="NotSupported"
 *
 *
 */
public class TelecomSessionBean implements SessionBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------
    private SessionContext mContext;
    private TelecomUserHomeLocal tuhl = null;
    private TelecomUserLocal tul = null;

    // -------------------------------------------------------------------------
    // Methods - SessionBean implementation
    // -------------------------------------------------------------------------

    /**
     * Create the All Projects Session Bean
     *
     * @throws CreateException
     *
     * @ejb:create-method
     */
    public void ejbCreate() throws CreateException, Exception {
        try {
            tuhl = TelecomUserBeanUtil.getLocalHome();
        } catch (NamingException e) {
            throw new Exception("NamingException TelecomDemo TelecomSessionBean constructor :\n", e);
        }
    }

    public void ejbPostCreate() throws CreateException {
    }

    public void setSessionContext(final SessionContext context) {
        mContext = context;
    }

    public void ejbRemove() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

    // -------------------------------------------------------------------------
    // Methods - public
    // -------------------------------------------------------------------------

    /**
     * @ejb:interface-method
     */
    public void addUser(String name, String firstName, String number, String street, String cp, String city) throws Exception {
	    tuhl.create(name, firstName, number, street, cp, city);
    }

    /**
     * @ejb:interface-method
     */
	public boolean archiveExist(String telecomRecordNb, String name)throws Exception{
	    boolean result = true;
	    try {
	        tuhl.findByPrimaryKey(new Integer(telecomRecordNb));
	    } catch (FinderException e) {
	        result = false;
	    }
	    return result;
	}

    /**
     * @ejb:interface-method
     */
    public Vector archiveFinish(String telecomRecordNb)throws Exception{
	    boolean finished = false;
	    String phoneNb = null;
	    String serviceNb = null;
	    Vector result = new Vector();
	    try {
	        tul = tuhl.findByPrimaryKey(new Integer(telecomRecordNb));
	        phoneNb = tul.getPhoneNb();
	        serviceNb = tul.getServiceNb();
	        if (phoneNb!=null && serviceNb!=null) {
		    	finished = true;
			} else {
				finished = false;
			}
	    } catch (FinderException e) {
	        finished = false;
	    }
	    result.addElement(new Boolean(finished));
	    result.addElement(phoneNb);
	    result.addElement(serviceNb);
	    return result;
	}

    /**
     * @ejb:interface-method
     */
	public long getTelecomRecordNb(String name, String firstName, String street) throws Exception{
	    long resultat = 0;
	    try {
	        tul = tuhl.findBy_N_FN_S(name, firstName,street);
	        resultat = tul.getId().longValue();
	    } catch (FinderException e) {}
		return resultat;
	}

    /**
     * @ejb:interface-method
     */
	public void modifyArchive(String telecomRecordNb,String name,String firstName,String phoneNb, String serviceNb)throws Exception{
	    try {
	        tul = tuhl.findByPrimaryKey(new Integer(telecomRecordNb));
	        tul.setPhoneNb(phoneNb);
	        tul.setServiceNb(serviceNb);
	    } catch (FinderException e) {
	        throw new Exception("Telecom user with Id : "+telecomRecordNb+" NOT FOUND !");
	    }
	}

}