/**
 * Copyright (C) 2005 Bull S.A.
 * Bull 1 rue de Provence 38130 ECHIROLLES
 **/
package org.objectweb.orchestra.demo.telecom;

import java.util.Collection;

import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import org.objectweb.jonas.service.ServiceException;

/**
 * TelecomUserBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: TelecomUserBean.java,v 1.1 2006/10/04 15:33:29 lejeuneg Exp $
 *
 * @ejb.bean name="TelecomUserBean"
 *           description="Deployment descriptor for the TelecomUserBean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/demo/telecom/TelecomUserHome"
 *           local-jndi-name="org/objectweb/orchestra/demo/telecom/TelecomUserHome_L"
 *           lock-policy="container-serialized-transacted"
 *
 * @ejb.home local-class="org.objectweb.orchestra.demo.telecom.TelecomUserHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.demo.telecom.TelecomUserLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "telecomUser"
 *                     automatic-pk="true"
 *
 *
 * @ejb:finder signature="org.objectweb.orchestra.demo.telecom.TelecomUserLocal findById(java.lang.Integer bpiId)"
 *             query="SELECT OBJECT(o) From TelecomUserBean as o Where o.id = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.demo.telecom.TelecomUserLocal findBy_N_FN_S(java.lang.String name, java.lang.String firstName, java.lang.String street)"
 *             query="SELECT OBJECT(o) From TelecomUserBean as o Where o.name = ?1 And o.firstName = ?2 And o.street = ?3"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.max-cache-size 10
 */

public abstract class TelecomUserBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String name, String firstName, String number, String street, String pc, String city) throws javax.ejb.CreateException {
        setName(name);
        setFirstName(firstName);
        setNb(number);
        setStreet(street);
        setPc(pc);
        setCity(city);
        return null;
    }

    public void ejbPostCreate(String name, String firstName, String number, String street, String pc, String city) {

    }

    // persistent fields

    /**
     * @return Integer : The BPI Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="Supports"
     */
    public abstract Integer getId();

    /**
     * @param id :
     *            Id of the BPI
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setId(Integer id);

    /**
     * @return Name of the serviceProvider
     *
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getName();

    /**
     * @param name : name of the serviceProvider
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setName(String name);

    /**
     * @return FirstName of the serviceProvider
     *
     * @ejb:persistence column-name="firstName"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getFirstName();

    /**
     * @param firstName : firstName of the serviceProvider
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setFirstName(String firstName);

    /**
     * @return Nb of the street
     *
     * @ejb:persistence column-name="nb"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getNb();

    /**
     * @param nb : nb
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setNb(String nb);

    /**
     * @return Street
     *
     * @ejb:persistence column-name="street"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getStreet();

    /**
     * @param street : street
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setStreet(String street);

    /**
     * @return pc postal code
     *
     * @ejb:persistence column-name="pc"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getPc();

    /**
     * @param pc : pc
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setPc(String pc);

    /**
     * @return city
     *
     * @ejb:persistence column-name="city"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getCity();

    /**
     * @param city : city
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setCity(String city);

    /**
     * @return phoneNb
     *
     * @ejb:persistence column-name="phoneNb"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getPhoneNb();

    /**
     * @param phoneNb : phoneNb
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setPhoneNb(String phoneNb);

    /**
     * @return serviceNb
     *
     * @ejb:persistence column-name="serviceNb"
     * @ejb.interface-method
     * @ejb:transaction type="Supports"
     */
    public abstract String getServiceNb();

    /**
     * @param serviceNb : serviceNb
     *
     * @ejb.interface-method
     * @ejb:transaction type="Required"
     */
    public abstract void setServiceNb(String serviceNb);

    public void setEntityContext(EntityContext ec) {
    }

    public void unsetEntityContext() {
    }

    public void ejbLoad() {
    }

    public void ejbStore() {
    }

    public void ejbActivate() {
    }

    public void ejbPassivate() {
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
    }

}