package org.objectweb.orchestra.samples.BSOASender.bf;


public class ConsoleBSOASender {

  final boolean debug = Boolean.getBoolean("org.objectweb.orchestra.samples.BSOASender.ConsoleBSOASender.debug");

  final static String appUri = "urn:org.objectweb.orchestra.samples.BSOASender.svc";
  final static String appPfx = "a";

  final static String soapUri = "http://schemas.xmlsoap.org/soap/envelope/";
  final static String soapPfx = "s";
  final static String soapNs = "xmlns:" + soapPfx + "=\"" + soapUri + "\"";


  /**
   * Display a message on the Orchestra console.
   *
   * @param soapEnvelope
   *          Soap request containing parameters to be assigned to an input
   *          record.
   * @param svc
   *          is not used by the echo sender.
   * @return a String containing a Soap response envelope
   * @throws Exception
   */
  public String invoke(String soapEnvelope, String svc) throws Exception {
    if (debug) System.out.println(soapEnvelope);

    SoapParser parser = new SoapParser(soapEnvelope);
    parser.addNamespace(soapPfx, soapUri);
    parser.addNamespace(appPfx, appUri);
    String s = parser.getNodeValue("//a:print/a:message");
    if (s == null) s = "DEFAULT ConsoleBSOASender Response";

    /*
     * Write the message to the console
     */
    System.out.println(s);

    /*
     * set up the soap response envelope
     */
    StringBuffer sb = new StringBuffer();
    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            + "<s:Envelope " + soapNs + "  >\n"
            + "<s:Body>\n"
            + "<printResponse xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n"
            + "    <printReturn>OK</printReturn>\n"
            + "</printResponse>\n"
            + "</s:Body>\n"
            + "</s:Envelope>\n");

    String result = sb.toString();

    if (debug)
      System.out.println(result);
    return result;
  }

  public static void main(String args[]) throws Exception {

    StringBuffer sb = new StringBuffer();

    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<s:Envelope " + soapNs + ">\n" +
      "<s:Body>\n" +
      "<print xmlns=\"urn:org.objectweb.orchestra.samples.BSOASender.svc\">\n" +
      "<message>");

    if (args.length > 0) {
      for (int i=0; i<args.length; ++i) {
        sb.append(args[i]).append(" ");
      }
    }
    else {
      sb.append("main: default message");
    }

    sb.append("</message>\n" +
      "</print>\n" +
      "</s:Body>\n" +
      "</s:Envelope>\n");
    ConsoleBSOASender sender = new ConsoleBSOASender();
    sender.invoke(sb.toString(), null);
  }
}