package org.objectweb.orchestra.samples.BSOASender.bf;

import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.net.URL;
import java.net.URLConnection;


/**
 * Forwards a SOAP request to the Servicemix JBI engine for processing.
 *
 * <p>This Sender is based on the HttpBinding Client.
 *
 * @author girouxm
 *
 */
public class JbiBSOASender {
  final boolean debug = Boolean.getBoolean("org.objectweb.orchestra.samples.BSOASender.JbiBSOASender.debug");

  final static String soapUri = "http://schemas.xmlsoap.org/soap/envelope/";
  final static String soapPfx = "s";
  final static String soapNs = "xmlns:" + soapPfx + "=\"" + soapUri + "\"";

  /**
   * Send a message to Servicemix JBI engine for processing.
   *
   * @param soapEnvelope
   *          Soap request containing parameters to be assigned to an input
   *          record.
   * @param svc
   *          host name (or ip address) and optional port.
   * @return a String containing a Soap response envelope
   * @throws Exception
   */
  public String invoke(String soapEnvelope, String svc) throws Exception {
    String result = "";

    if (debug) System.out.println(soapEnvelope);

    SoapParser parser = new SoapParser(soapEnvelope);
    parser.addNamespace(soapPfx, soapUri);
    String xml = parser.getNodeXml("//s:Body/*");

    if (xml == null) throw new Exception("required xml content not present in:\n" + soapEnvelope);

    /*
     * parse svc string for connection information
     */
    String JbiHost = svc;

    /*
     * Send the message to JBI engine
     */

    URLConnection connection = new URL("http://" + JbiHost).openConnection();
    connection.setDoOutput(true);
    OutputStream os = connection.getOutputStream();

    byte[] buf = xml.getBytes();
    os.write(buf);
    os.close();

    // Read the response.
    BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
    StringBuffer response = new StringBuffer();
    String inputLine;
    while ((inputLine = in.readLine()) != null) {
      response.append(inputLine);
    }
    in.close();

    /*
     * extract xml response
     */
    String respStr = response.toString().trim();
    for (int ltx = respStr.indexOf("<?"); ltx == 0; ltx = respStr.indexOf("<?")) {
      int gtx = respStr.indexOf("?>");
      respStr = respStr.substring(gtx+2);
    }
    SoapParser respParser = new SoapParser(response.toString());

    /*
     * set up the soap response envelope
     */
    StringBuffer sb = new StringBuffer();
    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            + "<s:Envelope " + soapNs + ">\n"
            + "<s:Body>\n"
            + respParser.getNodeXml("/")
            + "</s:Body>\n"
            + "</s:Envelope>\n");

    result = sb.toString();

    if (debug)
      System.out.println(result);
    return result;
  }

  public static void main(String args[]) throws Exception {

    StringBuffer sb = new StringBuffer();

    sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
      "<s:Envelope " + soapNs + ">\n" +
      "<s:Body>\n" );

    if (args.length > 1) {
      for (int i=1; i<args.length; ++i) {
        sb.append(args[i]).append("\n");
      }
    }
    else {
      sb.append(
          "<ns1:getQuote xmlns:ns1=\"urn:xmethods-delayed-quotes\"" +
          "  xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\">\n" +
          "  <symbol xsi:type=\"xsd:string\">IBM</symbol>\n" +
          "</ns1:getQuote>"
          );
    }

    sb.append(
      "</s:Body>\n" +
      "</s:Envelope>\n");
    JbiBSOASender sender = new JbiBSOASender();
    String answer = sender.invoke(sb.toString(), args[0]);
    SoapParser parser = new SoapParser(answer);
    parser.addNamespace(soapPfx, soapUri);
    // we know
    String quote = parser.getNodeValue("//s:Body/*/*");
    System.out.println("IBM selling at " + quote);
  }
}