/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.admin;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.loadservice.LoadProcessService;



/**
 * LoadProcessBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LoadProcessBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb:bean name="LoadProcess"
 *           display-name="Session LoadProcess Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="org/objectweb/orchestra/admin/LoadProcessHome"
 *           local-jndi-name="org/objectweb/orchestra/admin/LoadProcessHome_L"
 *           view-type="both"
 *
 * @ejb.home local-class="org.objectweb.orchestra.admin.LoadProcessHomeLocal"
 *           remote-class="org.objectweb.orchestra.admin.LoadProcessHome"
 * @ejb.interface local-class="org.objectweb.orchestra.admin.LoadProcessLocal"
 *                remote-class="org.objectweb.orchestra.admin.LoadProcess"
 *
 * @ejb:transaction type="NotSupported"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.max-cache-size @LoadProcessBean.max-cache-size@
 */

public class LoadProcessBean implements SessionBean {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------

    private SessionContext mContext;

    // -------------------------------------------------------------------------
    // Methods
    // -------------------------------------------------------------------------

    public SessionContext getContext() {
        return this.mContext;
    }

    /**
     * Create the Session Bean
     *
     * @throws CreateException
     *
     * @ejb.permission unchecked="yes"
     * @ejb:create-method view-type="both"
     */
    public void ejbCreate() throws CreateException {
        // Bean Method
    }

    public void setSessionContext(final javax.ejb.SessionContext context) {
        this.mContext = context;
    }

    /**
     * @ejb.permission unchecked="yes"
     */
    public void ejbRemove() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    /**
     * Add a jar of a new process to the engine
     *
     * @param jarFile
     * @ejb.permission unchecked="yes"
     * @ejb:interface-method view-type="both"
     *
     */
    public void addProcess(String jarFile) throws BPELEngineException {
        try {
            ServiceManager sm = ServiceManager.getInstance();
            LoadProcessService lp = (LoadProcessService) sm.getService("loadprocess");
            lp.addJar(jarFile);
        } catch (java.lang.Exception e) {
            throw new BPELEngineException("Exception LoadProcessBean addProcess :\n" , e);
        }
    }

    /**
     * Add a jar of a new process to the engine
     *
     * @param jarFile
     * @ejb.permission unchecked="yes"
     * @ejb:interface-method view-type="both"
     *
     */
    public void initProcesses() throws BPELEngineException {
        try {
            ServiceManager sm = ServiceManager.getInstance();
            LoadProcessService lp = (LoadProcessService) sm.getService("loadprocess");
            lp.setEjbJarClassLoader(Thread.currentThread().getContextClassLoader());
            lp.initProcesses();
        } catch (java.lang.Exception e) {
            throw new BPELEngineException("Exception LoadProcessBean addProcess :\n" , e);
        }
    }

}