/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework;

import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.axis.wsdl.toJava.Utils;
import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.util.BPELTrace;


/**
 * BCBaseWork is the BindingFramework base binding component class.  This the
 * the base for all component types supported by the BindingFramework.
 *
 * @author Eric Hardesty
 * @version $Id: BCBaseWork.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public abstract class BCBaseWork implements javax.resource.spi.work.Work {

    ClassLoader classL;
    String partLinkName;
    String qnamePortType;
    String invokeOp;
    BPELMessage jmsgin;
    BPELMessage jmsgout;
    String endpoint;
    String address;
    boolean outVar;
    String processName;
    int processId;
    ServiceListener sl;
    String serviceLocatorClassName;
    String port;


    /**
     * Constructor
     *
     * @param classL Classloader to find classes
     * @param partLinkName String partner link name
     * @param qnamePortType String QName port type
     * @param invokeOp String invoke operation
     * @param jmsgin BPELMessage input data
     * @param jmsgout BPELMessage output data
     * @param srvInfo String [] of service information
     * @param endpoint String endpoint to use
     * @param address String address to use
     * @param outVar boolean to deteremine if output variables are needed
     * @param processName String process name
     * @param processId String process Id
     * @param sl ServiceListener to get events
     */
    public BCBaseWork(ClassLoader classL, String partLinkName, String qnamePortType, String port, String serviceLocatorClassName,
            String invokeOp, BPELMessage jmsgin, BPELMessage jmsgout,
            String endpoint, String address, boolean outVar, String processName, int processId, ServiceListener sl) {
        this.classL = classL;
        this.partLinkName = partLinkName;
        this.qnamePortType = qnamePortType;
        this.port = port;
        this.serviceLocatorClassName = serviceLocatorClassName;
        this.invokeOp = invokeOp;
        this.jmsgin = jmsgin;
        this.jmsgout = jmsgout;
        this.endpoint = endpoint;
        this.address = address;
        this.outVar = outVar;
        this.processName = processName;
        this.processId = processId;
        this.sl = sl;
    }

    public abstract void release();

    public abstract void run();

    /**
     * Return the output parameters for the set provided
     *
     * @param set Set of mapped entries
     * @param key String of the key value to look for
     * @param usedparts int [] of parts
     * @param index int of offset of usedpart to update
     */
    Object getOutputParam(Set set, String key, int[] usedParts, int index) {
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BF BCBaseWork getOutputParam key = " + key);
        //get the output param corresponding to a key
        Object value = null;
        Map.Entry entry = null;
        for (Iterator n = set.iterator(); n.hasNext();) {
            entry = (Map.Entry) n.next();
            if (entry.getKey().toString().equals(key)) {
                value = entry.getValue();
                usedParts[index] = 1;
                break;
            }
        }
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BF BCBaseWork getOutputParam-end value = " + value);
        return value;
    }

    /**
     * Return the method for the given operation
     *
     * @param cl Class to look through
     * @param operation String of method to look for
     */
    Method getMethodByName(Class cl, String operation) {
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BF BCBaseWork getMethodByName  operation = " + operation + "  cl = " + cl);
        //find a method based on his name
        Method[] tabMet = null;
        tabMet = cl.getDeclaredMethods();
        int i = 0;
        while (i < tabMet.length && !tabMet[i].getName().equals(Utils.xmlNameToJava(operation))) {
            i++;
        }
        if (BPELTrace.isBFDebugEnabled())
            BPELTrace.traceBF("BF BCBaseWork getMethodByName-end  result = " + tabMet[i].getName());
        return tabMet[i];
    }

}