/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bindingframework;

import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.resource.spi.work.WorkManager;

import org.objectweb.orchestra.bindingframework.configuration.BFConfigInfo;
import org.objectweb.orchestra.bindingframework.configuration.BFConfigSaxHandler;
import org.objectweb.orchestra.bindingframework.configuration.BFEndpoint;
import org.objectweb.orchestra.bindingframework.configuration.BFSender;
import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.exception.BPELBindingFrameworkException;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELTrace;

import org.objectweb.orchestra.engine.ManagerHomeLocal;
import org.objectweb.orchestra.engine.ManagerLocal;
import org.objectweb.orchestra.engine.ManagerUtil;

/**
 * ServiceImpl.java.
 * @author Eric Hardesty
 * @version $Id: ServiceImpl.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class ServiceImpl {

    // -------------------------------------------------------------------------
    // Members
    // -------------------------------------------------------------------------

    private static BFConfigInfo bfConfig = null;

    private static WorkManager bfWrkMgr = null;

    public static void restart(ServiceListener sl) {
        // Walk thru all outstanding processes, posting with the given listener
    }

    public static boolean invoke(ClassLoader classL, String partLinkName, String qnamePortType, String port, String serviceLocatorClassName, String invokeOp,
            BPELMessage jmsgin, BPELMessage jmsgout, String address, boolean outVar,
            String processName, int processId, ServiceListener sl) throws BPELBindingFrameworkException {
        if (BPELTrace.isBFDebugEnabled()) {
            BPELTrace.traceBF("BF ServiceImpl invoke, classL = " + classL + "   partnerLink = " + partLinkName
                    + "   qnamePortType = " + qnamePortType + "   invokdeOp = " + invokeOp + "   jmsgin = " + jmsgin
                    + "  jmsgout = " + jmsgout + "   port = " + port + ", serviceLocatorClassName = "+serviceLocatorClassName
                    + "   address = " + address + "   sl = " + sl);
        }

        if (bfWrkMgr == null) {
            /*
             * MinThreads - 1 MaxThreads - 80 Waittimeout - 60 sec
             * ExecutionTimeout - 0 (unlimited)
             */

            bfWrkMgr = new BFWorkManager(null, 1, 80, 60, 0);
        }

        if (bfConfig == null || Boolean.getBoolean("org.objectweb.orchestra.bindingframework.cfgReread")) {
            BFConfigSaxHandler bfs = new BFConfigSaxHandler();
            bfConfig = bfs.parse("BFConfig.xml");
        }

        String endpoint = ServiceImpl.getEndpointURL(partLinkName, qnamePortType, invokeOp, address);

        // Determine component type to invoke
        // Persist data to restart work
        BCBaseWork sw = new BCSoapWork(classL, partLinkName, qnamePortType, port, serviceLocatorClassName, invokeOp, jmsgin,
                jmsgout, endpoint, address, outVar, processName, processId, sl);
        try {
//          bfWrkMgr.scheduleWork(sw);  //Used when BPEL is doing true aysnc processing
            bfWrkMgr.doWork(sw);
        } catch (Exception ex) {
            throw new BPELBindingFrameworkException("ServiceImpl.invoke, ex caught : ", ex);
        }
        return true;
    }

    /**
     * CallProcess.
     *
     * @param process : Name of the process
     * @param refPartner : Reference of the partner
     * @param portType : Name of the PortType
     * @param operation : Name of the Operation
     * @param jmsg : Message of the invoke
     * @param targetNamespace
     * @param oneWay
     * @throws BPELBindingFrameworkException
     */
    public void callProcess(String process, String refPartner, String portType, String operation, BPELMessage jmsg,
            String targetNamespace, boolean oneWay) throws BPELBindingFrameworkException {
        if (BPELTrace.isBFDebugEnabled()) {
            BPELTrace.traceBF("BF ServiceImpl.callProcess, process = " + process + "   refPartner = " + refPartner
                    + "   portType = " + portType + "   operation = " + operation + "   jmsg = " + jmsg
                    + "  targetNamespace = " + targetNamespace + "   oneWay = " + oneWay);
        }

        try {
            ManagerHomeLocal mhl = ManagerUtil.getLocalHome();
            ManagerLocal ml = mhl.create();
            ml.invoke(process, refPartner, portType, operation, jmsg, targetNamespace, oneWay);
            ml.remove();
        } catch (NamingException e) {
            throw new BPELBindingFrameworkException("NamingException BF callProcess : ", e);
        } catch (CreateException e) {
            throw new BPELBindingFrameworkException("CreateException BF callProcess : ", e);
        } catch (BPELEngineException e) {
            throw new BPELBindingFrameworkException("BPELEngineException BF callProcess :", e);
        } catch (RemoveException e) {
            throw new BPELBindingFrameworkException("RemoveException BF callProcess :", e);
        }

    }

    private static String getEndpointURL(String partLink, String qnamePortType, String invokeOp, String endpoint) {
        String url = endpoint;

        // Check if mapping specified for this url
        url = getEndpointURL(url);

        // Check if handler is being used
        int begIdx = url.indexOf("://");
        // process
        if (begIdx >= 0) {
            String subUrl = url.substring(begIdx+3);
            int protIdx = subUrl.indexOf('/');
            String protocol = "";
            String params = "";
            if (protIdx < 0) {
                protocol = subUrl.toLowerCase();
            } else {
                protocol = subUrl.substring(0, protIdx).toLowerCase();
                params = subUrl.substring(protIdx + 1);
            }

            if (BPELTrace.isBFDebugEnabled()) {
                bfConfig.printTables("Invoke");
            }
            String clsName = "";
            BFSender bfh  = bfConfig.getSenderClass(protocol);
            if (bfh != null) {
                clsName= bfh.getClassStr();
            }
            if (clsName != null && clsName.length() > 0) {
                url = "bsoa://" + clsName + "/" + params;
            }
        }

        return url;
    }

    private static String getEndpointURL(String url) {
        String ret= url;
        BFEndpoint bfe  = bfConfig.getEndpointURL(url);
        if (bfe != null) {
            ret = bfe.getAltref();
        }
        return ret;
    }

    //class end
}
