/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.model;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * ModelAttributes.java
 *
 * @author Bill Murray
 * @version $Id: ModelAttributes.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This abstract class forms the base for all data model elements. This
 *          includes leaf and container nodes, ports, and edges.
 */
public abstract class ModelAttributes {

    protected NamedNodeMap attributes;

    protected String key;

    private String keyAttrName;

    public ModelAttributes(String keyAttrName, NamedNodeMap attributes) {
        this.keyAttrName = keyAttrName;
        setAttributes(attributes);
    }

    public ModelAttributes(String key) {
        this.key = key;
        this.attributes = null;
    }

    /**
     * refresh - update the attribute data in this object with
     * the data from the node supplied.
     *
     * @param newMNNode
     */
    protected void refresh(ModelAttributes newMNNode) {
        setAttributes(newMNNode.getAttributes());
    }

    /**
     * getKey - gets the key for this ModelNode. The key is unique for the
     * entire data model.
     *
     * @return String
     */
    public String getKey() {
        return this.key;
    }

    /**
     * getAttrValue - return the value for the specified attribute.
     *
     * @param attrName
     * @return String
     */
    public String getAttrValue(String attrName) {
        if (this.attributes == null)
            return null;
        Node n = this.attributes.getNamedItem(attrName);
        if (n == null)
            return null;
        return n.getNodeValue();
    }

    /**
     * getAttributes - returns the attributes of the node.
     *
     * @return NamedNodeMap
     */
    public NamedNodeMap getAttributes() {
        return this.attributes;
    }

    /**
     * setAttributes - stores a new attribute node map and
     * update the key field. This should only be used internally
     * for refresh.
     *
     * @param attrs
     */
    protected void setAttributes(NamedNodeMap attrs) {
        this.attributes = attrs;
        this.key = getAttrValue(this.keyAttrName);
    }

    /**
     * getAllAttrs - returns a string containing all attribute name=value pairs.
     * This is used for tooltip as well as complete ascii display of the node.
     *
     * @return String
     */
    public String getAllAttrs() {
        StringBuffer st = new StringBuffer(120);

        for (int j = 0; j < this.attributes.getLength(); j++) {
            Node tmp = this.attributes.item(j);
            st.append(tmp.getNodeName());
            st.append("='");
            st.append(tmp.getNodeValue());
            st.append("' ");
        }
        return new String(st);
    }
}