/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.model;

import java.util.ArrayList;

/**
 * Options.java
 *
 * @author Bill Murray
 * @version $Id: Options.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class holds the various program options in static storage for
 *          global access.
 */
public class Options {

    public final static int TIMEFORMAT_NONE = 0;
    public final static int TIMEFORMAT_LONG = 1;
    public final static int TIMEFORMAT_RELATIVE = 2;
    public final static int TIMEFORMAT_SHORT = 3;

    private static boolean displayStmtNbrs = false;

    private static boolean displayLoopCount = false;

    private static int timeFormat = TIMEFORMAT_RELATIVE;

    private static ArrayList listeners = new ArrayList();

    public static boolean getDisplayStmtNbrs() {
        return displayStmtNbrs;
    }

    public static void setDisplayStmtNbrs(boolean displayStmtNbrs) {
        Options.displayStmtNbrs = displayStmtNbrs;
        changeOptionNotify();
    }

    public static boolean getDisplayLoopCount() {
        return displayLoopCount;
    }
    public static void setDisplayLoopCount(boolean displayLoopCount) {
        Options.displayLoopCount = displayLoopCount;
        changeOptionNotify();
    }

    public static int getTimeFormat() {
        return timeFormat;
    }

    public static void setTimeFormat(int timeFormat) {
        Options.timeFormat = timeFormat;
        changeOptionNotify();
    }

    /**
     * addListener - adds the specified optionsListener to the list and returns
     * the listener "ID". The can be used on subsequent calls.
     *
     * @param listener
     * @return int - the "ID" (index) assigned to this listener
     */
    public static int addListener(OptionsListener listener) {
        listeners.add(listener);
        return listeners.indexOf(listener);
    }

    public static void changeOptionNotify() {
        for (int i = 0; i < listeners.size(); i++) {
            ((OptionsListener) listeners.get(i)).optionChanged();
        }
    }
}