/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.GraphEdge;
import graph.GraphNode;
import graph.GraphPort;

/**
 * BPEMGraphitGraphEdge.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphEdge.java 5 2006-10-20 14:37:44Z lejeuneg $
 *          <p>
 *          This class extends the GraphEdge to account for the way we need to
 *          hdie edges.
 */
public class BPEMGraphitGraphEdge extends GraphEdge {

    /**
     * BPEMGraphitGraphEdge - constructor - This edge class gets its "hidden"
     * state from the ports it is connected to.
     *
     * @param key -
     *            the key for the edge
     */
    public BPEMGraphitGraphEdge(String key) {
        super(key);
    }

    /**
     * setUnderRetracted - Tells this edge that it is under a retracted
     * container. This function actually servers no purpose in this
     * implementation because of the way "isHidden" works.
     */
    public void setUnderRetracted(boolean b) {
        super.setUnderRetracted(b);
    }

    /**
     * isHidden - Determines if this edge is to be displayed or not. This is
     * complicated by the "underretracted" states of the ports and parents.
     * Basically, the edge keeps no state. Instead, it checks to see if any of
     * the ports are hidden or underretracted, or if any parent in the parent
     * chain is hidden or underretracted.
     */
    public boolean isHidden() {
        if (anyHiddenPort(this.srcPort))
            return true;
        if (anyHiddenPort(this.dstPort))
            return true;
        return super.isHidden();
    }

    private boolean anyHiddenPort(GraphPort port) {
        if (port.isUnderRetracted())
            return true;
        if (port.isHidden())
            return true;
        return anyHiddenParent(port.getNode());
    }

    private boolean anyHiddenParent(GraphNode node) {
        if (node == null)
            return false;
        if (node.isHidden())
            return true;
        if (node.isUnderRetracted())
            return true;

        return anyHiddenParent(node.getParent());
    }
}