/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import graph.DirectEdgeDisplay;
import graph.GraphAdapter;
import graph.GraphEdge;
import graph.GraphInterface;
import graph.GraphItem;
import graph.GraphLayoutManager;
import graph.GraphModel;
import graph.GraphMultiEdge;
import graph.GraphNode;
import graph.GraphPort;
import graph.SquareNodeGroupDisplay;
import graph.SquarePortDisplay;
import graph.TopoTrace;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.objectweb.orchestra.bpem.model.ModelEdge;
import org.objectweb.orchestra.bpem.model.ModelNode;


/**
 * BPEMGraphitGraphModel.java
 *
 * @author Bill Murray
 * @version $Id: BPEMGraphitGraphModel.java,v 1.2 2005/02/28 21:17:34 bmurray
 *          Exp $
 *          <p>
 *          This class implements the GraphModel to get the data model for the
 *          graph.
 */
public class BPEMGraphitGraphModel implements GraphModel {
    public final String RCSId = "@(#) $Id: BPEMGraphitGraphModel.java 5 2006-10-20 14:37:44Z lejeuneg $";

    private GraphInterface graphInterface;

    private GraphAdapter graphAdapter;

    /** The graph data structure. */
    private BPEMGraphitNodeGroup root;

    private Hashtable hModelNodes = new Hashtable();

    private Vector vModelNodes = new Vector();

    private Vector vAllNodeKeys = new Vector();

    private Hashtable hModelPorts = new Hashtable();

    private Vector vNodes = new Vector();

    private Vector vPorts = new Vector();

    private Vector vEdges = new Vector();

    private Vector vMultiEdges = new Vector();

    //private boolean useSubnets = true;

    public BPEMGraphitGraphModel(GraphInterface graphInterface,
            GraphAdapter graphAdapter) {
        final String methLabel = "SampleGraphModel";
        TopoTrace.debug(this, methLabel, "==>");
        this.graphAdapter = graphAdapter;
        //useSubnets = graphAdapter.separateSubnets;
        updateModel(graphInterface);
        TopoTrace.debug(this, methLabel, "<==");
    }

    public void separateSubnets(boolean b) {
        final String methLabel = "separateSubnets";
        TopoTrace.debug(this, methLabel, "==> b=" + b);
        //this.useSubnets = b;
        // update the model using the current GraphInterface
        updateModel(this.graphInterface);
    }

    /*
     * Updates data structures from the GraphInterface.
     */
    public synchronized void updateModel(GraphInterface graphInterface) {
        final String methLabel = "updateModel";
        TopoTrace.debug(this, methLabel, "==>");
        this.graphInterface = graphInterface;
        // rebuild the graph from the new interface
        this.hModelNodes.clear();
        this.vModelNodes.clear();
        this.vAllNodeKeys.clear();
        this.hModelPorts.clear();
        this.vPorts.clear();
        this.vEdges.clear();
        this.vMultiEdges.clear();
        // make a copy of vNodes
        Vector vOldNodes = new Vector(this.vNodes.size());
        for (int i = 0; i < this.vNodes.size(); i++) {
            vOldNodes.add(this.vNodes.get(i));
        }
        //
        // Create containers and leaf nodes in the order
        // they apear in the model.
        //
        String allNodeAndContainerKeys[] = ((BPEMGraphitGraphInterface) graphInterface)
        .getAllNodeAndContainerKeys();
        for (int i = 0; i < allNodeAndContainerKeys.length; i++) {
            ModelNode userObject = (ModelNode) graphInterface
            .getUserObject(allNodeAndContainerKeys[i]);
            GraphItem item;
            if (userObject.hasChildModelNodes()) {
                item = doContainer(userObject, vOldNodes);
            } else {
                item = doLeafNode(userObject, vOldNodes);
            }
            //###container.setCustomObject(new
            // SanGraphItem(container.getKey(),graphInterface));
            this.hModelNodes.put(allNodeAndContainerKeys[i], item);
            this.vModelNodes.add(item);
            this.vAllNodeKeys.add(allNodeAndContainerKeys[i]);

        }

        // create ports
        TopoTrace.debug(this, methLabel, "create ports");
        String[] portKeys = graphInterface.getPortKeys();
        for (int i = 0; i < portKeys.length; i++) {
            //###SanGraphPort port = new SanGraphPort(portKeys[i]);
            GraphPort port = new BPEMGraphitGraphPort(portKeys[i]);
            this.vPorts.add(port);
            this.hModelPorts.put(portKeys[i], port);
            // set node/port relationship
            String nodeKey = graphInterface.getNodeKey(port.getKey());
            GraphNode node = (GraphNode) this.hModelNodes.get(nodeKey);
            port.setNode(node);
            node.addPort(port);
            // customize the port
            port.setDisplay(new SquarePortDisplay(port));
            port.setGraphAdapter(this.graphAdapter);
            port.setUserObject(graphInterface.getUserObject(port.getKey()));
            //###port.setCustomObject(new
            // SanGraphItem(port.getKey(),graphInterface));
        }

        // create edges
        TopoTrace.debug(this, methLabel, "create edges");
        String[] edgeKeys = graphInterface.getEdgeKeys();
        for (int i = 0; i < edgeKeys.length; i++) {
            ModelEdge userEdge = (ModelEdge) graphInterface
            .getUserObject(edgeKeys[i]);
            //###GraphEdge edge = new GraphEdge(edgeKeys[i],userEdge);
            GraphEdge edge = new BPEMGraphitGraphEdge(edgeKeys[i]);
            // check and set port/edge relationship
            String srcKey = graphInterface.getSrcKey(edge.getKey());
            String dstKey = graphInterface.getDstKey(edge.getKey());
            GraphPort srcPort = (GraphPort) this.hModelPorts.get(srcKey);
            GraphPort dstPort = (GraphPort) this.hModelPorts.get(dstKey);
            if (srcPort == null) {
                TopoTrace.debug(this, methLabel, "ERROR: edge.key="
                        + edge.getKey() + ". No port for srcKey=" + srcKey);
                continue;
            }
            if (dstPort == null) {
                TopoTrace.debug(this, methLabel, "ERROR: edge.key="
                        + edge.getKey() + ". No port for dstKey=" + dstKey);
                continue;
            }
            // this edge is valid
            this.vEdges.add(edge);
            srcPort.addEdge(edge);
            dstPort.addEdge(edge);
            edge.setSrcPort(srcPort);
            edge.setDstPort(dstPort);
            // customize the edge
            edge.setGraphAdapter(this.graphAdapter);
            edge.setUserObject(userEdge);
            edge.setGrouping(userEdge.getGrouping());
            //###edge.setCustomObject(new
            // SanGraphItem(edge.getKey(),graphInterface));
            edge.setDisplay(new DirectEdgeDisplay(edge));
            // check if this edge must be included in a multiEdge
            if (edge.getGrouping() == GraphEdge.MULTIPLE) {
                boolean merged = false;
                for (int j = 0; j < this.vMultiEdges.size(); j++) {
                    BPEMGraphitGraphMultiEdge multi = (BPEMGraphitGraphMultiEdge) this.vMultiEdges
                    .get(j);
                    if (multi.intersects(edge)) {
                        multi.addEdge(edge);
                        merged = true;
                        edge.setMultiEdge(multi);
                        break;
                    }
                }
                // create a new MultiEdge
                if (!merged) {
                    BPEMGraphitGraphMultiEdge multi = new BPEMGraphitGraphMultiEdge(
                            edge.getKey());
                    //###multi.setCustomObject(new
                    // SanGraphItem(multi.getKey(),graphInterface));
                    this.vMultiEdges.add(multi);
                    multi.addEdge(edge);
                    edge.setMultiEdge(multi);
                }
            }
        }
        // consolidate multiple edges
        TopoTrace.debug(this, methLabel, "consolidate multiple edges");
//      Vector vMultiDeleted = new Vector();
        for (int i = 0; i < this.vMultiEdges.size(); i++) {
            BPEMGraphitGraphMultiEdge multi = (BPEMGraphitGraphMultiEdge) this.vMultiEdges
            .get(i);
//          if (vMultiDeleted.contains(multi))
//          continue;
//          for (int j = 0; j < multi.nbOfPorts(); j++) {
//          GraphPort port = (GraphPort) multi.getPort(j);
//          for (int k = 0; k < vMultiEdges.size(); k++) {
//          if (k == i)
//          continue;
//          BPEMGraphitGraphMultiEdge multi2 = (BPEMGraphitGraphMultiEdge) vMultiEdges
//          .get(k);
//          if (multi2.containsPort(port)) {
//          // transfer the edges
//          TopoTrace.debug(this, methLabel, "transfer multi2="
//          + multi2 + " to multi=" + multi);
//          for (int l = 0; l < multi2.nbOfEdges(); l++) {
//          GraphEdge edge = multi2.getEdge(l);
//          multi.addEdge(edge);
//          edge.setMultiEdge(multi);
//          }
//          vMultiDeleted.add(multi2);
//          }
//          }
//          }
            multi.setDisplay(new BPEMGraphitBusMultiEdgeDisplay(multi)); // ziggy
            // -
            // for
            // busMultiEdge
            multi.setGraphAdapter(this.graphAdapter); // ziggy - for busMultiEdge
            // customize each multiple edge
            //###customizeMultiEdge((SanGraphMultiEdge)multi);
        }
//      TopoTrace.debug(this, methLabel, "vMultiDeleted=" + vMultiDeleted);
//      for (int i = 0; i < vMultiDeleted.size(); i++) {
//      vMultiEdges.remove(vMultiDeleted.get(i));
//      }
        // Attach nodes to their parent
        // This has to be done in the original order to preserve
        // the desired flow.
        //
        TopoTrace.debug(this, methLabel, "hModelNodes=" + this.hModelNodes);
        Vector vSubContainers = new Vector();
        BPEMGraphitContainer topContainer = null;
        for (int i = 0; i < this.vAllNodeKeys.size(); i++) {
            GraphNode node = (GraphNode) this.hModelNodes.get(this.vAllNodeKeys.get(i));
            String contKey = graphInterface.getContainerKey(node.getKey());
            //          for (Enumeration nodes = hModelNodes.elements() ;
            // nodes.hasMoreElements() ; ) {
            //                  GraphNode node = (GraphNode)nodes.nextElement();
            //                  String contKey = graphInterface.getContainerKey(node.getKey());
            TopoTrace.debug(this, methLabel, "### node=" + node + " contKey="
                    + contKey);
            if (contKey != null) {
                BPEMGraphitContainer cont = (BPEMGraphitContainer) this.hModelNodes
                .get(contKey);
                cont.addChild(node);
                node.setParent(cont);
                vSubContainers.add(node.getKey());
                //
                // As nodes are added to their containers, set the initial
                // state to "underretracted"
                //
                node.setUnderRetracted(true);
            } else {
                topContainer = (BPEMGraphitContainer)node;
            }
        }
        topContainer.setExpanded(true);

        TopoTrace.debug(this, methLabel, "remove vSubContainers="
                + vSubContainers);
        // remove sub-containers
        for (int i = 0; i < vSubContainers.size(); i++) {
            this.hModelNodes.remove(vSubContainers.get(i));
        }

        // create a root group for the whole graph
        this.root = (BPEMGraphitNodeGroup) findItem(this.vNodes, "#");
        if (this.root == null) {
            this.root = new BPEMGraphitNodeGroup("#");
            this.vNodes.add(this.root);
            // customize the root node (once at creation)
            this.root.setDisplay(new SquareNodeGroupDisplay(this.root));
            this.root.setGraphAdapter(this.graphAdapter);
            //root.setPreferredLayout(GraphFlowLayout.class);
            this.root.setPreferredLayout(BPEMGraphitFlowLayout.class);
            this.root.getLayout().setOrientation(GraphLayoutManager.VERTICAL);
            this.root.getLayout().setAlignment(GraphLayoutManager.LEFT);
            this.root.setShowItemIcon(false);
            this.root.setShowItemLabel(false);
            this.root.setShowToggle(false);
            this.root.setShowLimit(false);
        } else {
            vOldNodes.remove(this.root);
            // already exists: remove contents
            this.root.removeAllChildren();
        }

//      // initialize the subnet ID of all nodes
//      for (int i = 0; i < vNodes.size(); i++) {
//      GraphNode node = (GraphNode) vNodes.get(i);
//      node.subnetId = null;
//      }

//      if (useSubnets) {
        // identify non-interconnected sub-networks
        Hashtable hSubnets = identifySubnets(this.hModelNodes);

        // create groups for sub-networks
        int n = 1;
        for (Enumeration subnetIds = hSubnets.keys(); subnetIds
        .hasMoreElements();) {
            String subnetId = (String) subnetIds.nextElement();
            String subnetName;
            if (subnetId.equals("")) {
                subnetName = "##";
            } else {
                subnetName = "#" + n;
                n++;
            }
            BPEMGraphitNodeGroup subg = (BPEMGraphitNodeGroup) findItem(
                    this.vNodes, subnetName);
            if (subg == null) {
                subg = new BPEMGraphitNodeGroup(subnetName);
                this.vNodes.add(subg);
                // customize the group node
                subg.setDisplay(new SquareNodeGroupDisplay(subg));
                subg.setGraphAdapter(this.graphAdapter);
                subg.setShowItemIcon(false);
                subg.setShowItemLabel(false);
                subg.setShowToggle(false);
                subg.setExpanded(true);
                subg.setShowLimit(false);
            } else {
                vOldNodes.remove(subg);
                // already exists: remove contents
                subg.removeAllChildren();
            }
            // attach each subgraph to the root
            this.root.addChild(subg);
            subg.setParent(this.root);
            // attach nodes to their subgraph
            Vector vSubnodes = (Vector) hSubnets.get(subnetId);
            for (int i = 0; i < vSubnodes.size(); i++) {
                GraphNode node = (GraphNode) vSubnodes.get(i);
                subg.addChild(node);
                node.setParent(subg);
            }
        }
//      } else {
//      // create only one subgraph
//      BPEMGraphitNodeGroup subg = (BPEMGraphitNodeGroup) findItem(vNodes,
//      "#0");
//      if (subg == null) {
//      subg = new BPEMGraphitNodeGroup("#0");
//      vNodes.add(subg);
//      // customize the group node
//      subg.setDisplay(new SquareNodeGroupDisplay(subg));
//      subg.setGraphAdapter(graphAdapter);
//      } else {
//      vOldNodes.remove(subg);
//      // already exists: remove contents
//      subg.removeAllChildren();
//      }
//      // attach the subgraph to the root
//      root.addChild(subg);
//      subg.setParent(root);
//      // attach all nodes to the subgraph
//      for (Enumeration nodes = hModelNodes.elements(); nodes
//      .hasMoreElements();) {
//      GraphNode node = (GraphNode) nodes.nextElement();
//      // attach each node to the subgraph
//      subg.addChild(node);
//      node.setParent(subg);
//      }
//      }
//      // delete the old items
//      TopoTrace.debug(this, methLabel, "vOldNodes=" + vOldNodes);
//      for (int i = 0; i < vOldNodes.size(); i++) {
//      vNodes.remove(vOldNodes.get(i));
//      }
//      vOldNodes = null; // gc

//      //DEBUG: TopoTrace.debug(this,methLabel,"Graph structure :");
//      //DEBUG: root.dump();

        // update the look'n feel of the items
        this.graphAdapter.dressItems(this);
        TopoTrace.debug(this, methLabel, "<==");
    }

    /**
     * doContainer - create or update a container item.
     *
     * @param mn -
     *            The model node for this item.
     * @param vOldNodes -
     *            list of old nodes
     * @return GraphItem - the new or updated item.
     */
    public GraphItem doContainer(ModelNode mn, Vector vOldNodes) {
        String key = mn.getKey();
        BPEMGraphitContainer container = (BPEMGraphitContainer) findItem(
                this.vNodes, key);
        if (container == null) {
            container = new BPEMGraphitContainer(mn.getKey());
            this.vNodes.add(container);
            container.setUserObject(mn);
            container.setDisplay(new BPEMGraphitContainerDisplay(container));
            container.setGraphAdapter(this.graphAdapter);
            container.setPreferredLayout(BPEMGraphitFlowLayout.class);
            if (mn.isParallelFlow()) {
                container.getLayout().setOrientation(
                        GraphLayoutManager.HORIZONTAL);
            } else {
                container.getLayout().setOrientation(
                        GraphLayoutManager.VERTICAL);
            }
            container.getLayout().setAlignment(GraphLayoutManager.CENTER);
        } else {
            vOldNodes.remove(container);
            // clean the node
            container.removeAllChildren();
            container.removeAllPorts();
            container.setParent(null);
        }
        return container;
    }

    /**
     * doLeafNode - create or update a leaf node item.
     *
     * @param mn -
     *            The model node for this item.
     * @param vOldNodes -
     *            list of old nodes
     * @return GraphItem - the new or updated item.
     */
    public GraphItem doLeafNode(ModelNode mn, Vector vOldNodes) {
        GraphNode node = (GraphNode) findItem(this.vNodes, mn.getKey());
        if (node == null) {
            //###node = new SanGraphNode(nodeKeys[i]);
            node = new GraphNode(mn.getKey());
            this.vNodes.add(node);
            // customize the node
            node.setUserObject(mn);
            node.setDisplay(new BPEMGraphitNodeDisplay(node));
            node.setGraphAdapter(this.graphAdapter);
        } else {
            vOldNodes.remove(node);
            // clean the node
            node.removeAllPorts();
            node.setParent(null);
        }
        return node;
    }

    public Hashtable identifySubnets(Hashtable hNodes) {
        final String methLabel = "identifySubnets";
        TopoTrace.debug(this, methLabel, "==>");

        // propagate the same subnet ID to interconnected nodes
        for (Enumeration nodes = hNodes.elements(); nodes.hasMoreElements();) {
            GraphNode node = (GraphNode) nodes.nextElement();
            if (node.nbOfEdges() == 0) {
                node.setSubnetId(""); // garbage sub-network
            } else {
                node.setSubnetId(node.getKey());
            }
        }

        // separate non-interconnected sub-networks
        Hashtable hSubnets = new Hashtable();
        for (Enumeration nodes = hNodes.elements(); nodes.hasMoreElements();) {
            GraphNode node = (GraphNode) nodes.nextElement();
            Vector vSubnodes = (Vector) hSubnets.get(node.subnetId);
            if (vSubnodes == null) {
                // create a new sub-network
                vSubnodes = new Vector();
                hSubnets.put(node.subnetId, vSubnodes);
            }
            // add this node in the sub-network
            vSubnodes.add(node);
        }
        TopoTrace.debug(this, methLabel, "<== hSubnets=" + hSubnets);
        return (hSubnets);
    }

    public int numOfNodes() {
        return (this.vNodes.size());
    }

    public int numOfPorts() {
        return (this.vPorts.size());
    }

    public int numOfEdges() {
        return (this.vEdges.size());
    }

    public int numOfMultiEdges() {
        return (this.vMultiEdges.size());
    }

    public GraphNode getRootNode() {
        return this.root;
    }

    public GraphNode getNode(int i) {
        return ((GraphNode) this.vNodes.get(i));
    }

    public GraphPort getPort(int i) {
        return ((GraphPort) this.vPorts.get(i));
    }

    public GraphEdge getEdge(int i) {
        return ((GraphEdge) this.vEdges.get(i));
    }

    public GraphMultiEdge getMultiEdge(int i) {
        return ((GraphMultiEdge) this.vMultiEdges.get(i));
    }

    /*
     * ### private void customizeMultiEdge(SanGraphMultiEdge multi) { final
     * String methLabel = "customizeMultiEdge";
     * TopoTrace.debug(this,methLabel,"==> key="+multi.getKey()); int protocol =
     * SanGraphEdge.UNKNOWN; for (int i = 0 ; i < multi.nbOfEdges() ; i++ ) {
     * SanGraphEdge edge = (SanGraphEdge)multi.getEdge(i); // protocol: avoid
     * UNKNOWN if mixed protocols protocol =
     * Math.min(protocol,edge.getProtocol()); } multi.setProtocol(protocol);
     * multi.setType(SanGraphAdapter.CNX); // only type supported yet
     * multi.setGraphAdapter(graphAdapter); switch(protocol) { case
     * SanGraphEdge.LOOP: //multi.setDisplay(new BusMultiEdgeDisplay(multi));
     * multi.setDisplay(new StarMultiEdgeDisplay(multi));
     * multi.setOrientation(DisplayManager.WEST); break; case SanGraphEdge.SCSI:
     * multi.setDisplay(new BusMultiEdgeDisplay(multi));
     * multi.setOrientation(DisplayManager.WEST); break; default:
     * multi.setDisplay(new StarMultiEdgeDisplay(multi)); }
     * TopoTrace.debug(this,methLabel," <=="); } ###
     */

    public Object getUserObject(String key) {
        return (this.graphInterface.getUserObject(key));
    }

    public GraphItem getItem(String key) {
        GraphItem graphItem = null;
        if ((graphItem = findItem(this.vNodes, key)) == null) {
            if ((graphItem = findItem(this.vPorts, key)) == null) {
                if ((graphItem = findItem(this.vEdges, key)) == null) {
                    graphItem = findItem(this.vMultiEdges, key);
                }
            }
        }
        return (graphItem);
    }

    private static GraphItem findItem(Vector vItems, String key) {
        for (int i = 0; i < vItems.size(); i++) {
            GraphItem item = (GraphItem) vItems.get(i);
            if (item.getKey().equals(key))
                return (item);
        }
        return (null);
    }
}