/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import java.text.SimpleDateFormat;

import org.objectweb.orchestra.data.BPIData;
import org.objectweb.orchestra.util.BPELConstants;



/**
 * BPIListInfo.java
 *
 * @author Bill Murray
 * @version $Id: BPIListInfo.java 5 2006-10-20 14:37:44Z lejeuneg $
 * <p>
 * This is the object stored in the BPI list. The only need of this
 * object is to have the "toString" method which is used by the list
 * view control to set the text for the list.
 */
public class BPIListInfo {

    private BPIData pi;

    /**
     * BPIListInfo - constructor
     *
     * @param pi -
     *            the ProcessInstance
     */
    public BPIListInfo(BPIData pi) {
        this.pi = pi;
    }

    /**
     * toString - called by the JComboBox to display the text for the list. Here
     * we customize the string to have the process name and the instanceID.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(50);
        String state = this.pi.getState();

        sb.append(this.pi.getProcessName());
        sb.append(" - ");
        sb.append(this.pi.getBPIId());

        if (state.equals(BPELConstants.BPI_ACTIVE)) {
            sb.append(" (Active) - ");
        } else if (state.equals(BPELConstants.BPI_FINISHED)) {
            sb.append(" (Finished) - ");
        } else if (state.equals(BPELConstants.BPI_TERMINATED)) {
            sb.append(" (Terminated) - ");
        }
        sb.append(getCreationTime());
        return new String(sb);
    }

    /**
     * getID - gets the ID for our ProcessInstance.
     *
     * @return - int - the instance ID.
     */
    public int getID() {
        return this.pi.getBPIId();
    }

    /**
     * getProcessInstance - gets the current ProcessInstance.
     *
     * @return
     */
    public BPIData getProcessInstance() {
        return this.pi;
    }

    public boolean isExecuting() {
        String state = this.pi.getState();
        if (state.equals(BPELConstants.BPI_ACTIVE)) {
            return true;
        } else if (state.equals(BPELConstants.BPI_FINISHED)) {
            return false;
        }
        return false;

    }

    public String getCreationTime() {
        SimpleDateFormat sdf = new SimpleDateFormat("MMMM/dd/yyy@hh:mm:ss");
        return sdf.format(this.pi.getCreationDate());
    }
}