/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.bpem.view;

import java.util.ArrayList;

import javax.swing.JTextArea;

import org.objectweb.orchestra.bpem.model.ExecutionModel;
import org.objectweb.orchestra.bpem.model.ModelListener;
import org.objectweb.orchestra.bpem.model.ModelNode;



/**
 * ViewAsText.java
 *
 * @author Bill Murray
 * @version $Id: ViewAsText.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class ViewAsText extends JTextArea implements ModelListener {

    private final ExecutionModel model;

    public ViewAsText(ExecutionModel model) {
        super();
        this.model = model;

        model.addListener(this);

        setLineWrap(true);
        setWrapStyleWord(true);
        setEditable(false);
    }

    /**
     * clear - Clear the text display area.
     */
    public void clear() {
        this.selectAll();
        this.replaceSelection("");
    }

    public void displayAll() {
        refreshPreserve(false);
    }

    public void nodeSelected(ModelNode mn) {
        // Nothing
    }

    /**
     * refresh - refresh the display by clearing it and then redisplay
     * everything.
     */
    public void refresh() {
        refreshPreserve(true);
    }

    private void refreshPreserve(boolean preserve) {
        // Noop if not displayed.
        //
        if (isShowing() == false)
            return;

        ModelNode root = this.model.getModelRoot();
        if (root != null) {
            int pos;
            //
            // Remember the current scroll position
            //
            if (preserve) {
                pos = this.getCaretPosition();
            } else {
                pos = 0;
            }
            // Refresh display
            //
            clear();
            dispNode(root, 0);
            //
            // Restore scroll position
            //
            this.setCaretPosition(pos);
        }
    }

    private void dispNode(ModelNode mn, int level) {
        ArrayList children = mn.getChildModelNodes();
        for (int i = 0; i < children.size(); i++) {
            ModelNode child = (ModelNode) children.get(i);
            String st = getSpacesAsString(level);

            //NamedNodeMap attributes = child.getAttributes();

            st += child.getName();

            st += " " + child.getAllAttrs();
            append(st + "\n");

            if (child.getChildModelNodes() != null) {
                dispNode(child, level + 1);
            }
        }
    }

    private String getSpacesAsString(int level) {
        StringBuffer result = new StringBuffer(level * 4);
        for (int i = 0; i < level; i++) {
            result.append("    ");
        }
        return new String(result);
    }

}