/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.client;

import java.rmi.RemoteException;

import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.SimpleCallbackHandler;

import org.objectweb.orchestra.admin.LoadProcess;
import org.objectweb.orchestra.admin.LoadProcessHome;
import org.objectweb.orchestra.admin.LoadProcessUtil;

/**
 * LoadProcessClient.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LoadProcessClient.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class LoadProcessClient {

    public static void main(String args[]) {
        String type = args[0];
        System.out.println("Running LoadProcessClient in mode "+type+"...");

        try {
            char[] password = BPELConstants.PASSWORD.toCharArray();
            SimpleCallbackHandler handler = new SimpleCallbackHandler(BPELConstants.USERNAME, password);
            LoginContext lc = new LoginContext(BPELConstants.LOGIN_MODULE, handler);
            lc.login();
        } catch (LoginException le) {
            System.out.println("User : bpeladmin not found in database.\nStopping LoadProcessClient in mode "+type+"...");
        }

        LoadProcessHome lph;
        LoadProcess lp = null;

        try {
            lph = LoadProcessUtil.getHome();
            lp = lph.create();
        } catch (javax.ejb.CreateException e) {
            System.out.println("CreateException in LoadProcessClient :\n" + e);
        } catch (NamingException e) {
            System.out.println("NamingException in LoadProcessClient :\n" + e);
        } catch (RemoteException e) {
            System.out.println("RemoteException in LoadProcessClient :\n" + e);
        }

        try {
            if (type.equals("init")) {
                lp.initProcesses();
            } else if (type.equals("add")) {
                String jar = args[1];
                System.out.println("Processing file " + jar + "...");
                lp.addProcess(jar);
            }
        } catch (BPELEngineException e) {
            System.out.println("BPELEngineException in LoadProcessClient when adding a process :\n" + e);
            e.printStackTrace();
        } catch (RemoteException e) {
            System.out.println("RemoteException in LoadProcessClient when adding a process :\n" + e);
            e.printStackTrace();
        }

        try {
            lp.remove();
        } catch (RemoveException e) {
            System.out.println("RemoveException in LoadProcessClient :\n" + e);
        } catch (RemoteException e) {
            System.out.println("RemoteException in LoadProcessClient :\n" + e);
        }
        System.out.println("Finishing LoadProcessClient...");

    }

}