/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.engine.BPELVariable;
import org.objectweb.orchestra.engine.MessageStructure;
import org.objectweb.orchestra.engine.PartStructure;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.exception.BPELMonitoringException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;
import org.objectweb.orchestra.util.BPELTrace;
import org.objectweb.orchestra.util.Util;
import org.objectweb.orchestra.util.WSDLJavaMatching;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.objectweb.orchestra.monitoring.BpelMonitoringSessionHomeLocal;
import org.objectweb.orchestra.monitoring.BpelMonitoringSessionLocal;
import org.objectweb.orchestra.monitoring.BpelMonitoringSessionUtil;

/**
 * BPICache.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPICache.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class BPICache {

    private static Vector bpilist = new Vector();
    private static int bpiNb = 0;
    private static Object mutex = new Object();

    public static int createBPI(String processName, String targetNamespace) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache createBPI");
        }
        int bpiId= -1;
        if (BPELEngine.isWithEjb()) {
            try {
                BPIEntityHomeLocal behl = BPIEntityBeanUtil.getLocalHome();
                BPIEntityLocal bel = behl.create(processName, targetNamespace);

                BpelProcessHomeLocal bphl = BpelProcessBeanUtil.getLocalHome();
                BpelProcessLocal bpl = bphl.findByName(processName);

                bel.setProcess(bpl);

                bpiId = (bel.getId()).intValue();
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache createBPI processName = " + processName
                        + " targetNamespace = " + targetNamespace + " : \n", e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException BPICache createBPI processName = " + processName
                        + " targetNamespace = " + targetNamespace + " : \n", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException BPICache createBPI processName = " + processName
                        + " targetNamespace = " + targetNamespace + " : \n", e);
            }
        } else {
            synchronized (mutex) {
                bpiId = bpiNb++;
            }
            BPIData bpid  = new BPIData(bpiId, processName, targetNamespace);
            bpid.setProcessId(BpelProcessCache.findProcess(processName));
            bpilist.add(bpid);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end createBPI");
        }
        return bpiId;
    }

    public static void removeBPI(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache removeBPI");
        }
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            BPIEntityLocal bpil = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                bpil = bpihl.findByPrimaryKey(new Integer(bpiId));
                bpil.remove();
            } catch (javax.ejb.FinderException fe) {
                //nothing
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache removeBPI :\n", e);
            } catch (javax.ejb.RemoveException re) {
                throw new BPELEngineException("RemoveException BPICache removeBPI bpiId = " + bpiId + " : \n", re);
            }
        } else {
            BPIData bpir = null;
            synchronized (bpilist) {
                for (Iterator it = bpilist.iterator(); it.hasNext();) {
                    BPIData bpid = (BPIData) it.next();
                    if (bpid.getBPIId() == bpiId) {
                        bpir = bpid;
                        break;
                    }
                }
                if (bpir != null) {
                    bpilist.remove(bpir);
                }
            }
            BPUCache.removeBPIBPUs(bpiId);
            VariableCache.removeBPIVariables(bpiId);
            ScopeCache.removeBPIScopes(bpiId);
            PartnerLinkCache.removeBPIPartnerLinks(bpiId);
            MessageCache.removeBPIMessages(bpiId);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end removeBPI");
        }
    }

    public static void storeMessages(BPIData bpi) throws BPELEngineException {
        String processName = bpi.getProcessName();
        int bpiId = bpi.getBPIId();
        long creationDateTime = bpi.getCreationDate().getTime();
        String fileName = BPELConstants.BUILD_XML_DIR+File.separator+processName+File.separator+bpiId+"_"+creationDateTime+"_"+"Mess"+".xml";
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = dbf.newDocumentBuilder();
            Document document = docBuilder.newDocument();
            Element root = document.createElement("Messages");
            document.appendChild(root);
            Collection messages = BPICache.getMessages(bpiId);
            for (Iterator i = messages.iterator(); i.hasNext();) {
                MessageData md = (MessageData) i.next();
                Element message = document.createElement("message");
                root.appendChild(message);
                message.setAttribute("statementNb", Integer.toString(md.getStatementNb()));
                message.setAttribute("isFault", Boolean.toString(md.isFault()));
                message.setAttribute("messageId", Integer.toString(md.getMessageId()));
                message.setAttribute("date", Long.toString(md.getDate()));
                message.setAttribute("targetNamespace", md.getTargetNamespace());
                message.setAttribute("refPartner", md.getRefPartner());
                message.setAttribute("processName", md.getProcessName());
                message.setAttribute("bpiId", Integer.toString(md.getBPIId()));
                message.setAttribute("portType", md.getPortType());
                message.setAttribute("partner", md.getPartner());
                message.setAttribute("operation", md.getOperation());
                message.setAttribute("faultName", md.getFaultName());
                message.setAttribute("faultMessage", md.getFaultMessage());
                addVariableValue(document, message, processName, md.getValue());
            }
            Util.writeXmlFile(document, fileName);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException caught in BPICache.storeMessages", e);
        } catch (ParserConfigurationException e) {
            throw new BPELEngineException("ParserConfigurationException caught in BPICache.storeMessages", e);
        }
    }

    public static void storeScopesVariables(BPIData bpi) throws BPELEngineException {
        String processName = bpi.getProcessName();
        int bpiId = bpi.getBPIId();
        long creationDateTime = bpi.getCreationDate().getTime();
        String fileName = BPELConstants.BUILD_XML_DIR+File.separator+processName+File.separator+bpiId+"_"+creationDateTime+"_"+"Vars"+".xml";
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = dbf.newDocumentBuilder();
            Document document = docBuilder.newDocument();
            Element root = document.createElement("Variables");
            document.appendChild(root);
            Collection scopes = getScopes(bpiId);
            for (Iterator i = scopes.iterator(); i.hasNext();) {
                ScopeData s = (ScopeData) i.next();
                int iter = s.getIter();
                int sId = s.getScopeId();
                String sNb = s.getNb();
                Collection variables = VariableCache.getScopeVariables(sId, bpiId);
                for (Iterator j = variables.iterator(); j.hasNext();) {
                    VariableData v = (VariableData) j.next();
                    String vName = v.getVariableName();
                    String vElement = v.getElement();
                    String vMT = v.getMessageType();
                    String vType = v.getType();
                    int vId = v.getVariableId();
                    Object vValue = v.getValue();

                    Element variable = document.createElement(vName);
                    root.appendChild(variable);
                    variable.setAttribute("element", vElement);
                    variable.setAttribute("messageType", vMT);
                    variable.setAttribute("type", vType);
                    variable.setAttribute("variableId", Integer.toString(vId));
                    variable.setAttribute("scopeId", Integer.toString(sId));
                    variable.setAttribute("scopeIter", Integer.toString(iter));
                    variable.setAttribute("scopeNb", sNb);
                    addVariableValue(document, variable, processName, vValue);
                }
            }
            Util.writeXmlFile(document, fileName);

        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException caught in BPICache.storeScopesVariables", e);
        } catch (ParserConfigurationException e) {
            throw new BPELEngineException("ParserConfigurationException caught in BPICache.storeScopesVariables", e);
        }

    }

    private static void addVariableValue(Document document, Element variable, String processName, Object val) throws BPELEngineException {
        Object value = null;
        try {
            value = Util.getValueObject(val, processName);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException caught in BPICache.addVariableValue", e);
        }
        String type = null;
        if (BPELMessage.class.isAssignableFrom(value.getClass())) {
            type = BPELConstants.BPELMESSAGE;
        } else if (BPELVariable.class.isAssignableFrom(value.getClass())) {
            type = BPELConstants.BPELVARIABLE;
        }

        if (type.equals(BPELConstants.BPELVARIABLE)) {
            BPELVariable jv = (BPELVariable) value;
            variable.setAttribute("value", getFinalValue(jv.getValue()));
            variable.setAttribute("class", jv.getValueClass().getName());
        } else if (type.equals(BPELConstants.BPELMESSAGE)) {
            BPELMessage jm = (BPELMessage) value;
            MessageStructure ms = jm.getStructure();
            PartStructure[] ps = ms.getParts();
            for (int i = 0; i < ps.length; i++) {
                String partName = ps[i].getName();
                Object partValue = jm.getPart(partName);
                Element part = document.createElement("part");
                part.setAttribute("partName", partName);
                //part.setAttribute("value","");
                part.setAttribute("class", WSDLJavaMatching.getWrapper(ps[i].getType()));
                variable.appendChild(part);
                addObjectValue(partValue, document, part);
            }
        }
    }

    private static String getFinalValue(Object value) {
        String s = "";
        if (value != null) {
            Class cl = value.getClass();
            if (Calendar.class.isAssignableFrom(cl)) {
                Calendar cal  =(Calendar) value;
                s = s + Integer.toString(cal.get(Calendar.YEAR))+",";
                s = s + Integer.toString(cal.get(Calendar.MONTH))+",";
                s = s + Integer.toString(cal.get(Calendar.DATE))+",";
                s = s + Integer.toString(cal.get(Calendar.HOUR_OF_DAY))+",";
                s = s + Integer.toString(cal.get(Calendar.MINUTE))+",";
                s = s + Integer.toString(cal.get(Calendar.SECOND));
            } else {
                s =  value.toString();
                //if value is a byte,long,integer or short, the toString method displays a decimal value
                //we must remove the ".xx"
                if (cl.equals(Integer.class) || cl.equals(Byte.class) || cl.equals(Long.class) || cl.equals(Short.class)) {
                    int index = s.indexOf(".");
                    if (index != -1) {
                        s = s.substring(0, index);
                    }
                }
            }
        }
        return s;
    }

    private static void addObjectValue(Object o, Document document, Element father) throws BPELEngineException {
        if (o == null) {
            father.setAttribute("value", "");
        } else {
            Class cl = o.getClass();
            if (WSDLJavaMatching.isFinalWsdlType(cl)) {
                father.setAttribute("value", getFinalValue(o));
                father.setAttribute("class", cl.getName());
            } else {
                //it is a complex object
                Vector fields = Util.getFields(cl);
                for (int i=0; i<fields.size(); i++) {
                    Field field = (Field) fields.get(i);
                    String fieldName = field.getName();
                    Class fieldClass = field.getType();
                    Class decaringClass = field.getDeclaringClass();
                    Element fieldEl = document.createElement(fieldName);
                    fieldEl.setAttribute("class", fieldClass.getName());
                    String prefix = "get";
                    if (fieldClass.equals(boolean.class)) {
                        prefix = "is";
                    }
                    Object newO = null;
                    try {
                        Method m = decaringClass.getMethod(prefix+BPELString.toUpper(fieldName), (Class[]) null);
                        newO = m.invoke(o, (Object[]) null);
                    } catch (Exception e) {
                        throw new BPELEngineException("Exception caught in BPICache.addObjectValue : ", e);
                    }
                    addObjectValue(newO, document, fieldEl);
                    father.appendChild(fieldEl);
                }
            }

        }
    }

    /**
     * Find BPIs finished having the given process.
     *
     */
    public static Collection getFinishedBPIs(String processName) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getFinishedBPIs(processName)");
        }
        Vector result = new Vector();
        String dirName = BPELConstants.BUILD_XML_DIR+File.separator+processName;
        File dir = new File(dirName);
        if (dir.exists() && dir.isDirectory()) {
            File[] files = dir.listFiles();
            for (int i=0; i<files.length; i++) {
                File f = files[i];
                String fileName = f.getName();
                if (fileName.endsWith("Inst.xml")) {
                    String[] s = fileName.replaceAll(".xml", "").split("_");
                    int bpiId = Integer.parseInt(s[0]);
                    Date creationDate = new Date(Long.parseLong(s[1]));
                    String state = "";
                    if (s[2].equals("F")) {
                        state = BPELConstants.BPI_FINISHED;
                    } else if (s[2].equals("T")) {
                        state = BPELConstants.BPI_TERMINATED;
                    }
                    result.addElement(new BPIData(bpiId, processName, state, creationDate));
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getFinishedBPIs(processName)");
        }
        return result;
    }

    /**
     * Find BPIs finished having the given process.
     *
     */
    public static Collection getFinishedBPIs() {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getFinishedBPIs");
        }
        Vector result = new Vector();
        String dirName = BPELConstants.BUILD_XML_DIR;
        File dir = new File(dirName);
        if (dir.exists() && dir.isDirectory()) {
            File[] files = dir.listFiles();
            for (int i=0; i<files.length; i++) {
                File f = files[i];
                String processName = f.getName();
                Collection bpisOfProcess = getFinishedBPIs(processName);
                result.addAll(bpisOfProcess);
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getFinishedBPIs");
        }
        return result;
    }

    /**
     * Find BPIs running a given process.
     *
     */
    public static Collection findProcessBPI(String processName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache findProcessBPI");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            Collection bpiBeans = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                bpiBeans = bpihl.findByProcess(processName);
                result = convert(bpiBeans);
            } catch (javax.ejb.FinderException fe) {
                // Nothing
                //throw new BPELEngineException("The processName called '"+processName+"' does not exists in BPEL engine. Perhaps it was not already created or there was an error during the creation.", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache findProcessBPI :\n", e);
            }
        } else {
            synchronized (bpilist) {
                for (Iterator it = bpilist.iterator(); it.hasNext();) {
                    BPIData bpid = (BPIData) it.next();
                    if (bpid.getProcessName().equals(processName)) {
                        result.add(bpid);
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end findProcessBPI");
        }
        return result;
    }

    /**
     * Find BPIs running a given process.
     *
     */
    public static int getNbRunningInstances(String processName) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache findProcessBPI");
        }
        int result = 0;
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                result = bpihl.getBPINameCnt(processName);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache getNbRunningInstances :\n", e);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getNbRunningInstances :\n", e);
            }
        } else {
            Vector inst = new Vector();
            synchronized (bpilist) {
                for (Iterator it = bpilist.iterator(); it.hasNext();) {
                    BPIData bpid = (BPIData) it.next();
                    if (bpid.getProcessName().equals(processName)) {
                        inst.add(bpid);
                    }
                }
            }
            result = inst.size();
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end findProcessBPI");
        }
        return result;
    }

    /**
     * Find BPIs running a given process.
     *
     */
    public static int getNbRunningInstances() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getNbRunningInstances");
        }
        int result = 0;
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                result = bpihl.getBPICnt();
            } catch (FinderException e) {
                // Nothing
                // throw new BPELEngineException("FinderException BPICache getNbRunningInstances :\n", e);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getNbRunningInstances :\n", e);
            }
        } else {
            result = bpilist.size();
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getNbRunningInstances");
        }
        return result;
    }

    /**
     * Find a BPI from its bpiId.
     *
     */
    public static BPIData findBPI(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache findBPI");
        }
        BPIData result = null;
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            BPIEntityLocal bpil = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                bpil = bpihl.findByPrimaryKey(new Integer(bpiId));
                result = convert(bpil);
            } catch (javax.ejb.FinderException fe) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache findBPI : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache findBPI :\n", e);
            }
        } else {
            synchronized (bpilist) {
                for (Iterator it = bpilist.iterator(); it.hasNext();) {
                    BPIData bpid = (BPIData) it.next();
                    if (bpid.getBPIId() == bpiId) {
                        result = bpid;
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end findBPI");
        }
        return result;
    }

    public static Collection getAllBPIs() throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getAllBPIs");
        }
        Vector result = bpilist;
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            Collection bpiBeans = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                bpiBeans = bpihl.findAllBPI();
                result = convert(bpiBeans);
            } catch (javax.ejb.FinderException fe) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache getAllBPIs : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getAllBPIs :\n", e);
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getAllBPIs");
        }
        return result;
    }

    public static Collection getBPUs(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getBPUs");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpiEH;
            BPIEntityLocal bpiE;
            Collection bpus;
            try {
                bpiEH = BPIEntityBeanUtil.getLocalHome();
                bpiE = bpiEH.findByPrimaryKey(new Integer(bpiId));
                bpus = bpiE.getBPUs();
                result = BPUCache.convert(bpus);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getBPUs : \n" , e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache getBPUs : \n" , e);
            }
        } else {
            result = BPUCache.getBPIBPUs(bpiId);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getBPUs");
        }
        return result;
    }

    public static Collection getScopes(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getScopes");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpiEH;
            BPIEntityLocal bpiE;
            Collection scopes;
            try {
                bpiEH = BPIEntityBeanUtil.getLocalHome();
                bpiE = bpiEH.findByPrimaryKey(new Integer(bpiId));
                scopes = bpiE.getScopes();
                result = ScopeCache.convert(scopes);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getScopes : \n" , e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache getScopes : \n" , e);
            }
        } else {
            result = ScopeCache.getBPIScopes(bpiId);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getScopes");
        }
        return result;
    }

    public static Collection getPartnerLinks(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getPartnerLinks");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpiEH;
            BPIEntityLocal bpiE;
            Collection pls;
            try {
                bpiEH = BPIEntityBeanUtil.getLocalHome();
                bpiE = bpiEH.findByPrimaryKey(new Integer(bpiId));
                pls = bpiE.getPartnerLinks();
                result = PartnerLinkCache.convert(pls);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getScopes : \n" , e);
            } catch (FinderException e) {
                // Nothing
                // throw new BPELEngineException("FinderException BPICache getScopes : \n" , e);
            }
        } else {
            result = PartnerLinkCache.getBPIPartnerLinks(bpiId);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getPartnerLinks");
        }
        return result;
    }

    public static Collection getMessages(int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache getMessages");
        }
        Vector result = new Vector();
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpiEH;
            BPIEntityLocal bpiE;
            Collection messages;
            try {
                bpiEH = BPIEntityBeanUtil.getLocalHome();
                bpiE = bpiEH.findByPrimaryKey(new Integer(bpiId));
                messages = bpiE.getMessages();
                result = MessageCache.convert(messages);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache getMessages : \n" , e);
            } catch (FinderException e) {
                // Nothing
                //throw new BPELEngineException("FinderException BPICache getMessages : \n" , e);
            }
        } else {
            result = MessageCache.getBPIMessages(bpiId);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end getMessages");
        }
        return result;
    }

    public static void setBPIState(int bpiId, String state) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache setBPIState bpi=" + bpiId + ", state=" + state);
        }
        if (BPELEngine.isWithEjb()) {
            BPIEntityHomeLocal bpihl = null;
            BPIEntityLocal bpil = null;
            try {
                bpihl = BPIEntityBeanUtil.getLocalHome();
                bpil = bpihl.findByPrimaryKey(new Integer(bpiId));
                bpil.setState(state);
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("FinderException BPICache setBPIState : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException BPICache setBPIState :\n", e);
            }
        } else {
            synchronized (bpilist) {
                for (Iterator it = bpilist.iterator(); it.hasNext();) {
                    BPIData bpid = (BPIData) it.next();
                    if (bpid.getBPIId() == bpiId) {
                        bpid.setState(state);
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("BPICache end setBPIState");
        }
    }

    private static Vector convert(Collection bpiLocalCol) {
        Vector result = new Vector();
        for (Iterator it = bpiLocalCol.iterator(); it.hasNext();) {
            BPIEntityLocal bpil = (BPIEntityLocal) it.next();
            result.add(convert(bpil));
        }
        return result;
    }

    private static BPIData convert(BPIEntityLocal bpil) {
        if (bpil != null) {
            BpelProcessLocal process = bpil.getProcess();
            int processId;
            if (process == null) {
                processId = -1;
            } else {
                processId = process.getId().intValue();
            }
            return new BPIData(bpil.getId().intValue(), bpil.getProcessName(), bpil.getTargetNamespace(), bpil.getState(), bpil.getCreationDate(), processId);
        }
        return null;
    }

    public static void flushBPIToXMLFile(BPIData bpi) throws BPELEngineException {
        String processName = bpi.getProcessName();
        int bpiId = bpi.getBPIId();
        long creationDateTime = bpi.getCreationDate().getTime();
        String state = bpi.getState();
        String s = "null";
        if (state.equals(BPELConstants.BPI_FINISHED)) {
            s = "F";
        } else if (state.equals(BPELConstants.BPI_TERMINATED)) {
            s = "T";
        }
        String fileName = BPELConstants.BUILD_XML_DIR+File.separator+processName+File.separator+bpiId+"_"+creationDateTime+"_"+s+"_"+"Inst"+".xml";
        try {
            BpelMonitoringSessionHomeLocal mHome = BpelMonitoringSessionUtil.getLocalHome();
            BpelMonitoringSessionLocal monitor = mHome.create();
            Document doc = monitor.getStatementsTree(bpiId);
            monitor.remove();
            Util.writeXmlFile(doc, fileName);
        } catch (BPELMonitoringException e) {
            throw new BPELEngineException("BPELMonitoringException BPICache flushBPIToXMLFile : ", e);
        } catch (BPELUtilException e) {
            throw new BPELEngineException("BPELUtilException BPICache flushBPIToXMLFile : ", e);
        } catch (javax.ejb.CreateException e) {
            throw new BPELEngineException("CreateException BPICache flushBPIToXMLFile : " , e);
        } catch (NamingException e) {
            throw new BPELEngineException("NamingException BPICache flushBPIToXMLFile : " , e);
        } catch (RemoveException e) {
            throw new BPELEngineException("RemoveException BPICache flushBPIToXMLFile : " , e);
        }
    }
}
