/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.io.Serializable;
import java.util.Date;

import org.objectweb.orchestra.util.BPELConstants;



/**
 * BPIData.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPIData.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BPIData implements Serializable {
    private int bpiId;
    private String processName;
    private String targetNamespace;
    private int processId;
    private String state;
    private Date creationDate;

    public BPIData(int id, String name, String tn) {
        this.bpiId = id;
        this.processName = name;
        this.targetNamespace = tn;
        this.creationDate = new Date();
        this.state = BPELConstants.BPI_ACTIVE;
    }

    public BPIData(int id, String name, String state, Date creationDate) {
        this.bpiId = id;
        this.processName = name;
        this.state = state;
        this.creationDate = creationDate;
    }

    public BPIData(int id, String name, String tn, String state, Date creationDate, int processId) {
        this.bpiId = id;
        this.processName = name;
        this.targetNamespace = tn;
        this.state = state;
        this.creationDate = creationDate;
        this.processId = processId;
    }

    public String toString() {
        String st = null;
        st = "BPIData : ";
        st += "bpiId = " + this.bpiId;
        st += ", processName = " + this.processName;
        st += ", targetNamespace = " + this.targetNamespace;
        st += ", processId = " + this.processId;
        st += ", state = " + this.state;
        st += ", creationDate = " + this.creationDate;
        return st;
    }

    public int getBPIId() {
        return this.bpiId;
    }

    public String getProcessName() {
        return this.processName;
    }

    public String getTargetNamespace() {
        return this.targetNamespace;
    }

    public int getProcessId() {
        return this.processId;
    }

    public String getState() {
        return this.state;
    }

    public Date getCreationDate() {
        return this.creationDate;
    }

    // WARNING Setters should only be called from Cache Class
    public void setProcessId(int id) {
        this.processId = id;
    }

    public void setState(String state) {
        this.state = state;
    }

    public void setCreationDate(Date d) {
        this.creationDate = d;
    }

}
