/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Date;

import javax.ejb.EntityContext;
import javax.ejb.FinderException;

import org.objectweb.orchestra.util.BPELConstants;



/**
 * BPIEntityBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPIEntityBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="BPIEntityBean"
 *           description="Deployment descriptor for the BPI bean"
 *           type="CMP"
 *           view-type="both"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/BPIEntityHome"
 *           local-jndi-name="org/objectweb/orchestra/data/BPIEntityHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.BPIEntityHomeLocal"
 *           remote-class="org.objectweb.orchestra.data.BPIEntityHome"
 *
 * @ejb.interface local-class="org.objectweb.orchestra.data.BPIEntityLocal"
 *                remote-class="org.objectweb.orchestra.data.BPIEntity"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelBPI"
 *                     automatic-pk="true"
 *
 * @ejb:finder signature="java.util.Collection findAllBPI()"
 *             query="SELECT OBJECT(o) From BPIEntityBean o"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByProcess(java.lang.String process)"
 *             query="SELECT OBJECT(o) From BPIEntityBean as o Where o.processName = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 *
 * @ejb:transaction type="Required"
 * @jonas.max-cache-size @BPIEntityBean.max-cache-size@
 */

public abstract class BPIEntityBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String processName, String targetNamespace) throws javax.ejb.CreateException {
        setProcessName(processName);
        setTargetNamespace(targetNamespace);
        //By default, stores in xml file
        setState(BPELConstants.BPI_ACTIVE);
        setCreationDate(new Date());
        return null;
    }

    // persistent fields

    /**
     * @return Integer : The BPI Id
     *
     * @ejb.pk-field
     * @ejb.interface-method
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @param id :
     *            Id of the BPI
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @return State of the Process
     *
     * @ejb:persistence column-name="state"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getState();

    /**
     * @param state : State of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setState(String state);

    /**
     * @return Name of the Process
     *
     * @ejb:persistence column-name="processName"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getProcessName();

    /**
     * @param process :
     *            Name of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setProcessName(String process);

    /**
     * @return ns the targetNamespace of the Process
     *
     * @ejb:persistence column-name="targetNamespace"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getTargetNamespace();

    /**
     * @param ns : the targetNamespace of the Process
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setTargetNamespace(String ns);

    /**
     * @return The creation date of the BPI
     *
     * @ejb:persistence column-name="creationDate"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Date getCreationDate();

    /**
     * @param state : the creation date of the BPI
     *
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCreationDate(Date creationDate);

    /**
     * @return the BPUs of the BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="BPI-BPU"
     *               role-name="BPI-has-BPU"
     *               target-multiple="no"
     *               target-ejb="BPUEntityBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract Collection getBPUs();

    /**
     * Set the BPUs of the BPI
     *
     * @param BPUs : the BPUs of the BPI
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPUs(Collection BPUs);

    /**
     * Retrieve the variables of this BPI.
     *
     * @return The collection of Variables of the BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIVariable"
     *               role-name="variables-of"
     *               target-ejb="VariableBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getVariables();

    /**
     * Set the variables of this BPI
     *
     * @param pVariables :
     *            The Collection of Variables of the BPI
     *
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setVariables(Collection pVariables);

    /**
     * Retrieve the messages of this BPI.
     *
     * @return The Collection of Messages of the BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIMessage"
     *               role-name="messages-of"
     *               target-ejb="MessageBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getMessages();

    /**
     * Set the messages of this BPI
     *
     * @param pMessages :
     *            the Collection of MEssages of the BPI
     *
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setMessages(Collection pMessages);

    /**
     * Retrieve the scopes of this BPI.
     *
     * @return The Collection of Scope of the BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIScope"
     *               role-name="scopes-of"
     *               target-ejb="ScopeBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getScopes();

    /**
     * Set the scopes of this BPI
     *
     * @param pScopes :
     *            the Collection of Scopes of the BPI
     *
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setScopes(Collection pScopes);

    /**
     * @return the Process of this BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="Process-BPI"
     *               role-name="BPI-has-Process"
     *               target-ejb="BpelProcessBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract org.objectweb.orchestra.data.BpelProcessLocal getProcess();

    /**
     * Set the Process of this BPI
     *
     * @param BPIs :
     *            the BPIs of this process
     *
     * @ejb:interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setProcess(org.objectweb.orchestra.data.BpelProcessLocal process);

    /**
     * Retrieve the partnerLinks of this BPI.
     *
     * @return The collection of partnerLinks of the BPI
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIPartnerLink"
     *               role-name="partnerLinks-of"
     *               target-ejb="PartnerLinkBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Collection getPartnerLinks();

    /**
     * Set the partnerLinks of this BPI
     *
     * @param pPartnerLinks : The Collection of PartnerLinks of the BPI
     *
     * @ejb:transaction type="@tx.entity.setMethod@"
     * @ejb:interface-method
     */
    public abstract void setPartnerLinks(Collection pPartnerLinks);

    /**
     *
     * @ejb.select
     *     signature="java.lang.Integer ejbGetBPICnt()"
     *     view-type="local" result-type-mapping="Local"
     *     query="SELECT COUNT(p.id) From BPIEntityBean as p"
     *     transaction-type="Supports"
     *
     */
    public abstract int ejbSelectBPICnt() throws FinderException;

    /**
     * @ejb.home-method view-type="local"
     * @ejb:transaction type="Supports"
     *
     **/
    public int ejbHomeGetBPICnt() throws FinderException {
        return ejbSelectBPICnt();
    }

    /**
     *
     * @ejb.select
     *     signature="java.lang.Integer ejbGetBPINameCnt(java.lang.String processName)"
     *     view-type="local" result-type-mapping="Local"
     *     query="SELECT COUNT(p.id) From BPIEntityBean as p
     *                            Where p.processName = ?1"
     *     transaction-type="Supports"
     *
     */
    public abstract int ejbSelectBPINameCnt(java.lang.String processName) throws FinderException;

    /**
     * @ejb.home-method view-type="local"
     * @param processName name of the process
     * @ejb:transaction type="Supports"
     *
     **/
    public int ejbHomeGetBPINameCnt(String processName) throws FinderException {
        return ejbSelectBPINameCnt(processName);
    }

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
        // Bean Method
    }

}