/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Vector;

import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import org.objectweb.orchestra.util.BPELTrace;



/**
 * BPUEntityBean.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPUEntityBean.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 * @ejb.bean name="BPUEntityBean"
 *           description="Deployment descriptor for the BPU bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/BPUEntityHome"
 *           local-jndi-name="org/objectweb/orchestra/data/BPUEntityHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.BPUEntityHomeLocal"
 *
 * @ejb.interface local-class="org.objectweb.orchestra.data.BPUEntityLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @ejb.ejb-ref ejb-name="BPIEntityBean"
 *              view-type="local"
 *              ref-name="ejb/BPIEntityHomeLocal"
 *
 * @ejb:transaction type="Required"
 *
 * @ejb.permission role-name="admin"
 *
 * @jonas.session-timeout session-timeout="60"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelBPU"
 *                     automatic-pk="true"
 * @ejb:finder signature="java.util.Collection findByBpiId(java.lang.Integer bpiId )"
 *             query="SELECT Object(u) From BPUEntityBean as u Where u.bPI.id = ?1"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByBpiIdAndState(java.lang.Integer bpiId, java.lang.String state )"
 *             query="SELECT Object(u) From BPUEntityBean as u Where u.bPI.id = ?1 and u.state = ?2"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @jonas.max-cache-size @BPUEntityBean.max-cache-size@
 */

public abstract class BPUEntityBean implements javax.ejb.EntityBean {

    /**
     * EJBCREATE.
     *
     * @return pk primary key set to null
     *
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String process, int bpiId, String state, int currentScopeId) throws javax.ejb.CreateException {
        //System.out.println("BPUEntityBean ejbCreate process= "+process+"
        // bpiId = "+bpiId);
        setProcess(process);
        setState(state);
        setMonitor(new Vector());
        setCurrentScopeId(currentScopeId);
        setStatementNb(-1);
        return null;
    }

    public void ejbPostCreate(String process, int bpiId, String state, int currentScopeId) {
        //System.out.println("BPUEntityBean ejbPostCreate process=
        // "+process+"ID = "+this.getId());
        try {
            // a mettre dans setEntityContext
            BPIEntityHomeLocal behl = BPIEntityBeanUtil.getLocalHome();

            setBPI(behl.findByPrimaryKey(new Integer(bpiId)));
            //BPIEntityLocal bpi = behl.findById(bpiId);
            //setBPI(bpi);
        } catch (FinderException e) {
            if (BPELTrace.isDataDebugEnabled()) {
                BPELTrace.traceData("FinderException in BPUEntityBean postCreate : " + e);
            }
            e.printStackTrace();
        } catch (NamingException e) {
            if (BPELTrace.isDataDebugEnabled()) {
                BPELTrace.traceData("NamingException in BPUEntityBean postCreate : " + e);
            }
            e.printStackTrace();
        }
    }

    // persistent fields
    /**
     * @ejb.interface-method
     * @ejb.pk-field
     * @ejb:persistence column-name="id"
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="process"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getProcess();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setProcess(String process);

    /**
     * @ejb:persistence column-name="state"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getState();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setState(String state);

    /**
     * @ejb:persistence column-name="statementNb"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract int getStatementNb();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setStatementNb(int nb);

    /**
     * @ejb:persistence column-name="monitor"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Vector getMonitor();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setMonitor(Vector monitor);

    /**
     * @ejb:persistence column-name="currentScopeId"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract int getCurrentScopeId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCurrentScopeId(int currentScopeId);


    /**
     * @return Returns a BPIEntityLocal
     *
     * @ejb:interface-method
     * @ejb:relation name="BPI-BPU"
     *               role-name="BPU-has-BPI"
     *               target-multiple="yes"
     *               cascade-delete="yes"
     *               target-ejb="BPIEntityBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract org.objectweb.orchestra.data.BPIEntityLocal getBPI();

    /**
     * Set the BPI.
     *
     * @ejb:interface-method
     *
     * @param bpi : The BPIEntity of this Property.
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPI(org.objectweb.orchestra.data.BPIEntityLocal bpi);

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws javax.ejb.RemoveException {
        // Bean Method
    }

}
