/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

/**
 * CSBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CSBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="CSBean"
 *           description="Deployment descriptor for the CS bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/CSHome"
 *           local-jndi-name="org/objectweb/orchestra/data/CSHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.CSHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.data.CSLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelCS"
 *                     automatic-pk="true"
 *
 * @ejb.permission role-name="admin"
 *
 * @ejb:finder signature= "java.util.Collection findCS( java.lang.String csname )"
 *             query ="SELECT OBJECT(o) FROM CSBean o where o.cSName = ?1"
 *             transaction-type="Required"
 *
 * @ejb:finder signature= "java.util.Collection findAllCS()"
 *             query ="SELECT OBJECT(o) FROM CSBean o"
 *
 * @ejb:finder signature="java.util.Collection findByBpiNameAndScope(java.lang.Integer bpiId, java.lang.String csName, java.lang.Integer scopeId)"
 *             query="SELECT Object(u) From CSBean as u Where u. bPIId = ?1 and u.cSName = ?2 and u.scope.id = ?3"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByNameAndBpi(java.lang.String csName, java.lang.Integer bpiId)"
 *             query="SELECT Object(u) From CSBean as u Where u.cSName = ?1 and u.bPIId = ?2"
 *             result-type-mapping="Local"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="java.util.Collection findByBpi(int bpi )"
 *             query="SELECT Object(u) From CSBean as u Where u.bPIId = ?1" transaction-type="Supports"
 *             result-type-mapping="Local"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.max-cache-size @CSBean.max-cache-size@
 */

public abstract class CSBean implements EntityBean {

    /**
     * ejbCreate
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     *
     */
    public Integer ejbCreate(Integer bpiId, String CSName, Object CSValue) throws CreateException {
        setBPIId(bpiId);
        setCSName(CSName);
        setCSValue(CSValue);
        return null;
    }

    /**
     * ejbCreate
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(Integer bpiId, String CSName) throws CreateException {
        setBPIId(bpiId);
        setCSName(CSName);
        return null;
    }

    // persistent fields

    /**
     * @ejb.pk-field
     * @ejb:persistence column-name="id"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="csName"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getCSName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCSName(String nam);

    /**
     * @ejb:persistence column-name="csValue"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Object getCSValue();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setCSValue(Object val);

    /**
     * @ejb:persistence column-name="BPIId"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getBPIId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPIId(Integer bpiId);

    /**
     * @return Returns a ScopeLocal
     * @ejb:interface-method
     * @ejb:relation name="Scope-CS"
     *               role-name="CS-has-Scope"
     *               target-multiple="yes"
     *               cascade-delete="yes"
     *               target-ejb="ScopeBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract org.objectweb.orchestra.data.ScopeLocal getScope();

    /**
     * Set the Scope.
     *
     * @ejb:interface-method
     * @param scope The Scope of this Property.
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setScope(org.objectweb.orchestra.data.ScopeLocal scope);

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws RemoveException {
        // Bean Method
    }

}