/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;
import javax.xml.namespace.QName;

/**
 * PartnerLinkBean.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PartnerLinkBean.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 * @ejb.bean name="PartnerLinkBean"
 *           description="Deployment descriptor for the PartnerLink bean"
 *           type="CMP"
 *           view-type="local"
 *           primkey-field="id"
 *           cmp-version="2.x"
 *           jndi-name="org/objectweb/orchestra/data/PartnerLinkHome"
 *           local-jndi-name="org/objectweb/orchestra/data/PartnerLinkHome_L"
 *           reentrant="true"
 *
 * @jonas.bean lock-policy="@lock.policy@"
 *
 * @ejb.home local-class="org.objectweb.orchestra.data.PartnerLinkHomeLocal"
 * @ejb.interface local-class="org.objectweb.orchestra.data.PartnerLinkLocal"
 *
 * @ejb:pk class="java.lang.Integer"
 *
 * @jonas.jdbc-mapping jndi-name="bpel"
 *                     jdbc-table-name = "bpelPartnerLink"
 *                     automatic-pk="true"
 *
 * @ejb:finder signature= "java.util.Collection findAllPartnerLink()"
 *             query ="SELECT OBJECT(o) FROM PartnerLinkBean o"
 *
 * @ejb:finder signature="java.util.Collection findByProcess( java.lang.String process )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From PartnerLinkBean as u Where u.process = ?1"
 *             transaction-type="Supports"
 *
 * @ejb:finder signature="org.objectweb.orchestra.data.PartnerLinkLocal findByProcessAndName( java.lang.String process,java.lang.String pName )"
 *             result-type-mapping="Local"
 *             query="SELECT Object(u) From PartnerLinkBean as u Where u.process = ?1 and u.name = ?2"
 *             transaction-type="Supports"
 *
 * @jonas.session-timeout session-timeout="60"
 * @ejb:transaction type="Required"
 *
 * @jonas.max-cache-size @PartnerLinkBean.max-cache-size@
 */

public abstract class PartnerLinkBean implements EntityBean {

    /**
     * ejbCreate
     *
     * @return pk primary key set to null
     *
     * @ejb.create-method
     */
    public Integer ejbCreate(String name, String process, QName type, String myRole, String partnerRole, QName myRolePortType, QName partnerRolePortType, String myRoleEndpoint, String partnerRoleEndpoint) throws CreateException {
        setName(name);
        setProcess(process);
        setType(type);
        setMyRole(myRole);
        setPartnerRole(partnerRole);
        setMyRolePortType(myRolePortType);
        setPartnerRolePortType(partnerRolePortType);
        setMyRoleEndpoint(myRoleEndpoint);
        setPartnerRoleEndpoint(partnerRoleEndpoint);
        return null;
    }

    // persistent fields

    /**
     * @ejb.pk-field
     * @ejb:persistence column-name="id"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract Integer getId();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setId(Integer id);

    /**
     * @ejb:persistence column-name="name"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getName();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setName(String name);

    /**
     * @ejb:persistence column-name="process"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getProcess();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setProcess(String value);

    /**
     * @ejb:persistence column-name="type"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract QName getType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setType(QName type);

    /**
     * @ejb:persistence column-name="myRole"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getMyRole();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setMyRole(String myRole);

    /**
     * @ejb:persistence column-name="partnerRole"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getPartnerRole();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setPartnerRole(String partnerRole);

    /**
     * @ejb:persistence column-name="myRolePortType"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract QName getMyRolePortType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setMyRolePortType(QName myRolePortType);

    /**
     * @ejb:persistence column-name="partnerRolePortType"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract QName getPartnerRolePortType();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setPartnerRolePortType(QName partnerRolePortType);

    /**
     * @ejb:persistence column-name="myRoleEndpoint"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getMyRoleEndpoint();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setMyRoleEndpoint(String myRoleEndpoint);

    /**
     * @ejb:persistence column-name="partnerRoleEndpoint"
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.getMethod@"
     */
    public abstract String getPartnerRoleEndpoint();

    /**
     * @ejb.interface-method
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setPartnerRoleEndpoint(String partnerRoleEndpoint);

    /**
     * @return Returns a BPIEntityLocal
     *
     * @ejb:interface-method
     * @ejb:relation name="BPIPartnerLink"
     *               role-name="BPI-of"
     *               target-multiple="yes"
     *               cascade-delete="yes"
     *               target-ejb="BPIEntityBean"
     * @ejb:transaction type="@tx.entity.getMethod@"
     *
     */
    public abstract org.objectweb.orchestra.data.BPIEntityLocal getBPI();

    /**
     * Set the BPI.
     *
     * @ejb:interface-method
     *
     * @param bpi : The BPIEntity of this Property.
     * @ejb:transaction type="@tx.entity.setMethod@"
     */
    public abstract void setBPI(org.objectweb.orchestra.data.BPIEntityLocal bpi);

    // standard call back methods

    public void setEntityContext(EntityContext ec) {
        // Bean Method
    }

    public void unsetEntityContext() {
        // Bean Method
    }

    public void ejbLoad() {
        // Bean Method
    }

    public void ejbStore() {
        // Bean Method
    }

    public void ejbActivate() {
        // Bean Method
    }

    public void ejbPassivate() {
        // Bean Method
    }

    public void ejbRemove() throws RemoveException {
        // Bean Method
    }

}