/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.data;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.xml.namespace.QName;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELTrace;


/**
 * PartnerLinkCache.java.
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PartnerLinkCache.java 39 2006-11-21 10:09:54Z lejeuneg $
 *
 */
public class PartnerLinkCache {

    private static Vector plList = new Vector();
    private static int plNb = 0;
    private static Object mutex = new Object();

    public static void createPartnerLink(String plName, String processName, QName plType, String plMyRole,
            String plPartnerRole, QName plMyRolePT, QName plPartnerRolePT, String plMyRoleEndpoint,
            String plPartnerRoleEndpoint, int bpiId) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache createPartnerLink");
        }
        if (BPELEngine.isWithEjb()) {
            try {
                PartnerLinkHomeLocal pl = PartnerLinkBeanUtil.getLocalHome();
                PartnerLinkLocal pLocal = pl.create(plName, processName, plType, plMyRole, plPartnerRole,
                        plMyRolePT, plPartnerRolePT, plMyRoleEndpoint, plPartnerRoleEndpoint);
                BPIEntityHomeLocal bpihl = BPIEntityBeanUtil.getLocalHome();
                BPIEntityLocal bpil = bpihl.findByPrimaryKey(new Integer(bpiId));
                pLocal.setBPI(bpil);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException PartnerLinkCache createPartnerLink", e);
            } catch (CreateException e) {
                throw new BPELEngineException("CreateException PartnerLinkCache createPartnerLink", e);
            } catch (FinderException e) {
                throw new BPELEngineException("FinderException PartnerLinkCache createPartnerLink", e);
            }
        } else {
            int plId;
            synchronized (mutex) {
                plId = plNb++;
            }
            PartnerLinkData pld = new PartnerLinkData(plId, plName, processName, plType, plMyRole, plPartnerRole, plMyRolePT,
                    plPartnerRolePT, plMyRoleEndpoint, plPartnerRoleEndpoint, bpiId);
            plList.add(pld);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache end createPartnerLink");
        }
    }

    public static void setPartnerRoleEndpoint(int plId, String endPoint) throws BPELEngineException {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache setPartnerRoleEndpoint");
        }
        if (BPELEngine.isWithEjb()) {
            PartnerLinkHomeLocal plhl = null;
            PartnerLinkLocal pl = null;
            try {
                plhl = PartnerLinkBeanUtil.getLocalHome();
                pl = plhl.findByPrimaryKey(new Integer(plId));
                pl.setPartnerRoleEndpoint(endPoint);
            } catch (javax.ejb.FinderException fe) {
                throw new BPELEngineException("FinderException PartnerLinkCache setPartnerRoleEndpoint : \n", fe);
            } catch (NamingException e) {
                throw new BPELEngineException("NamingException PartnerLinkCache setPartnerRoleEndpoint :\n", e);
            }
        } else {
            synchronized (plList) {
                for (Iterator it = plList.iterator(); it.hasNext();) {
                    PartnerLinkData pld = (PartnerLinkData) it.next();
                    if (pld.getPlId() == plId) {
                        pld.setPartnerRoleEndpoint(endPoint);
                        break;
                    }
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache end setPartnerRoleEndpoint");
        }
    }

    // Only called when working without EJB
    // With EJB relations are used
    public static Vector getBPIPartnerLinks(int bpiId) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache getBPIPartnerLinks");
        }
        Vector result = new Vector();
        synchronized (plList) {
            for (Iterator it = plList.iterator(); it.hasNext();) {
                PartnerLinkData pld = (PartnerLinkData) it.next();
                if (pld.getBPIId() == bpiId) {
                    result.add(pld);
                }
            }
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache end getBPIPartnerLinks");
        }
        return result;
    }

    // Only called when working without EJB
    // With EJB relations are used
    public static void removeBPIPartnerLinks(int bpiId) {
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache removeBPIPartnerLinks");
        }
        Vector result = new Vector();
        synchronized (plList) {
            for (Iterator it = plList.iterator(); it.hasNext();) {
                PartnerLinkData pld = (PartnerLinkData) it.next();
                if (pld.getBPIId() == bpiId) {
                    result.add(pld);
                }
            }
            plList.removeAll(result);
        }
        if (BPELTrace.isDataDebugEnabled()) {
            BPELTrace.traceData("PartnerLinkCache end removeBPIPartnerLinks");
        }
    }

    public static Vector convert(Collection plLocalCol) {
        Vector result = new Vector();
        for (Iterator it = plLocalCol.iterator(); it.hasNext();) {
            PartnerLinkLocal pl = (PartnerLinkLocal) it.next();
            result.add(convert(pl));
        }
        return result;
    }

    private static PartnerLinkData convert(PartnerLinkLocal pl) {
        if (pl != null) {
            BPIEntityLocal bpi = pl.getBPI();
            int bpiId;
            if (bpi == null) {
                bpiId = -1;
            } else {
                bpiId = bpi.getId().intValue();
            }
            return new PartnerLinkData(pl.getId().intValue(), pl.getName(), pl.getProcess(), pl.getType(), pl.getMyRole(),
                    pl.getPartnerRole(), pl.getMyRolePortType(), pl.getPartnerRolePortType(), pl.getMyRoleEndpoint(),
                    pl.getPartnerRoleEndpoint(), bpiId);
        }
        return null;
    }
}
