/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

/**
 * InvokeInbound.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: InvokeInbound.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class InvokeInbound extends Object {

    private String mOperation;
    private String mPortType;
    private String mPartner;
    private String mRefPartner;
    private boolean oneWay = true;

    /**
     * Constructor
     *
     * @param operation :
     *            Name of the Operation
     * @param portType :
     *            Name of the Port Type
     * @param partner :
     *            Name of the Partner
     */
    public InvokeInbound(String operation, String portType, String partner, boolean oneWay) {
        this.mOperation = operation;
        this.mPortType = portType;
        this.mPartner = partner;
        this.oneWay = oneWay;
    }

    /**
     * Constructor
     *
     * @param operation :
     *            Name of the Operation
     * @param portType :
     *            Name of the Port Type
     * @param partner :
     *            Name of the Partner
     * @param refPartner :
     *            Reference of the Partner
     */
    public InvokeInbound(String operation, String portType, String partner, String refPartner, boolean oneWay) {
        this.mOperation = operation;
        this.mPortType = portType;
        this.mPartner = partner;
        this.mRefPartner = refPartner;
        this.oneWay = oneWay;
    }

    /**
     * @return Name of the Operation
     */
    public String getOperation() {
        return this.mOperation;
    }

    /**
     * @return Name of the Port Type
     */
    public String getPortType() {
        return this.mPortType;
    }

    /**
     * @return Name of the Partner
     */
    public String getPartner() {
        return this.mPartner;
    }

    /**
     * @return Reference of the Partner
     */
    public String getRefPartner() {
        return this.mRefPartner;
    }

    /**
     * @return true if this ws is oneWay
     */
    public boolean isOneWay() {
        return this.oneWay;
    }

    /**
     * @param ii :
     *            InvokeInbound to be compared with
     * @return true if InvokeInbounds have the same Operation Name, Port Type
     *         Name and Partner Name
     */
    public boolean equals(InvokeInbound ii) {
        return ((this.mOperation.equals(ii.getOperation())) && (this.mPortType.equals(ii.getPortType())) && (this.mPartner.equals(ii.getPartner())));
    }
}
