/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.util.Vector;

/**
 * PendingEvent.java
 *
 * This class represents both pick events and eventhandlers
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PendingEvent.java 5 2006-10-20 14:37:44Z lejeuneg $
 * 
 **/

public class PendingEvent {

    private Vector onMessages;
    private Vector alarms;
    private int foundOnMessage;
    private int scopeId;
    private int id;
    private int statementNb;
    private int bpuId;
    private String type;//pick or event

    /**
     * Default Constructor
     *
     * @param pId
     * @param sId
     */
    public PendingEvent(int bpuId,int eId, int sId, int sNb,String type) {
        this.onMessages = new Vector();
        this.alarms = new Vector();
        this.foundOnMessage = -1;
        this.id = eId;
        this.scopeId = sId;
        this.statementNb = sNb;
        this.bpuId = bpuId;
        this.type = type;
    }

    /**
     * @param nb
     * @param pl
     * @param pt
     * @param o
     * @param v
     */
    public void addOnMessage(int nb, String pl, String pt, String o, String v) {
        this.onMessages.add(new PendingOnMessage(nb, pl, pt, o, v));
    }

    /**
     * @param op
     * @param pt
     * @param pl
     * @return
     */
    public boolean equals(String op, String pt, String pl) {
        boolean found = false;
        this.foundOnMessage = -1;
        for (int i = 0; i < this.onMessages.size(); i++) {
            PendingOnMessage pom = (PendingOnMessage) this.onMessages.get(i);
            found = (pom.getOperation().equals(op)) && (pom.getPortType().equals(pt)) && (pom.getPartnerLink().equals(pl));
            if (found) {
                this.foundOnMessage = i;
                break;
            }
        }
        return found;
    }

    /**
     * @return
     */
    public int getFoundOnMessageBrancheNb() {
        if (this.foundOnMessage != -1) {
            PendingOnMessage pom = (PendingOnMessage) this.onMessages.get(this.foundOnMessage);
            return pom.getBrancheNb();
        }
        return -1;
    }

    /**
     * @return
     */
    public int getScopeId() {
        return this.scopeId;
    }

    /**
     * @return
     */
    public int getId() {
        return this.id;
    }

    /**
     * @return
     */
    public int getStamentNb() {
        return this.statementNb;
    }

    /**
     * @return
     */
    public int getBpuId() {
        return this.bpuId;
    }

    public void addAlarm(String type) {
        this.alarms.addElement(type);
    }

    public Vector getAlarms() {
        return this.alarms;
    }

    public String getType() {
        return this.type;
    }


}


