/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.util.Enumeration;
import java.util.Vector;

/**
 * RunningBPI.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: RunningBPI.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class RunningBPI extends Object {

    private int bpiId;
    private String processName;
    private boolean active;
    private Vector callsWaiting;

    /**
     * Default Constructor
     *
     * @param id
     *            the BPI Id
     * @param name
     *            the Process Name
     */
    public RunningBPI(int id, String name) {
        this.bpiId = id;
        this.processName = name;
        this.active = true;
        this.callsWaiting = new Vector();
    }

    /**
     * Passivate Running BPI
     */
    public void passivate() {
        this.active = false;
    }

    /**
     * Activate Running BPI
     */
    public void activate() {
        this.active = true;
    }

    /**
     * @return true if the Running BPI is Active
     */
    public boolean isActive() {
        return this.active;
    }

    /**
     * @return the BPIId
     */
    public int getBPIId() {
        return this.bpiId;
    }

    /**
     * @return The Process Name
     */
    public String getProcessName() {
        return this.processName;
    }

    /**
     * @param pt
     *            portType
     * @param op
     *            operation
     */
    public void addCallWaiting(String pt, String op) {
        String[] callWaiting = { pt, op};
        this.callsWaiting.add(callWaiting);
    }

    /**
     * @param pt
     *            portType
     * @param op
     *            Operation
     * @return true if the BPI was waiting for the specified call
     */
    public boolean isWaiting(String pt, String op) {
        Vector psTmp = (Vector) this.callsWaiting.clone();
        for (Enumeration elmts = psTmp.elements(); elmts.hasMoreElements();) {
            String[] cw = (String[]) elmts.nextElement();
            if (pt.equals(cw[0]) && op.equals(cw[1])) {
                this.callsWaiting.remove(cw);
                return true;
            }
        }
        return false;
    }
}
