/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Serializable;
import java.util.Vector;

import org.objectweb.orchestra.exception.BPELEngineException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.BPELString;



/**
 * WsdlBinding.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WsdlBinding.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WsdlBinding implements Serializable {

    private static String myPath = BPELConstants.WSDLBINDING_DIR;
    private Vector properties = null;

    /**
     * Default Constructor
     *
     * @param processName
     * @throws FileNotFoundException
     * @throws IOException
     */
    public WsdlBinding(String processName) throws BPELEngineException {
        if (myPath.startsWith("null")) { throw new BPELEngineException("ProcessMap: "+BPELConstants.BPEL_HOME_PROPERTY+" property is not set",null); }
        try {
            File f = new File(myPath);
            File[] wsdlList = f.listFiles();
            this.properties = new Vector();

            for (int i = 0; i < wsdlList.length; i++) {
                if (wsdlList[i].getName().equals(processName + ".txt")) {
                    FileReader fr = new FileReader(wsdlList[i]);
                    BufferedReader br = new BufferedReader(fr);
                    String line = br.readLine();
                    while (line != null) {
                        //System.out.println("Line : " + line);
                        String[] elements = line.split(" ");
                        WsdlBindingElement we = new WsdlBindingElement(elements);
                        this.properties.add(we);
                        line = br.readLine();
                    }
                    fr.close();
                    br.close();
                    break;
                }
            }
        } catch (FileNotFoundException fe) {
            throw new BPELEngineException("FileNotFoundException WsdlBinding WsdlBinding : \n" , fe);
        } catch (IOException e) {
            throw new BPELEngineException("IOException WsdlBinding WsdlBinding :\n" , e);
        }
    }

    /**
     * @param portType
     * @return
     */
    public String[] getService(String portType) {
        WsdlBindingElement temp = null;
        String[] result = new String[2];
        for (int i = 0; i < this.properties.size(); i++) {
            temp = ((WsdlBindingElement) this.properties.get(i));
            if (temp.getBindingType() != null && temp.getBindingType().equals(portType)) {
                result[0] = BPELString.removeNS(temp.getServiceName());
                result[1] = temp.getAddressLocation();
                break;
            }
        }
        return result;
    }

    /**
     * @param portType
     * @param operation
     * @return
     */
    public String[] getMessages(String portType, String operation) {
        WsdlBindingElement temp = null;
        String[] result = new String[2];
        for (int i = 0; i < this.properties.size(); i++) {
            temp = ((WsdlBindingElement) this.properties.get(i));
            if (temp.getPortTypeName() != null && temp.getPortTypeName().equals(portType) && temp.getOperationName().equals(operation)) {
                result[0] = BPELString.removeNS(temp.getInputMessageName());
                result[1] = temp.getOutputMessageName();
                break;
            }
        }
        return result;
    }

    public String getFaultName(String portType, String operation, String faultMessage) {
        //System.out.println("WSDLBinding : getFaultName : portType : " + portType + ", operation : " + operation + ", faultMessage : " + faultMessage);
        WsdlBindingElement tmp = null;
        String faultName = null;
        for (int i=0 ;i < this.properties.size() ; i++) {
            tmp = ((WsdlBindingElement) this.properties.get(i));
            if (tmp.getPortTypeName() != null && tmp.getPortTypeName().equals(portType) && tmp.getOperationName().equals(operation)) {
                faultName = tmp.getFaultName(faultMessage);
                break;
            }

        }
        //System.out.println("WSDLBinding : end getFaultName : faultName : " + faultName);
        return faultName;
    }

}