/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.util.Map;

import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;



/**
 * BPELAnalyser.java, analyse wsdl and bpel file representing the process
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELAnalyser.java 126 2007-01-30 08:45:59Z souillac $
 *
 */
public class BPELAnalyser {

    static String processName = null;

    /**
     * executes WSDLCompilo on wsdl file exposing the process executes
     * BPELCompilo on bpel file describing the process
     *
     * @param args
     *            the wsdl file uri and eventualy the bpel file uri
     * @return Boolean equal to true if the BPI is finished
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        if (args.length < 1 || args.length > 2) {
            throw new BPELGeneratorException("The BPELAnalyser should only be called with one or two arguments.");
        }

        String wsdl_file_name = args[0];
        System.out.println("Examinig wsdl file : " + wsdl_file_name);
        BPELCompilo BPCompilo = new BPELCompilo();
        //extracts the processname from the wsdl file uri
        processName = wsdl_file_name.substring(wsdl_file_name.lastIndexOf(File.separator) + 1, wsdl_file_name.indexOf(".wsdl"));
        //executes WSDLCompilo on wsdl file
        WSDLCompilo WSCompilo = new WSDLCompilo(processName);

        try {
            String bpel_file_name = null;
            String targetNs = null;
            Document bpelDocument =  null;
            if (args.length == 2) {
                bpel_file_name = args[1];
                bpelDocument = BPCompilo.getDocumentFromBpelFile(bpel_file_name);
                Node processNode = bpelDocument.getDocumentElement();
                targetNs = processNode.getAttributes().getNamedItem("targetNamespace").getNodeValue();
            }
            WSCompilo.analyse(wsdl_file_name, targetNs);
            //if a bpel file is given in param, execute BPELCompilo on it
            if (args.length == 2) {
                System.out.println("Examinig bpel file : " + bpel_file_name);
                SymbolTable symbolTable = WSCompilo.getSymbolTable();
                Map plTypes = WSCompilo.getPartnerLinkTypes();
                Wsdl wsdl = WSCompilo.getWsdlNSdef();
                BPCompilo.analyseDocument(bpelDocument, symbolTable, plTypes, wsdl);
            }
        } catch (BPELGeneratorException e) {
            e.printStackTrace();
        }
    }
}