package org.objectweb.orchestra.generator;

/**
 * Entity resolver allowing to find schema within the classloader.
 * @author Florent Benoit
 */
public class BpelEntityResolver extends SchemaEntityResolver {

    /**
     * XML schema xsd file name.
     */
    public static final String XML_SCHEMA = "XMLSchema.xsd";

    /**
     * XML schema ns.
     */
    public static final String XML_NS = "http://www.w3.org/2001/XMLSchema";
    public static final String XML_NS2 = "http://www.w3.org/2001/xml.xsd";

    /**
     * WSDL schema xsd file name.
     */
    public static final String WSDL_SCHEMA = "wsdl.xsd";

    /**
     * WSDL schema ns.
     */
    public static final String WSDL_NS = "http://schemas.xmlsoap.org/wsdl/";

    /**
     * BPEL 1.1 schema xsd file name.
     */
    public static final String BPEL_1_1_SCHEMA = "BPEL4WS-1.1.xsd";

    /**
     * BPEL 1.1 schema ns.
     */
    public static final String BPEL_1_1_NS = "http://schemas.xmlsoap.org/ws/2003/03/business-process/";

    /**
     * BPEL 2.0 executable schema xsd file name.
     */
    public static final String BPEL_2_0_EXECUTABLE_SCHEMA = "ws_bpel_2_0_executable.xsd";

    /**
     * BPEL 2.0 executable schema ns.
     */
    public static final String BPEL_2_0_EXECUTABLE_NS = "http://docs.oasis-open.org/wsbpel/2.0/process/executable";

    /**
     * BPEL 2.0 abstract schema xsd file name.
     */
    public static final String BPEL_2_0_ABSTRACT_SCHEMA = "ws_bpel_2_0_abstract.xsd";

    /**
     * BPEL 2.0 abstract schema ns.
     */
    public static final String BPEL_2_0_ABSTRACT_NS = "http://docs.oasis-open.org/wsbpel/2.0/process/abstract";

    /**
     * BPEL 2.0 plnkType schema xsd file name.
     */
    public static final String BPEL_2_0_PLNKTYPE_SCHEMA = "ws_bpel_2_0_plnkType.xsd";

    /**
     * BPEL 2.0 plnkType schema ns.
     */
    public static final String BPEL_2_0_PLNKTYPE_NS = "http://docs.oasis-open.org/wsbpel/2.0/plnktype";

    /**
     * BPEL 2.0 varprop schema xsd file name.
     */
    public static final String BPEL_2_0_VARPROP_SCHEMA = "ws_bpel_2_0_varprop.xsd";

    /**
     * BPEL 2.0 varprop chema ns.
     */
    public static final String BPEL_2_0_VARPROP_NS = "http://docs.oasis-open.org/wsbpel/2.0/varprop";


    /**
     * All schemas.
     */
    public static final String[] ALL_SCHEMAS = new String[] {
        XML_SCHEMA,
        XML_SCHEMA,
        WSDL_SCHEMA,
        BPEL_1_1_SCHEMA,
        BPEL_2_0_ABSTRACT_SCHEMA,
        BPEL_2_0_EXECUTABLE_SCHEMA,
        BPEL_2_0_PLNKTYPE_SCHEMA,
        BPEL_2_0_VARPROP_SCHEMA};

    /**
     * All ns.
     */
    public static final String[] ALL_NS = new String[] {
        XML_NS,
        XML_NS2,
        WSDL_NS,
        BPEL_1_1_NS,
        BPEL_2_0_ABSTRACT_NS,
        BPEL_2_0_EXECUTABLE_NS,
        BPEL_2_0_PLNKTYPE_NS,
        BPEL_2_0_VARPROP_NS};


    /**
     * Constructor. Finds the XSD with classloader.
     */
    public BpelEntityResolver() {
        super(ALL_SCHEMAS, ALL_NS);
    }

}



