/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.util.Vector;

/**
 * MessageCS.java, represents a correlationSet in a message
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MessageCS.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class MessageCS {

    private String mCSName = null;
    private String mCSPart = null;
    private String mCSQuery = null;
    private Vector mCSMapping = null;

    /**
     * Constructor
     *
     * @param name
     *            the property name, for example : "urn:ns,name", urn:ns is the
     *            namespace of the property and name is the name of the property
     * @param part
     *            the part of the message concerned by the correlationSet
     * @param query
     *            the query of the message concerned by the correlationSet
     */
    public MessageCS(String name, String part, String query, Vector mapping) {
        this.mCSName = name;
        this.mCSPart = part;
        this.mCSQuery = query;
        this.mCSMapping = mapping;
    }

    /**
     * print this MessageCS
     *
     */
    public void printMessageCS() {
        System.out.println("  Print Message CS");
        System.out.println("    Message CS name : " + this.mCSName);
        System.out.println("    Message CS part : " + this.mCSPart);
        System.out.println("    Message CS query : " + this.mCSQuery);
    }

    /**
     * get the propertyName of this MessageCS
     *
     * @return String the propertyName
     */
    public String getCSName() {
        return this.mCSName;
    }

    /**
     * get the message part of this MessageCS
     *
     * @return String the message part
     */
    public String getCSPart() {
        return this.mCSPart;
    }

    /**
     * get the message query of this MessageCS
     *
     * @return String the message query
     */
    public String getCSQuery() {
        return this.mCSQuery;
    }

    public Vector getCSMapping() {
        return this.mCSMapping;
    }

}