/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.util.Vector;

import javax.xml.namespace.QName;

import org.objectweb.orchestra.exception.BPELGeneratorException;


/**
 * MessageType.java, represents a message
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MessageType.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class MessageType {

    private String mProcessName = null;
    private String mMessageName = null;
    private Vector mParts = null;
    private Vector mCS = null;

    /**
     * Constructor
     *
     * @param process
     *            the processName
     * @param name
     *            the message name : for example : "urn:ns,mess1"
     */
    public MessageType(String process, String name) {
        this.mProcessName = process;
        this.mMessageName = name;
        this.mParts = new Vector();
        this.mCS = new Vector();
    }

    /**
     * print this MessageType
     *
     */
    public void printMessageType() {
        System.out.println("Print Message Type");
        System.out.println("  Message Type name : " + this.mMessageName);
        for (int i = 0; i < this.mParts.size(); i++) {
            ((MessagePart) this.mParts.get(i)).printMessagePart();
        }
        for (int i = 0; i < this.mCS.size(); i++) {
            ((MessageCS) this.mCS.get(i)).printMessageCS();
        }
    }

    /**
     * get the message name of this MessageType
     *
     * @return String the message name
     */
    public String getMessageName() {
        return this.mMessageName;
    }

    /**
     * add a part to this message
     *
     * @param name
     *            the name of the part
     * @param type
     *            the type of the part
     * @param element
     *            the element of the part
     */
    public void addPart(String name, String type, QName element) {
        MessagePart part = new MessagePart(name, type, element);
        this.mParts.add(part);
    }

    /**
     * get the part type of a part of this message based on the part name
     *
     * @param partName
     *            the name of the part we want to get the type
     */
    public String getPartType(String partName) {
        int i = 0;
        while ((i < this.mParts.size()) && !(((MessagePart) this.mParts.get(i)).getPartName().equals(partName))) {
            i++;
        }
        if (i < this.mParts.size()) {
            return ((MessagePart) this.mParts.get(i)).getPartType();
        }
        return null;

    }

    /**
     * get all parts of this message
     *
     * @return Vector the vector repersenting all parts of this message
     */
    public Vector getParts() {
        return this.mParts;
    }

    /**
     * add a correlationSet to this message
     *
     * @param name
     *            the name of the preoperty of this correlationSet
     * @param part
     *            the name of the part of the message concerned by the
     *            correlationSet
     * @param query
     *            the query of the message concerned by the correlationSet
     */
    public void addCS(String name, String part, String query, Vector mapping) {
        MessageCS cs = new MessageCS(name, part, query, mapping);
        this.mCS.add(cs);
    }

    /**
     * print a BPELMessageFile for this message
     *
     * @param wsdl
     *            the wsdl object giving the namespaces mapping
     */
    public void printBPELFile() throws BPELGeneratorException {
        BPELMessageFile file = new BPELMessageFile(this.mProcessName, this.mMessageName, this.mParts, this.mCS);
        file.print();
    }

    /**
     * print a Factory implementing JXpath factory for this message
     *
     */
    public void printXPathFactory() throws BPELGeneratorException {
        JXPathFactory file = new JXPathFactory(this.mProcessName, this.mMessageName, this.mParts);
        file.print();
    }

}