/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Fault;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Output;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

import org.apache.axis.utils.WSDLUtils;
import org.apache.axis.wsdl.symbolTable.SymbolTable;
import org.apache.axis.wsdl.toJava.Emitter;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.exception.BPELUtilException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.UDDIDirectory;
import org.objectweb.orchestra.util.Util;

import com.ibm.wsdl.OperationImpl;
import com.ibm.wsdl.PartImpl;
import com.ibm.wsdl.factory.WSDLFactoryImpl;

/**
 * OutServiceCompilo.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: OutServiceCompilo.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class OutServiceCompilo {

    private String processName = null;

    public OutServiceCompilo(String processName) {
        this.processName = processName;
    }

    public void analyseWsdl(String wsdlFile) throws BPELGeneratorException {
        Definition def = null;
        WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
        WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
        try {
            def = wsdlReader.readWSDL(wsdlFile);
        } catch (javax.wsdl.WSDLException e) {
            throw new BPELGeneratorException("BPELGeneratorException OutServiceCompilo analyseWsdl :\n" , e);
        }
        analyseServices(def);
    }

    /**
     * Analyse 'service' tag of wsdl file
     *
     */
    private void analyseServices(Definition def) throws BPELGeneratorException {
        Map serviceMap = def.getServices();
        Set serviceSet = serviceMap.entrySet();
        for (Iterator i = serviceSet.iterator(); i.hasNext();) {
            Map.Entry entry0 = (Map.Entry) i.next();
            Service service = (Service) entry0.getValue();
            QName serviceQName = service.getQName();
            Map portMap = service.getPorts();
            Set portSet = portMap.entrySet();
            for (Iterator j = portSet.iterator(); j.hasNext();) {
                Map.Entry entry1 = (Map.Entry) j.next();
                Port port = (Port) entry1.getValue();
                String portName = port.getName();
                Binding binding = port.getBinding();
                QName bindingQName = binding.getQName();
                PortType portType = binding.getPortType();
                QName portTypeQName = portType.getQName();
                String addressLocation = WSDLUtils.getAddressFromPort(port);
                try {
                    UDDIDirectory.addService(this.processName,serviceQName, portName, bindingQName, addressLocation, portTypeQName);
                } catch (BPELUtilException e) {
                    throw new BPELGeneratorException("BPELUtilException caught in OutServiceCompilo.analyseServices : ",e);
                }

                Util.buildStringFromQName(portTypeQName);
                new StubServiceFile(this.processName, def.getTargetNamespace(), bindingQName.getLocalPart(),BPELConstants.WSDL_DIR);

                analyseMessages(portType, getSymbolTable(def.getDocumentBaseURI()));
            }
        }
    }

    private void analyseMessages(PortType portType, SymbolTable symbolTable) throws BPELGeneratorException {
        List operationsL = portType.getOperations();
        for (Iterator j = operationsL.iterator(); j.hasNext();) {
            OperationImpl op = (OperationImpl) j.next();
            Input in = op.getInput();
            if (in != null) {
                Message mIn = in.getMessage();
                analyseMessage(mIn, symbolTable);
            }
            Output out = op.getOutput();
            if (out != null) {
                Message mOut = out.getMessage();
                analyseMessage(mOut, symbolTable);
            }
            Map faultsMap = op.getFaults();
            Set faultsSet = faultsMap.entrySet();
            for (Iterator k = faultsSet.iterator(); k.hasNext();) {
                Map.Entry entry1 = (Map.Entry) k.next();
                Fault fault = (Fault) entry1.getValue();
                Message faultMessage = fault.getMessage();
                analyseMessage(faultMessage, symbolTable);
            }
        }
    }

    private void printBPELMessage(MessageType mt) throws BPELGeneratorException {
        mt.printBPELFile();
        mt.printXPathFactory();
    }

    public SymbolTable getSymbolTable(String file) throws BPELGeneratorException {
        try {
            Emitter emit = Util.buildEmitter(file);
            emit.setVerbose(false);
            return emit.getSymbolTable();
        } catch (BPELUtilException ex) {
            throw new BPELGeneratorException("BPELUtilException WSDLCompilo getTypesMapping :\n" , ex);
        }
    }

    private void analyseMessage(Message m, SymbolTable symbolTable) throws BPELGeneratorException {
        String messageName = Util.buildStringFromQName(m.getQName());
        MessageType mess = new MessageType(this.processName, messageName);
        List parts = m.getOrderedParts(null);
        for (Iterator j = parts.iterator(); j.hasNext();) {
            PartImpl part = (PartImpl) j.next();
            String partName = part.getName();
            String partType = null;
            QName partElementQN = part.getElementName();
            partType = Util.getJavaType(symbolTable, part.getTypeName(), part.getElementName());
            mess.addPart(partName, partType, partElementQN);
        }
        printBPELMessage(mess);
    }
}
