/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.util.Vector;

import javax.xml.namespace.QName;

/**
 * PartnerLinkType.java, represents a partnerLinkType tag of the wsdl file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PartnerLinkType.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class PartnerLinkType {

    private Vector roles = null;
    private QName name = null;

    /**
     * Default Constructor
     *
     * @param name
     *            the partnerLinkType name
     */
    public PartnerLinkType(QName name) {
        this.name = name;
        this.roles = new Vector();
    }

    /**
     * Add a role
     *
     * @param role
     *            the role to add
     */
    public void addRole(Role role) {
        this.roles.add(role);
    }

    /**
     * get a role based on its name
     *
     * @param roleName
     *            the role name of the role searched
     */
    public Role getRole(String roleName) {
        Role r = null;
        for (int i = 0; i < this.roles.size(); i++) {
            if (((Role) this.roles.get(i)).getName().equals(roleName)) {
                r = (Role) this.roles.get(i);
            }
        }
        return r;
    }

    /**
     * get all roles of this partnerLinkType
     */
    public Vector getRoles() {
        return this.roles;
    }

    /**
     * get the name of this partnerLinkType
     */
    public QName getName() {
        return this.name;
    }

    /**
     * get a string representation of this partnerLinkType
     *
     * @return String a string representation of this partnerLinkType
     */
    public String toString() {
        String st = "PartnerLinkType : \n";
        st += "name = " + this.name + "\n";
        for (int i = 0; i < this.roles.size(); i++) {
            st += this.roles.get(i) + "\n";
        }
        return st;
    }
}