/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * PartnerLinkTypeFile.java, writes a file giving the mapping between a property
 * name and its value (declared in wsdl file)
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PartnerLinkTypeFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class PartnerLinkTypeFile {

    private static String myPath = BPELConstants.PLTYPES_DIR;
    private FileWriter fileWriter = null;
    private File f = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     */
    public PartnerLinkTypeFile(String process) throws BPELGeneratorException {
        try {
            String fileName = myPath + File.separator + process + ".txt";
            this.f = new File(fileName);
            if (!this.f.exists()) {
                this.f.getAbsoluteFile().getParentFile().mkdirs();
            }
            this.fileWriter = new FileWriter(this.f,true);
        } catch (Exception e) {
            throw new BPELGeneratorException("PartnerLinkTypeFile.init : ",e);
        }

    }

    /**
     * insert a line in the file
     *
     * @param PartnerLinkType
     *            the partnerLinkType to insert
     */
    public void insertLine(PartnerLinkType p) throws BPELGeneratorException {
        try {
            String st = "";
            Role r = null;
            st += Util.buildStringFromQName(p.getName());
            for (int i = 0; i < p.getRoles().size(); i++) {
                r = (Role) p.getRoles().get(i);
                st += " ";
                st += r.getName();
                st += " ";
                st += Util.buildStringFromQName(r.getPortType());
            }
            st += "\n";
            this.fileWriter.write(st);

        } catch (Exception e) {
            throw new BPELGeneratorException("PartnerLinkTypeFile.insertLine : ",e);
        }
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("PartnerLinkTypeFile.finish : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

}