/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;

/**
 * PortWsdl.java, represents a partnerLinkType role tag of the wsdl file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: PortWsdl.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class PortWsdl {

    private String portName = null;
    private String wsdlLocation = null;

    /**
     * Default Constructor
     *
     * @param name
     *            the port name
     * @param wsdl
     *            the wsdl location
     */
    public PortWsdl(String name, String wsdl) {
        this.portName = name;
        wsdl = wsdl.replace('/', File.separatorChar);
        wsdl = wsdl.replace('\\', File.separatorChar);
        this.wsdlLocation = wsdl;
    }

    /**
     * get the port name
     *
     * @return String the port name
     */
    public String getPortName() {
        return this.portName;
    }

    /**
     * get the wsdl location
     *
     * @return String the wsdl location
     */
    public String getWsdlLocation() {
        return this.wsdlLocation;
    }

    /**
     * get a string representation of this PortWsdl
     *
     * @return String a string representation of this PortWsdl
     */
    public String toString() {
        String st = "PortWsdl : \n";
        st += "portName = " + this.portName + "\n";
        st += "wsdlLocation = " + this.wsdlLocation + "\n";
        return st;
    }

}