/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * ProcessFile.java, write a file giving all attributes of process tag in bpel
 * file
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ProcessFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ProcessFile {

    private static String myPath = BPELConstants.PROCESS_DIR;
    private FileWriter fileWriter = null;
    private String fileName = null;
    private String processName = null;
    private String f = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     */
    public ProcessFile(String process) throws BPELGeneratorException {

        this.processName = process;
        this.fileName = this.processName;

        try {
            this.f = myPath + File.separator + this.fileName + ".txt";
            //System.out.println("File : " + f);
            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessFile.init : ",e);
        }
    }

    /**
     * insert an element in the file
     *
     * @param el
     *            the processElement to insert
     */
    public void insertElement(String property, String value) throws BPELGeneratorException {
        insertLine(property, value);
    }

    /**
     * insert a line in the file
     *
     * @param property
     *            the property to insert
     * @param value
     *            the value of the property to insert
     */
    public void insertLine(String property, String value) throws BPELGeneratorException {
        try {
            this.fileWriter.write(property + "=" + value + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessFile.init : ",e);
        }
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessFile.init : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

}