/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

import java.io.File;
import java.io.FileWriter;

import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * ProcessMapFile.java, writes a file to save information needed to associate
 * receive and onMessage activities to their correlationsSets
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ProcessMapFile.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class ProcessMapFile {

    // the file will have one line for each receive/onMessage in the BPEL File
    // Here is the elements of that line :
    // type processName partnerLink portType operation (createInstance)
    // (correlationSetProperties list)

    private static String myPath = BPELConstants.PROCESSMAP_DIR;
    private FileWriter fileWriter = null;
    private String fileName = null;
    private String processName = null;
    private String f = null;

    /**
     * constructor
     *
     * @param process
     *            the name of the process
     */
    public ProcessMapFile(String process) throws BPELGeneratorException {

        this.processName = process;
        this.fileName = this.processName;

        try {
            this.f = myPath + File.separator + this.fileName + ".txt";
            //System.out.println("File : " + f);
            new File(this.f).getAbsoluteFile().getParentFile().mkdirs();
            this.fileWriter = new FileWriter(this.f);
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessMapFile.init : ",e);
        }

    }

    /**
     * insert a line in the file
     *
     * @param receiveInfo
     *            information needed : type (receive, onMessage), partnerLink,
     *            portType, operation, createInstance
     * @param cs
     */
    public void insertLine(String[] receiveInfo, CorrelationSet cs) throws BPELGeneratorException {
        try {
            //System.out.println("print : " + receiveInfo[0] + " " +
            // processName + " " + receiveInfo[1] + " " + receiveInfo[2] + " " +
            //	       receiveInfo[3] + " " + receiveInfo[4] + " " + cs);
            String[] correlationSetProperties = cs.getProperties();
            String properties = "";
            for (int i = 0; i < correlationSetProperties.length; i++)
                properties = properties + correlationSetProperties[i] + " ";
            this.fileWriter.write(receiveInfo[0] + " " + this.processName + " " + receiveInfo[1] + " " + receiveInfo[2] + " " + receiveInfo[3] + " " + receiveInfo[4] + " " + receiveInfo[5] + " " + cs.getName() + " " + properties + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessMapFile.insertLine(receiveInfo + cs) : ",e);
        }
    }

    /**
     * insert a line in the file
     *
     * @param receiveInfo
     *            information needed : type (receive, onMessage), partnerLink,
     *            portType, operation, createInstance
     */
    public void insertLine(String[] receiveInfo) throws BPELGeneratorException {
        try {
            //System.out.println("print : " + receiveInfo[0] + " " +
            // processName + " " + receiveInfo[1] + " " + receiveInfo[2] + " " +
            //	       receiveInfo[3] + " " + receiveInfo[4]);
            this.fileWriter.write(receiveInfo[0] + " " + this.processName + " " + receiveInfo[1] + " " + receiveInfo[2] + " " + receiveInfo[3] + " " + receiveInfo[4] + " " + receiveInfo[5] + "\n");
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessMapFile.insertLine(receiveInfo) : ",e);
        }
    }

    /**
     * write the last instructions of the file
     *
     */
    public void finish() throws BPELGeneratorException {
        try {
            this.fileWriter.close();
        } catch (Exception e) {
            throw new BPELGeneratorException("ProcessMapFile.finish : ",e);
        }
        System.out.println("File " + this.f + " created");
    }

}