/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.generator;

/**
 * WsdlElement.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: WsdlElement.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class WsdlElement implements java.io.Serializable {

    private String type = null;
    private String name = null;
    private String targetNamespace = null;
    private String prefix = null;
    private String namespaceURI = null;

    /**
     * Default Constructor
     *
     * @param properties
     *            the property and its value
     * @throws Exception
     */
    public WsdlElement(String[] properties) throws Exception {
        if (properties[0].startsWith("name")) {
            this.type = "name";
            this.name = properties[0].substring(properties[0].indexOf("=") + 1);
        } else if (properties[0].startsWith("targetNamespace")) {
            this.type = "targetNamespace";
            this.targetNamespace = properties[0].substring(properties[0].indexOf("=") + 1);
        } else {
            //xmlns
            this.type = "xmlns";
            this.prefix = properties[0].substring(0, properties[0].indexOf("="));
            this.namespaceURI = properties[0].substring(properties[0].indexOf("=") + 1);
        }
    }

    /**
     * get the type of this WsdlElement : name, targetNamespace or xmlns
     *
     * @return String the type
     */
    public String getType() {
        return this.type;
    }

    /**
     * get the name of this WsdlElement (attribute name in definitions tag in
     * wsdl file)
     *
     * @return String the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * get the targetNamespace of this WsdlElement (attribute targetNamespace in
     * definitions tag in wsdl file)
     *
     * @return String the targetNamespace
     */
    public String getTargetNamespace() {
        return this.targetNamespace;
    }

    /**
     * get the prefix of this WsdlElement (xmlns:prefix=namespaceURI in
     * definitions tag in wsdl file)
     *
     * @return String the prefix
     */
    public String getPrefix() {
        return this.prefix;
    }

    /**
     * get the namespaceURI of this WsdlElement (xmlns:prefix=namespaceURI in
     * definitions tag in wsdl file)
     *
     * @return String the namespaceURI
     */
    public String getNamespaceURI() {
        return this.namespaceURI;
    }
}