/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.monitoring;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Document;

/**
 * MonitoringMap.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MonitoringMap.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class MonitoringMap {

    private static HashMap instancesPool = null;
    private static HashMap variablesPool = null;
    private static HashMap instances = null;
    private static HashMap variables = null;
    private static int maxPoolSize = 20;

    public static synchronized Document getInstance(int bpiId) {
        Document result = null;
        init();
        Integer integ = new Integer(bpiId);
        if (instances.containsKey(integ)) {
            result = (Document) instances.get(integ);
            instancesPool.put(integ,new Date());
        }
        return result;
    }

    public static synchronized Document getVariable(int bpiId) {
        Document result = null;
        init();
        Integer integ = new Integer(bpiId);
        if (variables.containsKey(integ)) {
            result = (Document) variables.get(integ);
            variablesPool.put(integ,new Date());
        }
        return result;
    }

    public static synchronized void addInstance(int bpiId,Document doc) {
        init();
        Integer integ = new Integer(bpiId);
        if (instances.containsKey(integ)) {
            instances.remove(integ);
        }
        if (instances.size() == maxPoolSize) {
            Integer lastBpiAccessed = getMinDate(instancesPool);
            instances.remove(lastBpiAccessed);
            instancesPool.remove(lastBpiAccessed);
        }
        instances.put(integ,doc);
        instancesPool.put(integ,new Date());
    }

    public static synchronized void addVariable(int bpiId,Document doc) {
        init();
        Integer integ = new Integer(bpiId);
        if (variables.containsKey(integ)) {
            variables.remove(integ);
        }
        if (variables.size() == maxPoolSize) {
            Integer lastBpiAccessed = getMinDate(variablesPool);
            variables.remove(lastBpiAccessed);
            variablesPool.remove(lastBpiAccessed);
        }
        variables.put(integ,doc);
        variablesPool.put(integ,new Date());
    }

    private static Integer getMinDate(HashMap h) {
        Integer result = null;
        Date min = new Date();
        Set s = h.entrySet();
        for (Iterator i = s.iterator(); i.hasNext();) {
            Map.Entry e = (Map.Entry) i.next();
            Date d = (Date) e.getValue();
            if (d.before(min)) {
                result = (Integer) e.getKey();
            }
        }
        return result;
    }

    private static void init() {
        if (instances == null) {
            instances = new HashMap();
            variables = new HashMap();
            instancesPool = new HashMap();
            variablesPool = new HashMap();
        }
    }
}


