/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.monitoring;

import java.io.Serializable;
import java.util.Vector;


/**
 * MonitoringStatement.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: MonitoringStatement.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class MonitoringStatement implements Serializable {

    private int bpuId;
    private int statementNb;
    private long timeStamp;
    private String type;
    private int executionNb;
    private Vector args;

    public MonitoringStatement() {
        //empty constructor
    }

    /**
     * Default Constructor
     * @param bpuId the bpuId
     * @param statementNb the statementNb
     * @param timeStamp the statementNb execution date
     */
    public MonitoringStatement(int bpuId,String[] split) {
        this.bpuId = bpuId;
        this.type = split[0];
        this.statementNb = Integer.parseInt(split[1]);
        this.timeStamp = Long.parseLong(split[2]);
        this.args = new Vector();
        for (int i=3;i+1<split.length;i++) {
            this.args.addElement(new MonitoringArg(split[i],split[i+1]));
        }
    }

    public String toString() {
        String end = "\n";
        String st = "MonitoringStatement"+end;
        st += "  - statementNb = "+this.statementNb+end;
        st += "  - type        = "+this.type+end;
        st += "  - bpuId       = "+this.bpuId+end;
        st += "  - timeStamp   = "+this.timeStamp+end;
        st += "  - executionNb = "+this.executionNb+end;
        for (int i = 0 ; i< this.args.size();i++) {
            MonitoringArg ma = (MonitoringArg)this.args.get(i);
            st += "  - arg"+i+".name  = "+ma.getName();
            st += "  - arg"+i+".value = "+ma.getValue();
        }
        return st;
    }
    /**
     * get the bpuId
     *
     * @return int the bpuId
     */
    public int getBpuId() {
        return this.bpuId;
    }

    /**
     * get the statementNb
     *
     * @return int the statementNb
     */
    public int getStatementNb() {
        return this.statementNb;
    }

    /**
     * get the timeStamp
     *
     * @return long the timeStamp
     */
    public long getTimeStamp() {
        return this.timeStamp;
    }

    /**
     * get the associated statement
     *
     * @return Statement the statement
     */
    public int getExecutionNb() {
        return this.executionNb;
    }

    public String getType() {
        return this.type;
    }

    public Vector getArgs() {
        return this.args;
    }

    public void setExecutionNb(int executionNb) {
        this.executionNb = executionNb;
    }

    public void setTimeStamp(long timeStamp) {
        this.timeStamp = timeStamp;
    }

    public void setBpuId(int bpuId) {
        this.bpuId = bpuId;
    }

    public void setStatementNb(int statementNb) {
        this.statementNb = statementNb;
    }

    public void setType(String type) {
        this.type = type;
    }

    public void setArgs(Vector args) {
        this.args = args;
    }
}


