/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
/**
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 **/
package org.objectweb.orchestra.resource;

import java.util.ArrayList;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

/**
 * JmxRegisterResourceAdapter.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: JmxRegisterResourceAdapter.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class JmxRegisterResourceAdapter implements ResourceAdapter {

    /** Instance to be registered in the MBean Server */
    private BPELService object = null;

    /** Class to be instantiated. Specified in ra.xml. */
    private String objectClass = null;

    /** Object Name used during registration. Specified in ra.xml. */
    private String objectName = null;

    /** Reference to the MBean Server in which to register the instance */
    private MBeanServer server = null;


    public void endpointActivation(MessageEndpointFactory arg0, ActivationSpec arg1) throws ResourceException {
        //
    }

    public void endpointDeactivation(MessageEndpointFactory arg0, ActivationSpec arg1) {
        //
    }

    /**
     * Retrieves the class to be instantiated
     *
     * @return String representing the class to be instantiated
     */
    public String getObjectClass() {
        return this.objectClass;
    }

    /**
     * Retrieves the Object Name to be used during registration in the MBean
     * Server
     *
     * @return String representing the Object Name to be used during
     *         registration in the MBean Server
     */
    public String getObjectName() {
        return this.objectName;
    }

    public XAResource[] getXAResources(ActivationSpec[] arg0) throws ResourceException {
        return null;
    }

    /**
     * Default constructor.
     * Gets a reference to the first MBean Server instantiated
     */
    public JmxRegisterResourceAdapter() {
        ArrayList servers = MBeanServerFactory.findMBeanServer(null);
        this.server = (MBeanServer) servers.get(0);
    }

    /**
     * Sets the class to be instantiated
     *
     * @param objectClass String representing the class to be instantiated
     */
    public void setObjectClass(String objectClass) {
        this.objectClass = objectClass;
    }

    /**
     * Sets the Object Name to be used during registration in the MBean
     * Server
     *
     * @param objectName representing the Object Name to be used during
     *                   registration in the MBean Server
     */
    public void setObjectName(String objectName) {
        this.objectName = objectName;
    }

    public void start(BootstrapContext arg0) throws ResourceAdapterInternalException {
        try
        {
            this.object = (BPELService) Class.forName(this.objectClass).newInstance();
            this.server.registerMBean(this.object, new ObjectName(getObjectName()));
            this.object.startService();

            //log.info("The following Bonita MBean is registered : " + this.object.getClass().getName());
        } catch(Exception e) {
            throw new ResourceAdapterInternalException("Error while starting the Resource Adapter : " + e.getMessage());
        }
    }

    public void stop() {
        try {
            this.object.stopService();
            this.server.unregisterMBean(new ObjectName(getObjectName()));
            this.object = null;

            //log.info("The following Bonita MBean is unregistered : " + this.object.getClass().getName());
        } catch(Exception e) {
            System.err.println("An exception occured while stopping the Resource Adapter : " + e);
        }
    }
}