/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.resource;

import java.io.Serializable;
import java.util.HashMap;

import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.util.BPELConstants;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;


/**
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: OrchestraEngineInitRAR.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class OrchestraEngineInitRAR implements javax.resource.spi.ResourceAdapter, Serializable {

    /**
     * EndpointFactories
     */
    private transient HashMap endpointFactories = new HashMap();

    private static int mode = BPELConstants.WITH_EJB;
    private static String monitor = BPELConstants.MONITOR_ALL;
    private static int jbi = BPELConstants.JBI_OFF;

    /**
     * Start
     * @param bootstrapCtx bootstrapCtx
     * @throws ResourceAdapterInternalException if an error occurs
     */
    public void start(BootstrapContext bootstrapCtx) throws ResourceAdapterInternalException {
        //System.out.println("EngineInitRA start");

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        DocumentBuilder docBuilder;
        try {
            docBuilder = dbf.newDocumentBuilder();
            document = docBuilder.parse(BPELConstants.BPEL_CONF_FILE);
        } catch (Exception e) {
            throw new ResourceAdapterInternalException(e);
        }
        analyseChilds(document);
        BPELEngine.setMode(mode);
        BPELEngine.setMonitor(monitor);
        BPELEngine.setJbi(jbi);
        //System.out.println("EngineInitRA end start");
    }

    private static void analyseNode(Node n) {
        //System.out.println("EngineInitRA analyseNode");
        int type = n.getNodeType();

        switch (type) {
        case Node.ELEMENT_NODE:
            if (n.getNodeName().equals("EngineMode")) {
                //System.out.println("EngineInitRA analyseNode found mode");
                NamedNodeMap atts = n.getAttributes();
                Node att = null;
                for (int i = 0; i < atts.getLength(); i++) {
                    att = atts.item(i);
                    if (att.getNodeName().equals("mode")) {
                        String fileMode = new String(att.getNodeValue());
                        //System.out.println("EngineInitRA analyseNode found mode : mode = " + fileMode);
                        if (fileMode.equals("DB")) {
                            mode = BPELConstants.WITH_EJB;
                        } else {
                            mode = BPELConstants.WITHOUT_EJB;
                        }
                        //System.out.println("EngineInitRA analyseNode found mode : mode = " + mode);
                        break;
                    }
                }
            } else if (n.getNodeName().equals("Monitor")) {
                NamedNodeMap atts = n.getAttributes();
                Node att = null;
                for (int i = 0; i < atts.getLength(); i++) {
                    att = atts.item(i);
                    if (att.getNodeName().equals("mode")) {
                        String attr = att.getNodeValue();
                        if (attr.equals(BPELConstants.MONITOR_ALL)) {
                            monitor = BPELConstants.MONITOR_ALL;
                        } else if (attr.equals(BPELConstants.MONITOR_RUNNING_AND_MESSAGES)) {
                            monitor = BPELConstants.MONITOR_RUNNING_AND_MESSAGES;
                        } else if (attr.equals(BPELConstants.MONITOR_MESSAGES)) {
                            monitor = BPELConstants.MONITOR_MESSAGES;
                        } else if (attr.equals(BPELConstants.MONITOR_RUNNING)) {
                            monitor = BPELConstants.MONITOR_RUNNING;
                        } else if (attr.equals(BPELConstants.MONITOR_NOTHING)) {
                            monitor = BPELConstants.MONITOR_NOTHING;
                        }
                        break;
                    }
                }
            } else if (n.getNodeName().equals("Jbi")) {
                NamedNodeMap atts = n.getAttributes();
                Node att = null;
                for (int i = 0; i < atts.getLength(); i++) {
                    att = atts.item(i);
                    if (att.getNodeName().equals("mode")) {
                        String jbiMode = new String(att.getNodeValue());
                        if (jbiMode.equals("on")) {
                            jbi = BPELConstants.JBI_ON;
                        } else {
                            jbi = BPELConstants.JBI_OFF;
                        }
                        break;
                    }
                }
            } else {
                analyseChilds(n);
            }
            break;
        }
    }

    private static void analyseChilds(Node n) {
        //System.out.println("EngineInitRA analyseChilds");
        for (Node child = n.getFirstChild(); child != null; child = child.getNextSibling()) {
            analyseNode(child);
        }
    }

    /**
     * Stop
     */
    public void stop() {
        // Nothing here
    }

    /**
     * Called by the application server when a message-driven bean
     * (MessageEndpoint) is deployed.
     * @param factory MessageEndpointFactory
     * @param spec activation spec
     * @throws NotSupportedException if an error occurs
     * @throws ResourceAdapterInternalException if an error occurs
     */
    public void endpointActivation(MessageEndpointFactory factory, ActivationSpec spec) throws ResourceAdapterInternalException {
        this.endpointFactories.put(spec, factory);
    }

    /**
     * Endpoint Deactivation
     * @param factory MessageEndpointFactory
     * @param spec activation spec
     */
    public void endpointDeactivation(MessageEndpointFactory factory, ActivationSpec spec) {
        this.endpointFactories.remove(spec);
    }

    /**
     * Get XAResources
     * @param specs A tab of specs
     * @throws ResourceException if an error occurs
     * @return Tab of XAResources
     */
    public XAResource[] getXAResources(ActivationSpec[] specs) throws ResourceException {
        return null;
    }

}
