/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import org.apache.tools.ant.BuildException;
import org.objectweb.orchestra.engine.ProcessMap;
import org.objectweb.orchestra.util.BPELMap;
import org.objectweb.orchestra.util.UDDIDirectory;
import org.objectweb.orchestra.util.WSDLUtilMap;



/**
 * CleanProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CleanProcessTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class CleanProcessTask extends StandardDeployTask {

    public CleanProcessTask() {
        setTaskName("cleanProcess");
    }


    public void execute() throws BuildException {
        log("Starting CleanProcessTask...");
        deleteDirs();
        deleteFiles();
        removeFromUddi();
        removeFromMaps();
        log("Finishing CleanProcessTask...");
    }

    public void deleteDirs() {
        deleteProcessDir("build_classes_dir");
        deleteProcessDir("build_tmp_dir");
        deleteProcessDir("build_JXPath_dir");
        deleteProcessDir("generate_wsdl_dir");
        deleteProcessDir("generate_generator_dir");
        deleteProcessDir("build_process_files_dir");
        deleteProcessDir("build_xml_dir");
    }

    public void deleteFiles() {
        deleteFile(getJarFile("build_jars_dir"));
        deleteFile(getJarFile("build_clientJars_dir"));
        deleteFile(getJarFile("build_wsdd_dir"));
        deleteFile(getTxtFile("generate_process_dir"));
        deleteFile(getTxtFile("generate_processmap_dir"));
        deleteFile(getTxtFile("generate_wsdlfile_dir"));
        deleteFile(getTxtFile("generate_wsdlbindingfile_dir"));
        deleteFile(getTxtFile("generate_outServices_dir"));
        deleteFile(getTxtFile("generate_plTypes_dir"));
        deleteFile(getTxtFile("generate_complexTypes_dir"));
        deleteFile(getProp("build_wsdd_dir")+this.sep+getProcessName()+"deploy.wsdd");
        deleteFile(getProp("build_wsdd_dir")+this.sep+getProcessName()+"undeploy.wsdd");
    }

    public void removeFromUddi() throws BuildException {
        try {
            UDDIDirectory.removeProcess(getProcessName());
        } catch (Exception e) {
            throw new BuildException("Exception caught CleanProcessTask removeFromUddi : \n"+e,e);
        }
    }

    public void removeFromMaps() {
        ProcessMap.removeLoadedProcess(getProcessName());
        WSDLUtilMap.removeWSDLUtil(getProcessName());
        BPELMap.removeProcess(getProcessName());
    }
}