/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.util.Vector;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.ConditionTask;
import org.apache.tools.ant.taskdefs.Exit;
import org.apache.tools.ant.taskdefs.condition.Socket;
import org.objectweb.orchestra.engine.BPELEngine;


/**
 * CompileProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: CompileProcessTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class CompileProcessTask extends StandardDeployTask {

    private Vector tasks = null;
    private boolean skipwsdl = false;
    private boolean jbi = false;

    public CompileProcessTask(boolean skipwsdl,String[] outServices) {
        this.tasks = new Vector();
        setTaskName("compileProcess");
        this.skipwsdl = skipwsdl;
        if (BPELEngine.isJbi() && outServices != null && outServices.length>0) {
            this.jbi = true;
        }
        if (getTaskWithName("wsdl2javaProcess") == null) {
            //here we are sure that this task is called from the java may
            //not by ant
            if (!skipwsdl) {
                addTask(this.tasks, new Wsdl2JavaProcessTask(true));
            }
            if (this.jbi) {
                addTask(this.tasks, new JbiWSTask(outServices));
            }
            addTask(this.tasks, new AnalyseProcessFilesTask());
            addTask(this.tasks, new CompileGeneratedFilesTask());
        }
    }

    public void execute() {
        log("Starting CompileProcessTask...");
        verifyServer();
        createDirs();
        createProperties();
        this.tasks = organizeTasks();
        for (int i = 0; i < this.tasks.size(); i++) {
            StandardDeployTask task = (StandardDeployTask) this.tasks.get(i);
            task.setProcessName(getProcessName());
            ((Task) task).setProject(getProject());
            if (task.getTaskName().equals("wsdl2javaProcess")) {
                ((Wsdl2JavaProcessTask) task).setWsdlFile(getProp(getProcessName() + "_wsdl_file"));
                ((Wsdl2JavaProcessTask) task).setOutput(getProp("generate_wsdl_dir") + this.sep + getProcessName());
            }
            ((Task) task).execute();
        }
        log("Finishing CompileProcessTask...");
    }

    public void createProperties() {
        createProperty(getProcessName() + "_jar_name", getProcessName() + ".jar");
        String base = getProp("build_process_files_dir") + this.sep + getProcessName() + this.sep + getProcessName();
        createProperty(getProcessName() + "_bpel_file", base + ".bpel");
        createProperty(getProcessName() + "_wsdl_file", base + ".wsdl");
    }

    public Task getTaskWithName(String name) {
        Task t = null;
        for (int i = 0; i < this.tasks.size(); i++) {
            Task task = (Task) this.tasks.get(i);
            if (task.getTaskName().equals(name)) {
                t = task;
                break;
            }
        }
        return t;
    }

    public Vector organizeTasks() {
        Vector newV = new Vector();
        if (!this.skipwsdl) {
            newV.addElement(getTaskWithName("wsdl2javaProcess"));
        }
        newV.addElement(getTaskWithName("analyseProcessFiles"));
        if (this.jbi) {
            newV.addElement(getTaskWithName("jbiWSTask"));
        }
        newV.addElement(getTaskWithName("compileGeneratedFiles"));
        return newV;
    }

    public void verifyServer() {
        ConditionTask ct = new ConditionTask();
        ct.setProperty("run_ok");
        Socket s = new Socket();
        int port = Integer.parseInt(getProp("bpel.port"));
        s.setPort(port);
        s.setServer(getProp("bpel.host"));
        s.setProject(getProject());
        ct.addSocket(s);
        ct.setProject(getProject());

        ct.execute();

        Exit exit = new Exit();
        String st = "\n********** E R R O R ******************************";
        st += "\nNo listener on " + getProp("bpel.host") + ":" + getProp("bpel.port") + ".";
        st += "\nVerify that the Application Server is executing.";
        exit.setMessage(st);
        exit.setUnless("run_ok");
        exit.setProject(getProject());
        exit.setTaskName("exit");
        exit.execute();
    }

    public void createDirs() {
        createDirName(getProp("build_tmp_dir") + this.sep + getProcessName());
    }

    public void addConfiguredWsdlProcess(Wsdl2JavaProcessTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredAnalyseProcessFiles(AnalyseProcessFilesTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredJbiWSTask(JbiWSTask task) {
        addTask(this.tasks, task);
    }

    public void addConfiguredCompileGeneratedFiles(CompileGeneratedFilesTask task) {
        addTask(this.tasks, task);
    }

}