/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.FilterSet;
import org.apache.tools.ant.types.Mapper;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;



/**
 * ConfigTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: ConfigTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class ConfigTask extends StandardDeployTask {

    private String dir = null;
    private String bpelPort = null;
    private String bpelHost = null;
    private String carolJrmpUrl = null;
    private String bpelHome = null;

    public void setDir(String dir) {
        this.dir = dir;
    }

    public void setBpelport(String bpelPort) {
        this.bpelPort = bpelPort;
    }

    public void setBpelhost(String bpelHost) {
        this.bpelHost = bpelHost;
    }

    public void setBpelHome(String bpelHome) {
        this.bpelHome = bpelHome;
        System.setProperty(BPELConstants.BPEL_HOME_PROPERTY,bpelHome);
    }

    public void setCaroljrmpurl(String carolJrmpUrl) {
        this.carolJrmpUrl = carolJrmpUrl;
    }

    public String getDir() {
        return this.dir;
    }

    public String getBpelport() {
        return this.bpelPort;
    }

    public String getBpelhost() {
        return this.bpelHost;
    }

    public String getBpelHome() {
        return this.bpelHome;
    }

    public String getCaroljrmpurl() {
        return this.carolJrmpUrl;
    }

    public ConfigTask() {
        setTaskName("configWsdl");
    }

    public void execute() {
        log("Starting ConfigTask...");
        Copy c = new Copy();
        c.setProject(getProject());
        String st = getProject().getBaseDir().getAbsolutePath();
        st += File.separator + getDir();
        c.setTodir(new File(st));
        c.setOverwrite(true);
        FileSet fs = new FileSet();
        fs.setDir(new File(st));
        c.addFileset(fs);
        Mapper m = c.createMapper();
        Mapper.MapperType mt = new Mapper.MapperType();
        mt.setValue("glob");
        m.setType(mt);
        m.setFrom("*.wsdl.ori");
        m.setTo("*.wsdl");
        FilterSet fts = c.createFilterSet();
        fts.addFilter("BPELPORTNUMBER", getBpelport());
        fts.addFilter("BPELSRVNAME", getBpelhost());
        String base = Util.getFilePrefix();
        fts.addFilter("BPELDIRNAME", base+BPELConstants.BPEL_HOME);
        fts.addFilter("BPELTESTDIRNAME", base+BPELConstants.TESTS_JUNIT_PACK_DIR);
        fts.addFilter("BPELTESTUTILDIRNAME", base+BPELConstants.TESTS_UTIL_PACK_DIR);
        fts.addFilter("JRMPURL", getCaroljrmpurl());
        c.setTaskName("copy");
        c.execute();
        log("Finishing ConfigTask...");
    }
}