/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Operation;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.Service;
import javax.wsdl.xml.WSDLReader;
import javax.xml.namespace.QName;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Move;
import org.apache.tools.ant.types.FileSet;
import org.objectweb.orchestra.exception.BPELGeneratorException;
import org.objectweb.orchestra.generator.StubServiceFile;
import org.objectweb.orchestra.util.BPELConstants;
import org.objectweb.orchestra.util.Util;

import com.ibm.wsdl.factory.WSDLFactoryImpl;

/**
 * JbiWSTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: JbiWSTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class JbiWSTask extends StandardDeployTask {

    private String[] outServices = null;

    /**
     * @return the outServices
     */
    public String[] getOutServices() {
        return this.outServices;
    }

    /**
     * @param outServices the outServices to set
     */
    public void setOutServices(String[] outServices) {
        this.outServices = outServices;
    }

    public JbiWSTask(String[] outServices) {
        setTaskName("jbiWSTask");
        setOutServices(outServices);
    }

    public void execute() {
        log("Starting JbiWSTask...");
        createJbiWS();
        log("Finishing JbiWSTask...");
    }


    public void createJbiWS() {
        StringBuffer wsddToLoad = new StringBuffer();
        String output = getProp("build_tmp_dir") + this.sep + getProcessName();
        for (int i=0; i<this.outServices.length;i++) {
            String wsdlFile = this.outServices[i];
            try {
                wsdlFile = Util.addMessagePort(wsdlFile, getProcessName(), getProp("bpel.host"), getProp("bpel.port"));
                wsdl2Java(wsdlFile, true,output);
                WSDLFactoryImpl wsdlFactory = new WSDLFactoryImpl();
                WSDLReader wsdlReader = wsdlFactory.newWSDLReader();
                Definition def = wsdlReader.readWSDL(wsdlFile);
                String tns = def.getTargetNamespace();
                String directory = Util.getDirectoryFromPackage(Util.getPackageFromNamespace(tns));
                String wsddDir = output + this.sep + directory;
                String wsdd = updateWSDD(wsddDir);
                updateBindingImplFiles(def,output);
                wsddToLoad.append(wsdd);
                wsddToLoad.append(";");
                copyStub();
            } catch (Exception e) {
                throw new BuildException("Exception in JbiWSTask createJbiWS : \n"+e,e);
            }
        }
        createProperty("wsddToLoad", wsddToLoad.toString());
    }

    private void copyStub() {
        Copy c = new Copy();
        c.setProject(getProject());
        String st = getProp("build_tmp_dir") + this.sep + getProcessName();
        c.setTodir(new File(st));
        c.setOverwrite(true);

        FileSet fs = new FileSet();
        st = getProp("generate_generator_dir") + this.sep + getProcessName();
        fs.setDir(new File(st));
        fs.setIncludes("**/*Stub.java");
        c.addFileset(fs);

        c.setTaskName("copyStub");
        c.execute();
    }
    public void updateBindingImplFiles(Definition def,String sourceDir) {
        //for each service, rewrite the file with each operation => one method
        Map services = def.getServices();
        Set servSet = services.entrySet();
        for (Iterator i = servSet.iterator(); i.hasNext();) {
            Map.Entry entry1 = (Map.Entry) i.next();
            Service s = (Service) entry1.getValue();
            Map ports = s.getPorts();
            Set portsSet = ports.entrySet();
            for (Iterator i2 = portsSet.iterator(); i2.hasNext();) {
                Map.Entry entry2 = (Map.Entry) i2.next();
                Port p = (Port) entry2.getValue();
                updateBinding(p.getBinding(),sourceDir,s.getQName(),p);
                updateStubFile(p.getBinding());
            }
        }
    }

    public void updateStubFile(Binding binding) {
        QName bindingQN = binding.getQName();
        try {
            new StubServiceFile(getProcessName(), bindingQN.getNamespaceURI(), bindingQN.getLocalPart(), BPELConstants.BUILD_TMP_DIR);
        } catch (BPELGeneratorException e) {
            throw new BuildException("JbiWSTask.updateBindingFile, BPELGeneratorException caught : ",e);
        }
    }
    public void updateBinding(Binding binding, String sourceDir, QName serviceQN, Port p) {
        try {
            String serviceLocalPart = serviceQN.getLocalPart();
            serviceLocalPart = serviceLocalPart.substring(0,serviceLocalPart.lastIndexOf(BPELConstants.MESSAGE_PORT_SUFFIX));
            String portName = p.getName();
            portName = portName.substring(0,portName.lastIndexOf(BPELConstants.MESSAGE_PORT_SUFFIX));

            QName bindingQN = binding.getQName();
            String bindingPack = Util.getPackageFromNamespace(bindingQN.getNamespaceURI());
            String fileDir = Util.getDirectoryFromPackage(bindingPack);

            String className = bindingQN.getLocalPart()+"Impl";
            String fileName = className+".java";
            String bindingFilePath = sourceDir+this.sep+fileDir+this.sep+fileName;
            File bindingFile = new File(bindingFilePath);
            if (!bindingFile.exists()) {
                throw new BuildException("JbiWSTask.updateBinding, FileNotFound : "+bindingFile+", binding = "+binding+", sourceDir = "+sourceDir);
            }
            File newBindingFile = new File(bindingFilePath+".tmp");
            newBindingFile.createNewFile();
            PortType pt = binding.getPortType();
            FileWriter fw = new FileWriter(newBindingFile);
            BufferedWriter  bw = new BufferedWriter(fw);
            bw.write("package "+bindingPack+";");bw.newLine();bw.newLine();
            bw.write("import javax.xml.parsers.DocumentBuilder;");bw.newLine();
            bw.write("import javax.xml.parsers.DocumentBuilderFactory;");bw.newLine();
            bw.write("import javax.xml.namespace.QName;");bw.newLine();
            bw.write("import org.w3c.dom.Document;");bw.newLine();
            bw.write("import org.w3c.dom.Element;");bw.newLine();
            bw.write("import org.w3c.dom.Node;");bw.newLine();
            bw.write("import org.w3c.dom.Text;");bw.newLine();
            bw.write("import org.w3c.dom.Text;");bw.newLine();
            bw.write("import javax.jms.MessageConsumer;");bw.newLine();
            bw.write("import javax.jms.Queue;");bw.newLine();
            bw.write("import javax.jms.QueueConnection;");bw.newLine();
            bw.write("import javax.jms.QueueConnectionFactory;");bw.newLine();
            bw.write("import javax.jms.Session;");bw.newLine();
            bw.write("import javax.naming.Context;");bw.newLine();
            bw.write("import javax.naming.InitialContext;");bw.newLine();
            bw.write("import javax.jms.MapMessage;");bw.newLine();
            bw.write("import javax.jms.QueueSender;");bw.newLine();
            bw.write("import javax.jms.QueueReceiver;");bw.newLine();
            bw.write("import javax.jms.QueueSession;");bw.newLine();
            bw.write("import org.objectweb.orchestra.util.Util;");bw.newLine();
            bw.write("import org.xml.sax.InputSource;");bw.newLine();
            bw.write("import java.io.StringReader;");bw.newLine();

            bw.newLine();
            bw.write("public class "+className+" {");bw.newLine();bw.newLine();

            List operations = pt.getOperations();
            for (int i=0;i<operations.size();i++) {
                Operation op = (Operation) operations.get(i);
                String opName = op.getName();
                boolean oneWay = op.getOutput() == null;
                bw.write("  public Element[] "+opName+" (Element[] bodies) throws Exception {");bw.newLine();
                bw.write("    boolean oneWay = "+oneWay+";");bw.newLine();
                bw.write("    String operationNS = \""+pt.getQName().getNamespaceURI()+"\";");bw.newLine();
                bw.write("    String operationName = \""+opName+"\";");bw.newLine();
                bw.write("    String serviceNS = \""+serviceQN.getNamespaceURI()+"\";");bw.newLine();
                bw.write("    String serviceName = \""+serviceLocalPart+"\";");bw.newLine();
                bw.write("    String portTypeNS = \""+pt.getQName().getNamespaceURI()+"\";");bw.newLine();
                bw.write("    String portTypeName = \""+pt.getQName().getLocalPart()+"\";");bw.newLine();
                bw.write("    String portName = \""+portName+"\";");bw.newLine();
                //bw.write("    System.out.println(\"Dans la methode "+opName+" dans le impl style = message\");");bw.newLine();
                //bw.write("    for (int i=0;i<bodies.length;i++) {");bw.newLine();
                //bw.write("      System.out.println(\"bodies = \"+bodies[i]);");bw.newLine();
                //bw.write("    }");bw.newLine();
                bw.write("    if (bodies.length == 0 || bodies.length > 1) {");bw.newLine();
                bw.write("      throw new Exception(\"Exception in "+opName+", the number of bodies Element is equal to \"+bodies.length);");bw.newLine();
                bw.write("    }");bw.newLine();
                bw.write("    Element body = bodies[0];");bw.newLine();bw.newLine();

                //bw.write("    System.out.println(\"In the impl file, body received : \"+body);");bw.newLine();
                //bw.write("    Util.printNodeAsString(body);");bw.newLine();
                bw.write("    Element newBody = null;");bw.newLine();
                bw.write("    String id = java.lang.String.valueOf(Util.getUniqueRefpartner());");bw.newLine();bw.newLine();
                bw.write("    Context jndi = new InitialContext();");bw.newLine();
                bw.write("    QueueConnectionFactory QCF = (QueueConnectionFactory) jndi.lookup(\"QCF\");");bw.newLine();
                bw.write("    QueueConnection connection = QCF.createQueueConnection();");bw.newLine();
                bw.write("    Session session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);");bw.newLine();
                bw.write("    QueueSender prod = null;");bw.newLine();
                bw.write("    MapMessage message = null;");bw.newLine();
                bw.write("    //sendMessage to OrchestraJbiSEQueue");bw.newLine();
                bw.write("    Queue OrchestraJbiSEQueue = (Queue) jndi.lookup(\"OrchestraJbiSEQueue\");");bw.newLine();
                bw.write("    prod = ((QueueSession) session).createSender(OrchestraJbiSEQueue);");bw.newLine();
                bw.write("    message = session.createMapMessage();");bw.newLine();
                bw.write("    message.setString(\"operationNS\",operationNS);");bw.newLine();
                bw.write("    message.setString(\"operationName\",operationName);");bw.newLine();
                bw.write("    message.setString(\"serviceNS\",serviceNS);");bw.newLine();
                bw.write("    message.setString(\"serviceName\",serviceName);");bw.newLine();
                bw.write("    message.setString(\"portTypeNS\",portTypeNS);");bw.newLine();
                bw.write("    message.setString(\"portTypeName\",portTypeName);");bw.newLine();
                bw.write("    message.setString(\"portName\",portName);");bw.newLine();
                bw.write("    message.setStringProperty(\"destination\", \"OrchestraJbiSE\");");bw.newLine();
                bw.write("    message.setString(\"id\", id);");bw.newLine();
                bw.write("    message.setBoolean(\"oneWay\", oneWay);");bw.newLine();
                bw.write("    message.setBytes(\"body\", Util.getBytesFromObject(body));");bw.newLine();
                //bw.write("    System.out.println(\"In the impl file before sending the message\");");bw.newLine();
                bw.write("    prod.send(message);");bw.newLine();bw.newLine();
                bw.write("    prod.close();");bw.newLine();
                bw.write("    session.close();");bw.newLine();
                bw.write("    if (!oneWay) {");bw.newLine();
                //bw.write("      System.out.println(\"In the impl file after sending the message, not oneWay\");");bw.newLine();
                bw.write("      //wait for the response");bw.newLine();
                bw.write("      Session recSession = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);");bw.newLine();
                bw.write("      QueueReceiver rec = ((QueueSession) recSession).createReceiver(OrchestraJbiSEQueue, \"destination = 'OrchestraEngine' and id = '\" + id + \"'\");");bw.newLine();
                bw.write("      connection.start();");bw.newLine();
                bw.write("      MapMessage messageReturn = (MapMessage) rec.receive();");bw.newLine();
                //bw.write("      System.out.println(\"In the impl file after receiving the message, messageReturn = \"+messageReturn);");bw.newLine();
                bw.write("      byte[] byteTab = messageReturn.getBytes(\"newBody\");");bw.newLine();
                //bw.write("      System.out.println(\"In the impl file after receiving the message, byteTab = \"+byteTab);");bw.newLine();
                //bw.write("      System.out.println(\"In the impl file after receiving the message, byteTab.size = \"+byteTab.length);");bw.newLine();
                bw.write("      String domTreeAsString = (String) Util.getObjectFromBytes(byteTab);");bw.newLine();
                bw.write("      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();");bw.newLine();
                bw.write("      DocumentBuilder docBuilder = dbf.newDocumentBuilder();");bw.newLine();
                bw.write("      InputSource is = new InputSource(new StringReader(domTreeAsString));");bw.newLine();
                bw.write("      newBody = docBuilder.parse(is).getDocumentElement();");bw.newLine();
                bw.write("      //newBody = (Element) Util.getObjectFromBytes(byteTab);");bw.newLine();
                //bw.write("      System.out.println(\"In the impl file after receiving the message, newBody = \"+newBody);");bw.newLine();
                //bw.write("      Util.printNodeAsString(newBody);");bw.newLine();
                bw.write("      recSession.close();");bw.newLine();
                bw.write("    }");bw.newLine();

                bw.write("    connection.close();");bw.newLine();
                bw.write("    prod = null;");bw.newLine();
                bw.write("    session = null;");bw.newLine();
                bw.write("    connection = null;");bw.newLine();
                //bw.write("    System.out.println(\"In the impl file return\");");bw.newLine();
                //bw.write("    return new Element[]{document.getDocumentElement()};");bw.newLine();
                bw.write("    return new Element[]{newBody};");bw.newLine();
                bw.write("  }");bw.newLine();
            }
            bw.write("}");
            bw.close();
            fw.close();

            Move m = new Move();
            m.setProject(getProject());
            m.setTofile(bindingFile);
            m.setFile(newBindingFile);
            m.setTaskName("moveServiceImpl");
            m.execute();

        } catch (Exception e) {
            throw new BuildException("Exception caught in JbiWSTask.updateBinding, binding = "+binding+", sourceDir = "+sourceDir,e);
        }
    }

    public String updateWSDD(String wsddDir) {
        File wsddFile = null;
        try {
            wsddFile = new File(wsddDir + this.sep + "deploy.wsdd");
            File newWsddFile = new File(wsddDir + this.sep + "newdeploy.wsdd");
            FileReader fr = new FileReader(wsddFile);
            newWsddFile.createNewFile();
            FileWriter fw = new FileWriter(newWsddFile);
            BufferedReader br = new BufferedReader(fr);
            BufferedWriter  bw = new BufferedWriter(fw);
            String st = null;
            while ((st = br.readLine()) != null) {
                if (st.indexOf("<service ") != -1) {
                    String toWrite = removeStyle(st);
                    toWrite = replaceProvider(toWrite);
                    toWrite = replaceUse(toWrite);
                    bw.write(toWrite);
                    bw.newLine();
                    bw.write("      <parameter name=\"bpelProcessName\" value=\""+getProcessName()+"\"/>");
                    bw.newLine();
                    br.readLine();
                    st = br.readLine();
                }/* else if (st.indexOf("<parameter name=\"className\" value=\"") != -1) {
                    String toWrite = replaceClassName(st);
                    bw.write(toWrite);
                    bw.newLine();
                    br.readLine();
                    st = br.readLine();
                }*/
                bw.write(st);
                bw.newLine();
            }
            fr.close();
            br.close();
            bw.close();

            Move m = new Move();
            m.setProject(getProject());
            m.setTofile(wsddFile);
            m.setFile(newWsddFile);
            m.setTaskName("moveWsdd");
            m.execute();
        } catch (Exception e) {
            throw new BuildException("JbiWSTask.updateWSDD : ",e);
        }
        return wsddFile.getAbsolutePath();
    }

    public String removeStyle(String line) throws Exception {
        String searched = " style=\"";
        int searchedeIndex = line.indexOf(searched);
        String newServiceLine = line.substring(0,searchedeIndex);
        String end = line.substring(searchedeIndex+searched.length()+1);
        end = end.substring(end.indexOf("\"")+1);
        newServiceLine += end;
        return newServiceLine;
    }

    public String replaceProvider(String line) throws Exception {
        String searched = " provider=\"";
        int searchedeIndex = line.indexOf(searched);
        String newServiceLine = line.substring(0,searchedeIndex)+searched+"java:BPELMsg";
        String end = line.substring(searchedeIndex+searched.length()+1);
        end = end.substring(end.indexOf("\""));
        newServiceLine += end;
        return newServiceLine;
    }

    public String replaceClassName(String line) throws Exception {
        String searched = " value=\"";
        int searchedeIndex = line.indexOf(searched);
        String end = line.substring(searchedeIndex+searched.length()+1);
        String oldValue = end.substring(0, end.indexOf("\""));
        String newServiceLine = line.substring(0,searchedeIndex)+searched+oldValue+BPELConstants.MESSAGE_PORT_SUFFIX;
        end = end.substring(end.indexOf("\""));
        newServiceLine += end;
        return newServiceLine;
    }

    public String replaceUse(String line) throws Exception {
        String searched = " use=\"";
        int searchedeIndex = line.indexOf(searched);
        String newServiceLine = line.substring(0,searchedeIndex)+searched+"literal";
        String end = line.substring(searchedeIndex+searched.length()+1);
        end = end.substring(end.indexOf("\""));
        newServiceLine += end;
        return newServiceLine;
    }

    public void wsdl2Java(String wsdlFile, boolean serverSide, String output) {
        Wsdl2JavaProcessTask wsdl2java = new Wsdl2JavaProcessTask(serverSide, wsdlFile,output);
        wsdl2java.setProject(getProject());
        wsdl2java.setProcessName(getProcessName());
        wsdl2java.execute();
    }

}



