/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;

import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.Path;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * LaunchProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LaunchProcessTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class LaunchProcessTask extends StandardDeployTask {

    private String clientName = null;

    private String args = null;

    public void setClientname(String clientName) {
        this.clientName = clientName;
    }

    public void setArgs(String args) {
        this.args = args;
    }

    public String getClientname() {
        return this.clientName;
    }

    public String getArgs() {
        return this.args;
    }

    public LaunchProcessTask() {
        setTaskName("launchProcess");
    }

    public void execute() {
        log("Starting LaunchProcessTask...");
        Java j = new Java();
        j.setProject(getProject());
        j.setFork(true);
        j.init();

        Commandline.Argument secu = j.createJvmarg();
        String l = "-Djava.security.auth.login.config=";
        l += getProp("jaas_config");
        secu.setValue(l);

        j.setClassname("org.objectweb.jonas.client.ClientContainer");

        Path classpath = new Path(getProject());
        Path.PathElement pe3 = classpath.createPathElement();
        pe3.setLocation(new File(getProp("jonas_base_conf")));
        Path.PathElement pe1 = classpath.createPathElement();
        pe1.setLocation(new File(getProp("client.jar")));
        Path.PathElement pe2 = classpath.createPathElement();
        pe2.setLocation(new File(getProp("bpel.jar")));
        Path.PathElement pe5 = classpath.createPathElement();
        String mail = getProp("jonas_root_lib") + this.sep + "commons" + this.sep + "j2ee" + this.sep + "mail.jar";
        pe5.setLocation(new File(mail));
        Path.PathElement pe6 = classpath.createPathElement();
        String activation = getProp("jonas_root_lib") + this.sep + "commons" + this.sep + "j2ee" + this.sep + "activation.jar";
        pe6.setLocation(new File(activation));

        Path.PathElement pe8 = classpath.createPathElement();
        String junit = getProp("lib_dir") + this.sep + "junit" + this.sep + "junit.jar";
        pe8.setLocation(new File(junit));

        if (getProcessName().equals("unitSamples")) {
            try {
                ArrayList samples = readSamples();
                for (int i = 0; i < samples.size(); i++) {
                    String processName = (String) samples.get(i);
                    Path.PathElement pe4 = classpath.createPathElement();
                    String jar = getProp("build_jars_dir") + this.sep + processName + ".jar";
                    pe4.setLocation(new File(jar));
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        } else {
            Path.PathElement pe4 = classpath.createPathElement();
            String jar = getProp("build_jars_dir") + this.sep + getProcessName() + ".jar";
            pe4.setLocation(new File(jar));
        }

        String clientJar = getProp("build_clientJars_dir") + this.sep + getProcessName() + ".jar";
        File clientJarFile = new File(clientJar);
        if(clientJarFile.exists()) {
            Path.PathElement pe7 = classpath.createPathElement();
            pe7.setLocation(clientJarFile);
        }

        classpath.add(removeBootStrapJar());

        Commandline.Argument line = j.createArg();
        line.setValue(getClientname());

        Commandline.Argument args = j.createArg();
        args.setLine(getArgs());

        Environment.Variable env = new Environment.Variable();
        env.setKey("CLASSPATH");
        env.setPath(classpath);
        j.addEnv(env);

        Environment.Variable env2 = new Environment.Variable();
        env2.setKey(BPELConstants.BPEL_HOME_PROPERTY);
        env2.setPath(new Path(getProject(), BPELConstants.BPEL_HOME));
        j.addEnv(env2);

        j.setTaskName("java");
        j.execute();

        log("Finishing LaunchProcessTask...");
    }

    private ArrayList readSamples() throws Exception {
        String prefix = "sample";
        ArrayList samples = new ArrayList();
        String file = BPELConstants.TESTS_DIR + this.sep + "testsList.txt";
        File f = new File(file);
        BufferedReader br = new BufferedReader(new FileReader(f));
        String line = br.readLine();
        while (line != null) {
            String[] elements = line.split(":");
            if (elements.length >= 2 && elements[0].indexOf("#") == -1 && elements[0].startsWith(prefix)) {
                //it is not a comment
                //substring remove " character
                String processName = elements[1].substring(1, elements[1].length() - 1);
                samples.add(processName);
            }
            line = br.readLine();
        }
        return samples;
    }
}