/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;

import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Environment;
import org.apache.tools.ant.types.Path;
import org.objectweb.orchestra.util.BPELConstants;


/**
 * LoadProcessTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: LoadProcessTask.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class LoadProcessTask extends StandardDeployTask {

    public LoadProcessTask() {
        setTaskName("loadProcessJar");
    }

    public void execute() {
        log("Starting LoadProcessTask...");
        loadProcessJarInServer();
        log("Finishing LoadProcessTask...");
    }

    public void loadProcessJarInServer() {
        Java j = new Java();
        j.setProject(getProject());
        j.setFork(true);
        j.init();

        Commandline.Argument secu = j.createJvmarg();
        String l = "-Djava.security.auth.login.config=";
        l += getProp("jaas_config");
        secu.setValue(l);

        Commandline.Argument jh = j.createJvmarg();
        l = "-D"+BPELConstants.BPEL_HOME_PROPERTY;
        l += BPELConstants.BPEL_HOME;
        jh.setValue(l);

        j.setClassname("org.objectweb.jonas.client.ClientContainer");

        Path classpath = new Path(getProject());
        Path.PathElement pe3 = classpath.createPathElement();
        pe3.setLocation(new File(getProp("jonas_base_conf")));
        Path.PathElement pe1 = classpath.createPathElement();
        pe1.setLocation(new File(getProp("client.jar")));
        Path.PathElement pe2 = classpath.createPathElement();
        pe2.setLocation(new File(getProp("bpel.jar")));
        classpath.add(removeBootStrapJar());

        Commandline.Argument line = j.createArg();
        line.setValue("org.objectweb.orchestra.client.LoadProcessClient");

        Commandline.Argument mode = j.createArg();
        mode.setValue("add");

        Commandline.Argument path = j.createArg();
        Path p = new Path(getProject());
        String jar = getProp("build_jars_dir") + this.sep + getProp(getProcessName() + "_jar_name");
        p.setLocation(new File(jar));
        path.setPath(p);

        Environment.Variable env = new Environment.Variable();
        env.setKey("CLASSPATH");
        env.setPath(classpath);
        j.addEnv(env);

        j.setTaskName("java");
        j.execute();
    }

}