/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tasks;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import org.apache.axis.utils.ClassUtils;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.objectweb.orchestra.engine.BPELEngine;
import org.objectweb.orchestra.engine.FileMap;
import org.objectweb.orchestra.util.ClassLoaderMap;
import org.objectweb.orchestra.util.Util;



/**
 * ConfigTask.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: UndeployProcessTask.java 49 2006-11-28 12:51:29Z souillac $
 */
public class UndeployProcessTask extends StandardDeployTask {

    public UndeployProcessTask() {
        setTaskName("undeployProcess");
    }

    public void execute() {
        log("Starting UndeployProcessTask...");
        undeployService();
        log("Finishing UndeployProcessTask...");
    }

    private void undeployService() {
        FileMap.removeAll(getProcessName());

        String fileToUndeploy = getProp("build_wsdd_dir") + this.sep + getProcessName()+"undeploy.wsdd";
        undeployWS(fileToUndeploy);
        deleteFile(fileToUndeploy);
        String jar = getProp("build_jars_dir") + this.sep + getProp(getProcessName() + "_jar_name");
        deleteFile(jar);
        if (BPELEngine.isJbi()) {
            File extWsddDir = new File(getProp("wsddExtJbi")+this.sep+getProcessName());
            File[] ext = extWsddDir.listFiles();
            for (int i=0;i<ext.length;i++) {
                File f = ext[i];
                if (f.getName().endsWith("_undeploy.wsdd")) {
                    undeployWS(f.getAbsolutePath());
                }
            }
        }

    }

    private void undeployWS(String fileToUndeploy) {
        ClassLoader old = ClassUtils.getDefaultClassLoader();
        try {
            ClassLoader classloader = ClassLoaderMap.getCL(getProcessName());
            ClassUtils.setDefaultClassLoader(classloader);

            String l = "-lhttp://";
            l += getProp("bpel.host") + ":" + getProp("bpel.port");
            l += "/axis/servlet/AxisServlet";

            Method m = null;
            Class cl = null;
            URL[] urlTab = new URL[1];
            String url = Util.getFilePrefix();
            urlTab[0] = new URL(url+getProp("axis-ant"));
            //ClassLoader parent = Thread.currentThread().getContextClassLoader();
            URLClassLoader classL = new URLClassLoader(urlTab,classloader);
            cl = Class.forName("org.apache.axis.tools.ant.axis.AdminClientTask",true, classL);
            Object adminClientTask = cl.newInstance();
            m = cl.getMethod("setProject", new Class[]{Project.class});
            m.invoke(adminClientTask,new Object[]{getProject()});
            m = cl.getMethod("init", (Class[])null);
            m.invoke(adminClientTask,(Object[])null);
            m = cl.getMethod("setUrl", new Class[]{java.lang.String.class});
            m.invoke(adminClientTask,new Object[]{l});
            m = cl.getMethod("setXmlFile", new Class[]{java.io.File.class});
            m.invoke(adminClientTask,new Object[]{new File(fileToUndeploy)});
            m = cl.getMethod("setTaskName", new Class[]{java.lang.String.class});
            m.invoke(adminClientTask,new Object[]{"AdminClientTask"});
            m = cl.getMethod("execute", (Class[])null);
            m.invoke(adminClientTask,(Object[])null);

            //org.apache.axis.tools.ant.axis.AdminClientTask act = new org.apache.axis.tools.ant.axis.AdminClientTask();
            //act.setProject(getProject());
            //act.init();
            //act.setUrl(l);
            //act.setXmlFile(new File(file));
            //act.setTaskName("AdminClientTask");
            //act.execute();
        } catch (Exception e) {
            throw new BuildException("Exception in UndeployProcessTask undeployService : \n"+e,e);
        } finally {
            ClassUtils.setDefaultClassLoader(old);
        }
    }
}