/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.tree;

/**
 * JavaSnippet.java represents a Java snippet.
 *
 * @author Jean-Noel Rouvignac
 * @version $Id: JavaSnippet.java 5 2006-10-20 14:37:44Z lejeuneg $
 */
public class JavaSnippet extends ExpressionImpl {

    /**
     * The name of the BPELJ process
     */
    private String BPELJProcessName;
    /**
     * The class containing the java code to execute
     */
    private String javaClassName;
    /**
     * The method of the java class containing the java code to execute
     */
    private String javaMethodName;

    /**
     * Constructor for a JavaSnippet.
     * The java code of this statement is encapsulated into a static method of the java
     * class representing the BPELJ process. This class is generated by the BPEL generator.
     * @param expression the text of the expression as it appears inside the BPEL process file.
     * @param BPELJProcessName the name of the BPELJ process to deploy
     * @param className the fully qualified name of the Java class containing the Java snippet
     * @param methodName the Java method name containing the Java snippet
     */
    public JavaSnippet(String expression, String BPELJProcessName, String className, String methodName) {
        super(expression);
        this.BPELJProcessName = BPELJProcessName;
        this.javaClassName = className;
        this.javaMethodName = methodName;
    }

    /**
     * Returns the name of the BPELJ process
     * @return the name of the BPELJ process
     */
    public String getBPELJProcessName() {
        return this.BPELJProcessName;
    }

    /**
     * Returns the class containing the java code to execute
     * @return the class containing the java code to execute
     */
    public String getClassName() {
        return this.javaClassName;
    }

    /**
     * Returns the method of the java class containing the java code to execute
     * @return the method of the java class containing the java code to execute
     */
    public String getMethodName() {
        return this.javaMethodName;
    }
}
