/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.objectweb.orchestra.exception.BPELUtilException;


/**
 * BPELTransaction.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: BPELTransaction.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class BPELTransaction {
    private UserTransaction ut = null;

    public BPELTransaction() throws BPELUtilException {
        try {
            Context jndi = new InitialContext();
            this.ut = (javax.transaction.UserTransaction)jndi.lookup("javax.transaction.UserTransaction");
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException BPELTransaction Contructor :\n" , e);
        }
    }

    public void commitTx() throws BPELUtilException {
        try {
            this.ut.commit();
        } catch (SecurityException e) {
            throw new BPELUtilException("SecurityException BPELTransaction commitTx :\n" , e);
        } catch (IllegalStateException e) {
            throw new BPELUtilException("IllegalStateException BPELTransaction commitTx :\n" , e);
        } catch (RollbackException e) {
            throw new BPELUtilException("RollbackException BPELTransaction commitTx :\n" , e);
        } catch (HeuristicMixedException e) {
            throw new BPELUtilException("HeuristicMixedException BPELTransaction commitTx :\n" , e);
        } catch (HeuristicRollbackException e) {
            throw new BPELUtilException("HeuristicRollbackException BPELTransaction commitTx :\n" , e);
        } catch (SystemException e) {
            throw new BPELUtilException("SystemException BPELTransaction commitTx :\n" , e);
        }
    }

    public void beginTx() throws BPELUtilException {
        try {
            this.ut.begin();
        } catch (NotSupportedException e) {
            throw new BPELUtilException("NotSupportedException BPELTransaction beginTx :\n" , e);
        } catch (SystemException e) {
            throw new BPELUtilException("SystemException BPELTransaction beginTx :\n" , e);
        }
    }

    public void roolbackTx() throws BPELUtilException {
        try {
            if (this.ut != null) {
                this.ut.rollback();
            }
        } catch (SystemException e) {
            throw new BPELUtilException("SystemException BPELTransaction roolbackTx :\n" , e);
        }
    }
}


