/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

/**
 * DynamicArray.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: DynamicArray.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class DynamicArray {

    private Object[] array;

    final static int INCR = 10;

    public DynamicArray(int initialCapacity) {
        this.array = new Object[initialCapacity];
    }

    public synchronized int size() {
        return this.array.length;
    }

    public void set(int i, Object o) {
        if (i < 0) throw new IndexOutOfBoundsException();
        synchronized (this) {
            if (i >= size()) {
                Object[] tmp = this.array;
                this.array = new Object[i + INCR];
                System.arraycopy(tmp, 0, this.array, 0, tmp.length);
            }
            this.array[i] = o;
        }
    }

    public synchronized Object get(int i) {
        return this.array[i];
    }
}