/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.objectweb.orchestra.util;

import java.io.Serializable;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.orchestra.engine.BPELMessage;
import org.objectweb.orchestra.exception.BPELUtilException;



/**
 * JmsUtil.java
 *
 * @author Goulven Le Jeune & Charles Souillard
 * @version $Id: JmsUtil.java 5 2006-10-20 14:37:44Z lejeuneg $
 *
 */
public class JmsUtil {

    private static Context jndi = null;
    private static Queue BPUQueue = null;
    private static Queue BPIQueue = null;
    private static Queue InvokerQueue = null;
    private static Queue BPELQueue = null;
    private static Queue BPELReplyQueue = null;
    private static QueueConnectionFactory QCF = null;

    public static void sendMessageBPI(String source, int bpiId, String type, String[] argsName, Object[] argsValue) throws BPELUtilException {
        try {
            //create connections, sessions
            QueueConnection connection = getConnection();
            Session session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            QueueSender prod = null;
            MapMessage message = null;

            //sendMessage to BPIQueue
            prod = ((QueueSession) session).createSender(getBPIQueue());
            message = session.createMapMessage();
            message.setIntProperty("bpiId", bpiId);
            prod.send(message);
            prod.close();

            //sendMessage to BPELQueue
            prod = ((QueueSession) session).createSender(getBPELQueue());
            message = session.createMapMessage();
            message.setStringProperty("destination", "BPI");
            message.setString("source", source);
            message.setIntProperty("bpiId", bpiId);
            message.setString("type", type);
            for (int i=0;i<argsName.length;i++) {
                String propertyName = argsName[i];
                Object tmp = argsValue[i];
                if (tmp == null) {
                    message.setString(propertyName, null);
                } else if (tmp instanceof String) {
                    message.setString(propertyName, (String)tmp);
                } else if (tmp instanceof Integer) {
                    message.setInt(propertyName, ((Integer)tmp).intValue());
                } else {
                    byte[] tab = (byte[])Util.putValueObject(tmp);
                    message.setBytes(propertyName, tab);
                }
            }
            prod.send(message);

            //closse connection, session
            prod.close();
            session.close();
            connection.close();
            prod = null;
            session = null;
            connection = null;
            message = null;
        } catch (JMSException e) {
            throw new BPELUtilException("Util, sendMessageBPI, JMSException, ",e);
        }
    }

    public static void sendMessageInvoker(Serializable o, String processName) throws BPELUtilException {
        try {
            //create connections, sessions
            QueueConnection connection = getConnection();
            Session session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            QueueSender prod = null;
            MapMessage message = null;

            //sendMessage to InvokerQueue
            prod = ((QueueSession) session).createSender(getInvokerQueue());
            message = session.createMapMessage();
            byte[] bytes = Util.getBytesFromObject(o);
            message.setBytes("invokerObject",bytes);
            message.setString("processName",processName);
            prod.send(message);
            prod.close();

            //closse connection, session
            prod.close();
            session.close();
            connection.close();
            prod = null;
            session = null;
            connection = null;
            message = null;
        } catch (JMSException e) {
            throw new BPELUtilException("Util, sendMessageBPI, JMSException, ",e);
        }
    }

    public static void sendReplyBPELQueue(int bpiId, int scopeId, String refpartner,String processName, Object value) throws BPELUtilException {
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("JmsUtil sendReplyBPELQueue, BPI = " + bpiId + "  scope = " + scopeId + " refpartner = " + refpartner);
        try {
            BPELMessage jm = (BPELMessage) Util.getValueObject(value,processName);
            Queue queue = getBPELReplyQueue();
            QueueConnection connectionMDB = getConnection();
            Session session = connectionMDB.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            QueueSender prod = ((QueueSession) session).createSender(queue);

            MapMessage message = session.createMapMessage();
            message.setStringProperty("refpartner", refpartner);
            message.setIntProperty("bpiId", bpiId);
            message.setBytes("result", Util.getBytesFromObject(jm));
            message.setString("processName", processName);
            prod.send(message);
            prod.close();
            session.close();
            connectionMDB.close();
            prod = null;
            session = null;
            connectionMDB = null;
            message = null;
        } catch (JMSException e) {
            throw new BPELUtilException("JMSException JmsUtil sendReplyBPELQueue :\n" , e);
        }
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("JmsUtil end sendReplyBPELQueue");
    }

    public static void sendReplyErrorBPELQueue(int bpiId, String refpartner,String ex) throws BPELUtilException {
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("JmsUtil sendReplyErrorBPELQueue, BPI = " + bpiId + " refpartner = " + refpartner+", ex = "+ex);
        try {
            Queue queue = getBPELReplyQueue();
            QueueConnection connectionMDB = getConnection();
            Session session = connectionMDB.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            QueueSender prod = ((QueueSession) session).createSender(queue);

            MapMessage message = session.createMapMessage();
            message.setStringProperty("refpartner", refpartner);
            message.setIntProperty("bpiId", bpiId);
            message.setString("error", ex);
            prod.send(message);
            session.close();
            connectionMDB.close();
            prod.close();
            session = null;
            connectionMDB = null;
            prod = null;
            message = null;
        } catch (JMSException e) {
            throw new BPELUtilException("JMSException JmsUtil sendReplyErrorBPELQueue :\n" , e);
        }
        if (BPELTrace.isEngineDebugEnabled())
            BPELTrace.traceEngine("JmsUtil end sendReplyErrorBPELQueue");
    }

    private static void initJndi() throws BPELUtilException {
        try {
            if (jndi == null) {
                jndi = new InitialContext();
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initJndi : ",e);
        }
    }

    private static void initBpuQueue() throws BPELUtilException {
        initJndi();
        try {
            if (BPUQueue == null) {
                BPUQueue = (Queue) jndi.lookup("BPUQueue");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initBpuQueue : ",e);
        }
    }

    private static void initBpiQueue() throws BPELUtilException {
        initJndi();
        try {
            if (BPIQueue == null) {
                BPIQueue = (Queue) jndi.lookup("BPIQueue");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initBpiQueue : ",e);
        }
    }

    private static void initInvokerQueue() throws BPELUtilException {
        initJndi();
        try {
            if (InvokerQueue == null) {
                InvokerQueue = (Queue) jndi.lookup("InvokerQueue");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initInvokerQueue : ",e);
        }
    }

    private static void initBpelQueue() throws BPELUtilException {
        initJndi();
        try {
            if (BPELQueue == null) {
                BPELQueue = (Queue) jndi.lookup("BPELQueue");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initBpelQueue : ",e);
        }
    }

    private static void initBpelReplyQueue() throws BPELUtilException {
        initJndi();
        try {
            if (BPELReplyQueue == null) {
                BPELReplyQueue = (Queue) jndi.lookup("BPELReplyQueue");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initBpelReplyQueue : ",e);
        }
    }

    private static void initQcf() throws BPELUtilException {
        initJndi();
        try {
            if (QCF == null) {
                QCF = (QueueConnectionFactory) jndi.lookup("QCF");
            }
        } catch (NamingException e) {
            throw new BPELUtilException("NamingException in JmsUtil.initQcf : ",e);
        }
    }

    public static Queue getBPUQueue() throws BPELUtilException{
        initBpuQueue();
        return BPUQueue;
    }

    public static Queue getBPELReplyQueue() throws BPELUtilException{
        initBpelReplyQueue();
        return BPELReplyQueue;
    }

    public static Queue getBPIQueue() throws BPELUtilException{
        initBpiQueue();
        return BPIQueue;
    }

    public static Queue getInvokerQueue() throws BPELUtilException{
        initInvokerQueue();
        return InvokerQueue;
    }

    public static Queue getBPELQueue() throws BPELUtilException{
        initBpelQueue();
        return BPELQueue;
    }

    public static QueueConnection getConnection() throws BPELUtilException{
        initQcf();
        QueueConnection connection = null;
        try {
            connection = QCF.createQueueConnection();
        } catch (JMSException e) {
            throw new BPELUtilException("JMSException in JmsUtil.getConnection : ",e);
        }
        return connection;
    }
}