package org.objectweb.orchestra.tests.load;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Vector;
import java.util.Set;
import java.util.Iterator;
import java.util.Map;
import java.util.Date;

import org.objectweb.orchestra.tests.load.ParameterElement;
import org.objectweb.orchestra.tests.load.Util;


public class ReportAnalyser {

    private String logFilePath = null;
    private long execTime = -1;
    private HashMap datas = null;
    private Vector params = null;
    private Vector instPerMin = null;
    private Vector avg = null;
    private int printsByLine = 25;
    private String statsSeparator = " ";

    public ReportAnalyser(String logFilePath) {
        this.logFilePath = logFilePath;
    }

    public void analyseReport(HashMap bestTimes, HashMap worthTimes) throws Exception {
        readDatas();
        printResults(bestTimes,worthTimes);
    }

    private void readDatas() throws Exception {
        this.datas = new HashMap();
        this.params = new Vector();
        this.instPerMin = new Vector();
        this.avg = new Vector();
        BufferedReader br = new BufferedReader(new FileReader(new File(this.logFilePath)));
        int mode = 0;
        //mode=0 => PARTAMETERS
        //mode=1 => DATAS
        //mode=2 => EXECTIME
        String line = br.readLine();
        while (line != null) {
            if (line.equals(Util.PARAMETERS)) {
                mode = 0;
            } else if (line.equals(Util.DATAS)) {
                mode = 1;
            } else if (line.equals(Util.EXECTIME)) {
                mode = 2;
            } else {
                if (mode == 0) {
                    this.params.addElement(new ParameterElement(line));
                } else if (mode == 1) {
                    if (line.startsWith(Util.STATS)) {
                        String[] tab = line.split(";");
                        this.instPerMin.add(tab[1]);
                        this.avg.add(tab[2]);
                    } else {
                        ThreadElement elem = getElem(line);
                        String pn = elem.getProcessName();
                        if (this.datas.get(pn) == null) {
                            this.datas.put(pn,new Vector());
                        }
                        Vector v = (Vector) this.datas.get(pn);
                        v.addElement(elem);
                    }
                } else if (mode == 2) {
                    this.execTime = Long.parseLong(line);
                }
            }
            line = br.readLine();
        }
    }

    private ThreadElement getElem(String line) {
        String[] elements = line.split(" ");
        String processName = elements[0];
        String threadNb = elements[1];
        String startTime = elements[2];
        String execTime = elements[3];
        String success = elements[4];
        return new ThreadElement(processName,threadNb,startTime,execTime,success);
    }

    private Vector sortByStartDate(Vector v) {
        Vector result = new Vector();
        long time = new Date().getTime();
        while (v.size() > 0) {
            int minIndex = 0;
            long min = time;
            for (int i = 0; i <v.size(); i++) {
                ThreadElement te = (ThreadElement)v.get(i);
                if (te.getStartTime()<min) {
                    min = te.getStartTime();
                    minIndex = i;
                }
            }
            result.addElement(v.get(minIndex));
            v.removeElementAt(minIndex);
        }
        return result;
    }

    private void printResults(HashMap bestTimes, HashMap worthTimes) throws Exception {
        Vector instancesPerMinutes = new Vector();
        String st = null;
        Set set = this.datas.entrySet();
        long totalExec = 0;
        int instancesNb = 0;

        int totalSuccess = 0;
        int totalErrors = 0;
        for (Iterator i = set.iterator();i.hasNext();) {
            Map.Entry me = (Map.Entry) i.next();
            String processName = (String)me.getKey();
            Vector v = (Vector) me.getValue();
            v = sortByStartDate(v);
            long processExec = 0;
            int success = 0;
            int errors = 0;
            for (int j=0;j<v.size();j++) {
                ThreadElement elem = (ThreadElement) v.get(j);
                processExec += elem.getExecTime();
                if (elem.getSuccess()) {
                    success++;
                } else {
                    errors++;
                }
            }
            st = "Results for process "+processName+" : "+success+" success, "+errors+" errors";
            System.out.println(st);
            st = "Average for process "+processName+" : "+processExec/v.size()+" ms = "+Util.getTime(processExec/v.size());
            System.out.println(st);
            st = "Best time for process "+processName+" : "+bestTimes.get(processName)+", worth time for process "+processName+" : "+worthTimes.get(processName);
            System.out.println(st);
            System.out.println("");
            totalExec += processExec;
            totalSuccess += success;
            totalErrors += errors;
            instancesNb += v.size();
        }
        st = "Results for all processes : "+totalSuccess+" success, "+totalErrors+" errors";
        System.out.println(st);
        st = "Global execution time : "+this.execTime+" ms = "+Util.getTime(this.execTime);
        System.out.println(st);
        st = "Average for all processes : "+totalExec/instancesNb+" ms = "+Util.getTime(totalExec/instancesNb);
        System.out.println(st);
        float timeDif = this.execTime;
        timeDif = timeDif / 60000;
        float result = instancesNb / timeDif;
        st = "Average Number of process Instances per Minutes : "+Math.round(result);
        System.out.println(st);
        System.out.println("");

        System.out.print("Evolution of the number of instances per minute (each timeBetweenPrints miliseconds : set in build.properties)");
        String s = "";
        String sep = " ";
        for (int i=0;i<this.instPerMin.size();i++) {
            if ((i % this.printsByLine) == 0) {
                s += "\n";
            } else if (i != 0) {
                s += this.statsSeparator;
            }
            s += (String) this.instPerMin.get(i);
        }
        System.out.println(s);
        System.out.println("");

        System.out.print("Evolution of the average in seconds (each timeBetweenPrints miliseconds : set in build.properties)");
        s = "";
        for (int i=0;i<this.avg.size();i++) {
            if ((i % this.printsByLine) == 0) {
                s += "\n";
            } else if (i != 0) {
                s += this.statsSeparator;
            }
            float fl = Float.parseFloat((String) this.avg.get(i));
            fl = fl / 1000;
            s += formatFloat(fl,1);
        }
        System.out.println(s);
        System.out.println("");

        System.out.println("XLSH;Test Mode;Process Type;Total Real Time (mn);Simultaneous Tests;Processes per minute;Total Tests;Avg Time per Test (s);Errors");
        String res = "XLSD;";
        String mode = getParameterValue("mode");
        res += mode+";";
        res += getParameterValue("toLaunch")+";";
        if (mode.equals("pick")) {
            res += "?"+";";
        } else {
            res += Integer.parseInt(getParameterValue("time_to_load"))/60+"+"+getParameterValue("loadTime")+";";
        }
        res += getParameterValue("testNb")+";";
        res += Math.round(result)+";";
        res += instancesNb+";";
        float f = totalExec/instancesNb;
        f = f / 1000;
        res += formatFloat(f,2)+";";
        res += totalErrors;
        System.out.println(res);
    }

    private String getParameterValue(String parameterName) throws Exception {
        String st = null;
        for (int i=0;i<this.params.size();i++) {
            ParameterElement pe = (ParameterElement) this.params.get(i);
            if (pe.getName().equals(parameterName)) {
                st = pe.getValue();
                break;
            }
        }
        return st;
    }

    private String formatFloat(float f,int digits) {
        NumberFormat nf = NumberFormat.getIntegerInstance();
        nf.setMinimumFractionDigits(digits);
        nf.setMaximumFractionDigits(digits);
        return (nf.format(f));
    }
}
