package org.objectweb.orchestra.tests.unit.unitCopyJAXRPC2;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC2.CopyJAXRPC2PT;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC2.CopyJAXRPC2ServiceBP;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC2.CopyJAXRPC2ServiceBPLocator;

import java.math.BigInteger;

import org.objectweb.orchestra.util.BPELConstants;

public class TestCopyJAXRPC2 extends TestCase {

    private boolean tmp;

    private CopyJAXRPC2ServiceBP fs = null;
    private CopyJAXRPC2PT fpt = null;

    public TestCopyJAXRPC2(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCopyJAXRPC2.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.fs = new CopyJAXRPC2ServiceBPLocator();
        this.fpt = this.fs.getCopyJAXRPC2PT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCopyJAXRPC2(testtorun));
        }
    }

    public void testCopyJAXRPC21() throws Exception {
        String st = new String("hello");
        String st2 = this.fpt.copyJAXRPC21(st);
        this.tmp = st.equals(st2);
        assertTrue("copyJAXRPC21 error",this.tmp);
    }

    public void testCopyJAXRPC22() throws Exception {
        String dur = new String("P3Y2M1DT3H4M2S");
        String dur2 = this.fpt.copyJAXRPC22(dur);
        this.tmp = dur.equals(dur2);
        assertTrue("copyJAXRPC22 error",this.tmp);
    }

    public void testCopyJAXRPC23() throws Exception {
        String ym = new String("2004,06");
        String ym2 = this.fpt.copyJAXRPC23(ym);
        this.tmp = ym.equals(ym2);
        assertTrue("copyJAXRPC23 error",this.tmp);
    }

    public void testCopyJAXRPC24() throws Exception {
        String y = new String("2004");
        String y2 = this.fpt.copyJAXRPC24(y);
        this.tmp = y.equals(y2);
        assertTrue("copyJAXRPC24 error",this.tmp);
    }

    public void testCopyJAXRPC25() throws Exception {
        String md = new String("06,18");
        String md2 = this.fpt.copyJAXRPC25(md);
        this.tmp = md.equals(md2);
        assertTrue("copyJAXRPC25 error",this.tmp);
    }

    public void testCopyJAXRPC26() throws Exception {
        String d = new String("18");
        String d2 = this.fpt.copyJAXRPC26(d);
        this.tmp = d.equals(d2);
        assertTrue("copyJAXRPC26 error",this.tmp);
    }

    public void testCopyJAXRPC27() throws Exception {
        String m = new String("06");
        String m2 = this.fpt.copyJAXRPC27(m);
        this.tmp = m.equals(m2);
        assertTrue("copyJAXRPC27 error",this.tmp);
    }

    public void testCopyJAXRPC28() throws Exception {
        String t = new String("correct token");
        String t2 = this.fpt.copyJAXRPC28(t);
        this.tmp = t.equals(t2);
        assertTrue("copyJAXRPC28 error",this.tmp);
    }

    public void testCopyJAXRPC29() throws Exception {
        String l = new String("correct language");
        String l2 = this.fpt.copyJAXRPC29(l);
        this.tmp = l.equals(l2);
        assertTrue("copyJAXRPC29 error",this.tmp);
    }

    public void testCopyJAXRPC30() throws Exception {
        String n = new String("correct:name");
        String n2 = this.fpt.copyJAXRPC30(n);
        this.tmp = n.equals(n2);
        assertTrue("copyJAXRPC30 error",this.tmp);
    }

    public void testCopyJAXRPC31() throws Exception {
        String ncn = new String("correct_ncname");
        String ncn2 = this.fpt.copyJAXRPC31(ncn);
        this.tmp = ncn.equals(ncn2);
        assertTrue("copyJAXRPC31 error",this.tmp);
    }

    public void testCopyJAXRPC32() throws Exception {
        String id = new String("correct_id");
        String id2 = this.fpt.copyJAXRPC32(id);
        this.tmp = id.equals(id2);
        assertTrue("copyJAXRPC32 error",this.tmp);
    }

    public void testCopyJAXRPC33() throws Exception {
        String nmt = new String("ab:4");
        String nmt2 = this.fpt.copyJAXRPC33(nmt);
        this.tmp = nmt.equals(nmt2);
        assertTrue("copyJAXRPC33 error",this.tmp);
    }

    public void testCopyJAXRPC34() throws Exception {
        String nmts = new String("ab:5");
        String nmts2 = this.fpt.copyJAXRPC34(nmts);
        this.tmp = nmts.equals(nmts2);
        assertTrue("copyJAXRPC34 error",this.tmp);
    }

    public void testCopyJAXRPC35() throws Exception {
        BigInteger npi = new BigInteger("-4");
        BigInteger npi2 = this.fpt.copyJAXRPC35(npi);
        this.tmp = npi.equals(npi2);
        assertTrue("copyJAXRPC35 error",this.tmp);
    }

    public void testCopyJAXRPC36() throws Exception {
        BigInteger ni = new BigInteger("-4");
        BigInteger ni2 = this.fpt.copyJAXRPC36(ni);
        this.tmp = ni.equals(ni2);
        assertTrue("copyJAXRPC36 error",this.tmp);
    }

    public void testCopyJAXRPC37() throws Exception {
        BigInteger nni = new BigInteger("4");
        BigInteger nni2 = this.fpt.copyJAXRPC37(nni);
        this.tmp = nni.equals(nni2);
        assertTrue("copyJAXRPC37 error",this.tmp);
    }

    public void testCopyJAXRPC38() throws Exception {
        BigInteger ul = new BigInteger("400");
        BigInteger ul2 = this.fpt.copyJAXRPC38(ul);
        this.tmp = ul.equals(ul2);
        assertTrue("copyJAXRPC38 error",this.tmp);
    }

    public void testCopyJAXRPC39() throws Exception {
        BigInteger pi = new BigInteger("40");
        BigInteger pi2 = this.fpt.copyJAXRPC39(pi);
        this.tmp = pi.equals(pi2);
        assertTrue("copyJAXRPC39 error",this.tmp);
    }
}



