package org.objectweb.orchestra.tests.unit.unitCopyJAXRPC3;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC3.CopyJAXRPC3PT;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC3.CopyJAXRPC3ServiceBP;
import org.objectweb.orchestra.tests.unit.unitCopyJAXRPC3.CopyJAXRPC3ServiceBPLocator;


import java.math.BigDecimal;
import java.math.BigInteger;

import javax.xml.namespace.QName;
import javax.xml.rpc.holders.BigDecimalHolder;
import javax.xml.rpc.holders.BigIntegerHolder;
import javax.xml.rpc.holders.BooleanHolder;
import javax.xml.rpc.holders.ByteArrayHolder;
import javax.xml.rpc.holders.ByteHolder;
import javax.xml.rpc.holders.CalendarHolder;
import javax.xml.rpc.holders.DoubleHolder;
import javax.xml.rpc.holders.FloatHolder;
import javax.xml.rpc.holders.IntHolder;
import javax.xml.rpc.holders.LongHolder;
import javax.xml.rpc.holders.QNameHolder;
import javax.xml.rpc.holders.ShortHolder;
import javax.xml.rpc.holders.StringHolder;

import org.objectweb.orchestra.util.BPELConstants;

import java.util.Locale;
import java.util.TimeZone;
import java.util.Calendar;
import java.util.GregorianCalendar;

public class TestCopyJAXRPC3 extends TestCase {

    private boolean tmp;

    private CopyJAXRPC3ServiceBP fs = null;
    private CopyJAXRPC3PT fpt = null;

    public TestCopyJAXRPC3(String testname) {
        super(testname);
    }

    public static TestSuite suite() {
        return new TestSuite(TestCopyJAXRPC3.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.fs = new CopyJAXRPC3ServiceBPLocator();
        this.fpt = this.fs.getCopyJAXRPC3PT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestCopyJAXRPC3(testtorun));
        }
    }

    public void testCopyJAXRPC3_1() throws Exception {
        String st = new String("copy1");
        StringHolder sth = new StringHolder(st);
        this.fpt.copyJAXRPC3_1(sth);
        this.tmp = sth.value.equals(st);
        assertTrue("copyJAXRPC3_1 error",this.tmp);
    }

    public void testCopyJAXRPC3_2() throws Exception {
        BigInteger bi = new BigInteger("-10");
        BigIntegerHolder bih = new BigIntegerHolder(bi);
        this.fpt.copyJAXRPC3_2(bih);
        this.tmp = bih.value.equals(bi);
        assertTrue("copyJAXRPC3_2 error",this.tmp);
    }

    public void testCopyJAXRPC3_3() throws Exception {
        int i = 54;
        IntHolder ih = new IntHolder(i);
        this.fpt.copyJAXRPC3_3(ih);
        this.tmp = ih.value == i;
        assertTrue("copyJAXRPC3_3 error",this.tmp);
    }

    public void testCopyJAXRPC3_4() throws Exception {
        long l = 5;
        LongHolder lh = new LongHolder(l);
        this.fpt.copyJAXRPC3_4(lh);
        this.tmp = l == lh.value;
        assertTrue("copyJAXRPC3_4 error",this.tmp);
    }

    public void testCopyJAXRPC3_5() throws Exception {
        short s = 4;
        ShortHolder sh = new ShortHolder(s);
        this.fpt.copyJAXRPC3_5(sh);
        this.tmp = s == sh.value;
        assertTrue("copyJAXRPC3_5 error",this.tmp);
    }

    public void testCopyJAXRPC3_6() throws Exception {
        BigDecimal bd = new BigDecimal("+15");
        BigDecimalHolder bdh = new BigDecimalHolder(bd);
        this.fpt.copyJAXRPC3_6(bdh);
        this.tmp = bd.equals(bdh.value);
        assertTrue("copyJAXRPC3_6 error",this.tmp);
    }

    public void testCopyJAXRPC3_7() throws Exception {
        float f = 5;
        FloatHolder fh = new FloatHolder(f);
        this.fpt.copyJAXRPC3_7(fh);
        this.tmp = f == fh.value;
        assertTrue("copyJAXRPC3_7 error",this.tmp);
    }

    public void testCopyJAXRPC3_8() throws Exception {
        double d = 4;
        DoubleHolder dh = new DoubleHolder(d);
        this.fpt.copyJAXRPC3_8(dh);
        this.tmp = d == dh.value;
        assertTrue("copyJAXRPC3_8 error",this.tmp);
    }

    public void testCopyJAXRPC3_9() throws Exception {
        boolean bool = true;
        BooleanHolder boolh = new BooleanHolder(bool);
        this.fpt.copyJAXRPC3_9(boolh);
        this.tmp = bool == boolh.value;
        assertTrue("copyJAXRPC3_9 error",this.tmp);
    }

    public void testCopyJAXRPC3_10() throws Exception {
        byte by = 5;
        ByteHolder byh = new ByteHolder(by);
        this.fpt.copyJAXRPC3_10(byh);
        this.tmp = by == byh.value;
        assertTrue("copyJAXRPC3_10 error",this.tmp);
    }

    public void testCopyJAXRPC3_11() throws Exception {
        long ui = 10;
        LongHolder uih = new LongHolder(ui);
        this.fpt.copyJAXRPC3_11(uih);
        this.tmp = ui== uih.value;
        assertTrue("copyJAXRPC3_11 error",this.tmp);
    }

    public void testCopyJAXRPC3_12() throws Exception {
        int us = 10;
        IntHolder ush = new IntHolder(us);
        this.fpt.copyJAXRPC3_12(ush);
        this.tmp = us == ush.value;
        assertTrue("copyJAXRPC3_12 error",this.tmp);
    }

    public void testCopyJAXRPC3_13() throws Exception {
        short ub = 2;
        ShortHolder ubh = new ShortHolder(ub);
        this.fpt.copyJAXRPC3_13(ubh);
        this.tmp = ub == ubh.value;
        assertTrue("copyJAXRPC3_13 error",this.tmp);
    }

    public void testCopyJAXRPC3_14() throws Exception {
        QName qn = new QName("tns","toto");
        QNameHolder qnh = new QNameHolder(qn);
        this.fpt.copyJAXRPC3_14(qnh);
        this.tmp = qn.equals(qnh.value);
        assertTrue("copyJAXRPC3_14 error",this.tmp);
    }

    public void testCopyJAXRPC3_15() throws Exception {
        Calendar cal = new GregorianCalendar(TimeZone.getTimeZone("GMT"),Locale.FRANCE);
        CalendarHolder calh = new CalendarHolder(cal);
        this.fpt.copyJAXRPC3_15(calh);
        this.tmp = cal.getTime().equals(calh.value.getTime());
        assertTrue("copyJAXRPC3_15 error",this.tmp);
    }

    public void testCopyJAXRPC3_16() throws Exception {
        Calendar c = new GregorianCalendar();
        c.setTimeInMillis(System.currentTimeMillis());
        CalendarHolder calh = new CalendarHolder(c);
        this.fpt.copyJAXRPC3_16(calh);
        this.tmp = ((calh.value.get(Calendar.DAY_OF_MONTH) == c.get(Calendar.DAY_OF_MONTH)) &&
                (calh.value.get(Calendar.MONTH) == c.get(Calendar.MONTH)) &&
                (calh.value.get(Calendar.YEAR) == c.get(Calendar.YEAR)));
        assertTrue("copyJAXRPC3_16 error",this.tmp);
    }

    public void testCopyJAXRPC3_17() throws Exception {
        //Time time = new Time("12:24:52");
        //TimeHolder timeh = new TimeHolder(time);
        //fpt.copyJAXRPC3_17(timeh);
        //tmp = time.toString().equals(timeh.value.toString());
        Calendar c = new GregorianCalendar(TimeZone.getTimeZone("GMT"),Locale.FRANCE);
        CalendarHolder calh = new CalendarHolder(c);
        this.fpt.copyJAXRPC3_17(calh);
        this.tmp = ((calh.value.get(Calendar.HOUR_OF_DAY) == c.get(Calendar.HOUR_OF_DAY)) &&
                (calh.value.get(Calendar.MINUTE) == c.get(Calendar.MINUTE)) &&
                (calh.value.get(Calendar.SECOND) == c.get(Calendar.SECOND)));
        assertTrue("copyJAXRPC3_17 error",this.tmp);
    }

    public void _testCopyJAXRPC3_18() throws Exception {
        // URIHolder does not exist for java.net.URI
        //URI uri = new URI("http://www.google.com");
        //URIHolder urih = new URIHolder(uri);
        //fpt.copyJAXRPC3_18(urih);
        //tmp = uri.equals(urih.value);
        assertTrue("copyJAXRPC3_18 error",this.tmp);
    }

    public void testCopyJAXRPC3_19() throws Exception {
        byte[] tabByte = new byte[] {1,3};
        ByteArrayHolder tabByteh = new ByteArrayHolder(tabByte);
        this.fpt.copyJAXRPC3_19(tabByteh);
        this.tmp = (tabByte[0] == tabByteh.value[0]) && (tabByte[1] == tabByteh.value[1]);
        assertTrue("copyJAXRPC3_19 error",this.tmp);
    }

    public void testCopyJAXRPC3_20() throws Exception {
        byte[] tabByte21 = new byte[] {2,4};
        ByteArrayHolder tabByte2h = new ByteArrayHolder(tabByte21);
        this.fpt.copyJAXRPC3_20(tabByte2h);
        this.tmp = (tabByte21[0] == tabByte2h.value[0]) && (tabByte21[1] == tabByte2h.value[1]);
        assertTrue("copyJAXRPC3_20 error",this.tmp);
    }
}
