package org.objectweb.orchestra.tests.unit.unitPick;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.objectweb.orchestra.tests.unit.unitPick.PickPT;
import org.objectweb.orchestra.tests.unit.unitPick.PickServiceBP;
import org.objectweb.orchestra.tests.unit.unitPick.PickServiceBPLocator;


import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.objectweb.orchestra.util.BPELConstants;

public class TestPick extends TestCase {

    private boolean tmp;

    private PickServiceBP fs = null;
    private PickPT fpt = null;

    private String cas = new String("1");
    private int integer = 0;
    private String message;
    private String dur = new String ("PT2S");
    private Calendar cal = new GregorianCalendar ();

    public TestPick(String testname) {
        super(testname);
        this.cal.setTimeInMillis(System.currentTimeMillis()+5);
    }

    public static TestSuite suite() {
        return new TestSuite(TestPick.class);
    }

    public void setUp() throws Exception {
        System.out.println(BPELConstants.TEST_MESS_BEGIN+this.getName()+BPELConstants.TEST_MESS_END);
        this.tmp = false;
        this.message = null;
        this.fs = new PickServiceBPLocator();
        this.fpt = this.fs.getPickPT();
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args: <test-name>
        if (args.length > 0) {
            testtorun = args[0];
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new TestPick(testtorun));
        }
    }

    public void testPick1() throws Exception {
        //<onMessage portType="tns:PickPT" operation="pick1" variable="in1" partnerLink="client">
        this.message = this.fpt.pick1(this.cas,this.integer);
        this.tmp = this.message.equals("pick1");
        assertTrue("pick1 error",this.tmp);
    }

    public void testPick2() throws Exception {
        //<onMessage portType="tns:PickPT" operation="pick2" variable="in1" partnerLink="client">
        this.message = this.fpt.pick2(this.cas,this.integer);
        this.tmp = this.message.equals("pick2");
        assertTrue("pick2 error",this.tmp);
    }

    public void testPick3() throws Exception {
        //<onMessage portType="tns:PickPT" operation="pick3" variable="in1" partnerLink="client">
        this.message = this.fpt.pick3(this.cas,this.integer);
        this.tmp = this.message.equals("pick3");
        assertTrue("pick3 error",this.tmp);
    }

    public void testPick4() throws Exception {
        //<onAlarm for="'PT5S'">
        this.message = this.fpt.pick4(this.cas,this.integer);
        this.tmp = this.message.equals("Alarm pick4");
        assertTrue("pick4 error",this.tmp);
    }

    public void testPick5() throws Exception {
        //<onAlarm for="getVariableData('in2','duration')">
        this.message = this.fpt.pick5(this.dur);
        this.tmp = this.message.equals("Alarm pick5");
        assertTrue("pick5 error",this.tmp);
    }

    public void testPick6() throws Exception {
        //<onAlarm for="getVariableProperty('in2','duration')">
        this.message = this.fpt.pick6(this.dur);
        this.tmp = this.message.equals("Alarm pick6");
        assertTrue("pick6 error",this.tmp);
    }

    public void testPick7() throws Exception {
        //<onAlarm until="2010-01-01T00:00:00">
        //<onAlarm until="getVariableData('in3','dateTime')">
        this.message = this.fpt.pick7(this.cal);
        this.tmp = this.message.equals("Alarm pick7");
        assertTrue("pick7 error",this.tmp);
    }

    public void testPick8() throws Exception {
        //<onAlarm until="getVariableProperty('in3','dateTime')">
        this.message = this.fpt.pick8(this.cal);
        this.tmp = this.message.equals("Alarm pick8");
        assertTrue("pick8 error",this.tmp);
    }

    public void testPick9() throws Exception {
        //<onAlarm until="getVariableData('in4','time')">
        //message = fpt.pick9(tim);
        this.cal.setTime(new Date());
        this.cal.add(Calendar.SECOND,7);
        this.message = this.fpt.pick9(this.cal);
        this.tmp = this.message.equals("Alarm pick9");
        assertTrue("pick9 error",this.tmp);
    }

    public void testPick10() throws Exception {
        //<onAlarm until="getVariableProperty('in4','time')">
        //message = fpt.pick10(tim);
        this.cal.setTime(new Date());
        this.cal.add(Calendar.SECOND,7);
        this.message = this.fpt.pick10(this.cal);
        this.tmp = this.message.equals("Alarm pick10");
        assertTrue("pick10 error",this.tmp);
    }

    public void testPick11() throws Exception {
        //<onAlarm until="getVariableData('in5','date')">
        //message = fpt.pick11(dat);
        this.cal.setTime(new Date (System.currentTimeMillis()+5));
        this.message = this.fpt.pick11(this.cal);
        this.tmp = this.message.equals("Alarm pick11");
        assertTrue("pick11 error",this.tmp);
    }

    public void testPick12() throws Exception {
        //<onAlarm until="getVariableProperty('in5','date')">
        //message = fpt.pick12(dat);
        this.cal.setTime(new Date (System.currentTimeMillis()+5));
        this.message = this.fpt.pick12(this.cal);
        this.tmp = this.message.equals("Alarm pick12");
        assertTrue("pick12 error",this.tmp);
    }

}
